@@ -17,7 +17,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 *******************************************************
 
@@ -35,8 +35,8 @@ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
-Street, Fifth Floor, Boston, MA 02110-1335 USA
+this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02111-1301 USA
 
 *******************************************************/
 
@@ -57,9 +57,6 @@ Street, Fifth Floor, Boston, MA 02110-1335 USA
 #include "backup_copy.h"
 #include "backup_mysql.h"
 #include <btr0btr.h>
-#ifdef _WIN32
-#include <direct.h> /* rmdir */
-#endif
 
 #define ROCKSDB_BACKUP_DIR "#rocksdb"
 
@@ -198,8 +195,6 @@ datadir_iter_new(const char *path, bool skip_first_level = true)
 	datadir_iter_t *it;
 
 	it = static_cast<datadir_iter_t *>(malloc(sizeof(datadir_iter_t)));
-	if (!it)
-		goto error;
 	memset(it, 0, sizeof(datadir_iter_t));
 
 	pthread_mutex_init(&it->mutex, NULL);
@@ -241,7 +236,7 @@ datadir_iter_next_database(datadir_iter_t *it)
 		if (os_file_closedir(it->dbdir) != 0) {
 
 			msg("Warning: could not"
-			      " close database directory %s", it->dbpath);
+			      " close database directory %s\n", it->dbpath);
 
 			it->err = DB_ERROR;
 
@@ -282,7 +277,7 @@ datadir_iter_next_database(datadir_iter_t *it)
 		}
 
 		if (check_if_skip_database_by_path(it->dbpath)) {
-			msg("Skipping db: %s", it->dbpath);
+			msg("Skipping db: %s\n", it->dbpath);
 			continue;
 		}
 
@@ -483,7 +478,7 @@ struct datafile_cur_t {
 	{
 		memset(rel_path, 0, sizeof rel_path);
 		if (filename) {
-			strncpy(abs_path, filename, sizeof abs_path - 1);
+			strncpy(abs_path, filename, sizeof abs_path);
 			abs_path[(sizeof abs_path) - 1] = 0;
 		} else {
 			abs_path[0] = '\0';
@@ -517,8 +512,7 @@ datafile_open(const char *file, datafile_cur_t *cursor, uint thread_n)
 	5.6+. We want to make "local" copies for the backup. */
 	strncpy(cursor->rel_path,
 		xb_get_relative_path(cursor->abs_path, FALSE),
-		(sizeof cursor->rel_path) - 1);
-	cursor->rel_path[(sizeof cursor->rel_path) - 1] = '\0';
+		sizeof(cursor->rel_path));
 
 	cursor->file = os_file_create_simple_no_error_handling(
 		0, cursor->abs_path,
@@ -527,15 +521,19 @@ datafile_open(const char *file, datafile_cur_t *cursor, uint thread_n)
 		/* The following call prints an error message */
 		os_file_get_last_error(TRUE);
 
-		msg(thread_n,"error: cannot open "
-			"file %s", cursor->abs_path);
+		msg("[%02u] error: cannot open "
+			"file %s\n",
+			thread_n, cursor->abs_path);
 
 		return(false);
 	}
 
 	if (!my_stat(cursor->abs_path, &cursor->statinfo, 0)) {
-		msg(thread_n, "error: cannot stat %s", cursor->abs_path);
+		msg("[%02u] error: cannot stat %s\n",
+			thread_n, cursor->abs_path);
+
 		datafile_close(cursor);
+
 		return(false);
 	}
 
@@ -563,9 +561,9 @@ datafile_read(datafile_cur_t *cursor)
 		return(XB_FIL_CUR_EOF);
 	}
 
-	if (os_file_read(IORequestRead,
+	if (!os_file_read(IORequestRead,
 			  cursor->file, cursor->buf, cursor->buf_offset,
-			  to_read) != DB_SUCCESS) {
+			  to_read)) {
 		return(XB_FIL_CUR_ERROR);
 	}
 
@@ -726,8 +724,9 @@ directory_exists(const char *dir, bool create)
 
 		if (mkdirp(dir, 0777, MYF(0)) < 0) {
 			my_strerror(errbuf, sizeof(errbuf), my_errno);
-			msg("Can not create directory %s: %s", dir, errbuf);
+			msg("Can not create directory %s: %s\n", dir, errbuf);
 			return(false);
+
 		}
 	}
 
@@ -736,7 +735,7 @@ directory_exists(const char *dir, bool create)
 
 	if (os_dir == NULL) {
 		my_strerror(errbuf, sizeof(errbuf), my_errno);
-		msg("Can not open directory %s: %s", dir,
+		msg("Can not open directory %s: %s\n", dir,
 			errbuf);
 
 		return(false);
@@ -765,7 +764,7 @@ directory_exists_and_empty(const char *dir, const char *comment)
 	os_dir = os_file_opendir(dir, FALSE);
 
 	if (os_dir == NULL) {
-		msg("%s can not open directory %s", comment, dir);
+		msg("%s can not open directory %s\n", comment, dir);
 		return(false);
 	}
 
@@ -774,7 +773,7 @@ directory_exists_and_empty(const char *dir, const char *comment)
 	os_file_closedir(os_dir);
 
 	if (!empty) {
-		msg("%s directory %s is not empty!", comment, dir);
+		msg("%s directory %s is not empty!\n", comment, dir);
 	}
 
 	return(empty);
@@ -823,7 +822,7 @@ datafile_copy_backup(const char *filepath, uint thread_n)
 	of the filters value. */
 
 	if (check_if_skip_table(filepath)) {
-		msg(thread_n,"Skipping %s.", filepath);
+		msg_ts("[%02u] Skipping %s.\n", thread_n, filepath);
 		return(true);
 	}
 
@@ -869,26 +868,34 @@ datafile_rsync_backup(const char *filepath, bool save_to_list, FILE *f)
 	return(true);
 }
 
-bool backup_file_print_buf(const char *filename, const char *buf, int buf_len)
+
+static
+bool
+backup_file_vprintf(const char *filename, const char *fmt, va_list ap)
 {
 	ds_file_t	*dstfile	= NULL;
 	MY_STAT		 stat;			/* unused for now */
+	char		*buf		= 0;
+	int		 buf_len;
 	const char	*action;
 
 	memset(&stat, 0, sizeof(stat));
 
+	buf_len = vasprintf(&buf, fmt, ap);
+
 	stat.st_size = buf_len;
 	stat.st_mtime = my_time(0);
 
 	dstfile = ds_open(ds_data, filename, &stat);
 	if (dstfile == NULL) {
-		msg("error: Can't open the destination stream for %s",
-			filename);
+		msg("[%02u] error: "
+			"cannot open the destination stream for %s\n",
+			0, filename);
 		goto error;
 	}
 
 	action = xb_get_copy_action("Writing");
-	msg("%s %s", action, filename);
+	msg_ts("[%02u] %s %s\n", 0, action, filename);
 
 	if (buf_len == -1) {
 		goto error;
@@ -899,7 +906,8 @@ bool backup_file_print_buf(const char *filename, const char *buf, int buf_len)
 	}
 
 	/* close */
-	msg("        ...done");
+	msg_ts("[%02u]        ...done\n", 0);
+	free(buf);
 
 	if (ds_close(dstfile)) {
 		goto error_close;
@@ -908,29 +916,17 @@ bool backup_file_print_buf(const char *filename, const char *buf, int buf_len)
 	return(true);
 
 error:
+	free(buf);
 	if (dstfile != NULL) {
 		ds_close(dstfile);
 	}
 
 error_close:
-	msg("Error: backup file failed.");
+	msg("[%02u] Error: backup file failed.\n", 0);
 	return(false); /*ERROR*/
-
-	return true;
-};
-
-static
-bool
-backup_file_vprintf(const char *filename, const char *fmt, va_list ap)
-{
-	char		*buf		= 0;
-	int		 buf_len;
-	buf_len = vasprintf(&buf, fmt, ap);
-	bool result = backup_file_print_buf(filename, buf, buf_len);
-	free(buf);
-	return result;
 }
 
+
 bool
 backup_file_printf(const char *filename, const char *fmt, ...)
 {
@@ -986,7 +982,7 @@ run_data_threads(datadir_iter_t *it, os_thread_func_t func, uint n)
 	for (i = 0; i < n; i++) {
 		ret = data_threads[i].ret && ret;
 		if (!data_threads[i].ret) {
-			msg("Error: thread %u failed.", i);
+			msg("Error: thread %u failed.\n", i);
 		}
 	}
 
@@ -995,65 +991,6 @@ run_data_threads(datadir_iter_t *it, os_thread_func_t func, uint n)
 	return(ret);
 }
 
-#ifdef _WIN32
-#include <windows.h>
-#include <accctrl.h>
-#include <aclapi.h>
-/*
-  On Windows, fix permission of the file after "copyback"
-  We assume that after copyback, mysqld will run as service as NetworkService
-  user, thus well give full permission on given file to that user.
-*/
-
-static int fix_win_file_permissions(const char *file)
-{
-	struct {
-		TOKEN_USER tokenUser;
-		BYTE buffer[SECURITY_MAX_SID_SIZE];
-	} tokenInfoBuffer;
-	HANDLE hFile = CreateFile(file, READ_CONTROL | WRITE_DAC, 0, NULL, OPEN_EXISTING,
-		FILE_FLAG_BACKUP_SEMANTICS, NULL);
-	if (hFile == INVALID_HANDLE_VALUE)
-		return -1;
-	ACL* pOldDACL;
-	SECURITY_DESCRIPTOR* pSD = NULL;
-	EXPLICIT_ACCESS ea = { 0 };
-	BOOL isWellKnownSID = FALSE;
-	PSID pSid = NULL;
-
-	GetSecurityInfo(hFile, SE_FILE_OBJECT, DACL_SECURITY_INFORMATION, NULL, NULL,
-		&pOldDACL, NULL, (void**)&pSD);
-	DWORD size = SECURITY_MAX_SID_SIZE;
-	pSid = (PSID)tokenInfoBuffer.buffer;
-	if (!CreateWellKnownSid(WinNetworkServiceSid, NULL, pSid,
-		&size))
-	{
-		return 1;
-	}
-	ea.Trustee.TrusteeForm = TRUSTEE_IS_SID;
-	ea.Trustee.ptstrName = (LPTSTR)pSid;
-
-	ea.grfAccessMode = GRANT_ACCESS;
-	ea.grfAccessPermissions = GENERIC_ALL;
-	ea.grfInheritance = CONTAINER_INHERIT_ACE | OBJECT_INHERIT_ACE;
-	ea.Trustee.TrusteeType = TRUSTEE_IS_UNKNOWN;
-	ACL* pNewDACL = 0;
-	DWORD err = SetEntriesInAcl(1, &ea, pOldDACL, &pNewDACL);
-	if (pNewDACL)
-	{
-		SetSecurityInfo(hFile, SE_FILE_OBJECT, DACL_SECURITY_INFORMATION, NULL, NULL,
-			pNewDACL, NULL);
-	}
-	if (pSD != NULL)
-		LocalFree((HLOCAL)pSD);
-	if (pNewDACL != NULL)
-		LocalFree((HLOCAL)pNewDACL);
-	CloseHandle(hFile);
-	return 0;
-}
-
-#endif
-
 
 /************************************************************************
 Copy file for backup/restore.
@@ -1068,7 +1005,6 @@ copy_file(ds_ctxt_t *datasink,
 	ds_file_t		*dstfile = NULL;
 	datafile_cur_t		 cursor;
 	xb_fil_cur_result_t	 res;
-	DBUG_ASSERT(datasink->datasink->remove);
 	const char	*dst_path =
 		(xtrabackup_copy_back || xtrabackup_move_back)?
 		dst_file_path : trim_dotslash(dst_file_path);
@@ -1081,12 +1017,14 @@ copy_file(ds_ctxt_t *datasink,
 
 	dstfile = ds_open(datasink, dst_path, &cursor.statinfo);
 	if (dstfile == NULL) {
-		msg(thread_n,"error: "
-			"cannot open the destination stream for %s", dst_name);
+		msg("[%02u] error: "
+			"cannot open the destination stream for %s\n",
+			thread_n, dst_name);
 		goto error;
 	}
 
-	msg(thread_n, "%s %s to %s", xb_get_copy_action(), src_file_path, dstfile->path);
+	msg_ts("[%02u] %s %s to %s\n",
+	       thread_n, xb_get_copy_action(), src_file_path, dstfile->path);
 
 	/* The main copy loop */
 	while ((res = datafile_read(&cursor)) == XB_FIL_CUR_SUCCESS) {
@@ -1094,7 +1032,6 @@ copy_file(ds_ctxt_t *datasink,
 		if (ds_write(dstfile, cursor.buf, cursor.buf_read)) {
 			goto error;
 		}
-		DBUG_EXECUTE_IF("copy_file_error", errno=ENOSPC;goto error;);
 	}
 
 	if (res == XB_FIL_CUR_ERROR) {
@@ -1102,12 +1039,8 @@ copy_file(ds_ctxt_t *datasink,
 	}
 
 	/* close */
-	msg(thread_n,"        ...done");
+	msg_ts("[%02u]        ...done\n", thread_n);
 	datafile_close(&cursor);
-#ifdef _WIN32
-	if (xtrabackup_copy_back || xtrabackup_move_back)
-		ut_a(!fix_win_file_permissions(dstfile->path));
-#endif
 	if (ds_close(dstfile)) {
 		goto error_close;
 	}
@@ -1116,12 +1049,11 @@ copy_file(ds_ctxt_t *datasink,
 error:
 	datafile_close(&cursor);
 	if (dstfile != NULL) {
-		datasink->datasink->remove(dstfile->path);
 		ds_close(dstfile);
 	}
 
 error_close:
-	msg(thread_n,"Error: copy_file() failed.");
+	msg("[%02u] Error: copy_file() failed.\n", thread_n);
 	return(false); /*ERROR*/
 }
 
@@ -1153,38 +1085,36 @@ move_file(ds_ctxt_t *datasink,
 
 	if (file_exists(dst_file_path_abs)) {
 		msg("Error: Move file %s to %s failed: Destination "
-			"file exists", src_file_path, dst_file_path_abs);
+			"file exists\n",
+			src_file_path, dst_file_path_abs);
 		return(false);
 	}
 
-	msg(thread_n,"Moving %s to %s", src_file_path, dst_file_path_abs);
+	msg_ts("[%02u] Moving %s to %s\n",
+	       thread_n, src_file_path, dst_file_path_abs);
 
 	if (my_rename(src_file_path, dst_file_path_abs, MYF(0)) != 0) {
 		if (my_errno == EXDEV) {
-			/* Fallback to copy/unlink */
-			if(!copy_file(datasink, src_file_path,
-					dst_file_path, thread_n))
-					return false;
-			msg(thread_n,"Removing %s", src_file_path);
+			bool ret;
+			ret = copy_file(datasink, src_file_path,
+					dst_file_path, thread_n);
+			msg_ts("[%02u] Removing %s\n", thread_n, src_file_path);
 			if (unlink(src_file_path) != 0) {
 				my_strerror(errbuf, sizeof(errbuf), errno);
-				msg("Warning: unlink %s failed: %s",
+				msg("Error: unlink %s failed: %s\n",
 					src_file_path,
 					errbuf);
 			}
-			return true;
+			return(ret);
 		}
 		my_strerror(errbuf, sizeof(errbuf), my_errno);
-		msg("Can not move file %s to %s: %s",
+		msg("Can not move file %s to %s: %s\n",
 			src_file_path, dst_file_path_abs,
 			errbuf);
 		return(false);
 	}
-#ifdef _WIN32
-	if (xtrabackup_copy_back || xtrabackup_move_back)
-		ut_a(!fix_win_file_permissions(dst_file_path_abs));
-#endif
-	msg(thread_n,"        ...done");
+
+	msg_ts("[%02u]        ...done\n", thread_n);
 
 	return(true);
 }
@@ -1289,7 +1219,6 @@ copy_or_move_file(const char *src_file_path,
 
 			if (!directory_exists(dst_dir, true)) {
 				ret = false;
-				free(link_filepath);
 				goto cleanup;
 			}
 
@@ -1337,13 +1266,13 @@ backup_files(const char *from, bool prep_mode)
 			prep_mode ? 1 : 2);
 		rsync_tmpfile = fopen(rsync_tmpfile_name, "w");
 		if (rsync_tmpfile == NULL) {
-			msg("Error: can't create file %s",
+			msg("Error: can't create file %s\n",
 				rsync_tmpfile_name);
 			return(false);
 		}
 	}
 
-	msg("Starting %s non-InnoDB tables and files",
+	msg_ts("Starting %s non-InnoDB tables and files\n",
 	       prep_mode ? "prep copy of" : "to backup");
 
 	datadir_node_init(&node);
@@ -1359,7 +1288,7 @@ backup_files(const char *from, bool prep_mode)
 				ret = datafile_copy_backup(node.filepath, 1);
 			}
 			if (!ret) {
-				msg("Failed to copy file %s", node.filepath);
+				msg("Failed to copy file %s\n", node.filepath);
 				goto out;
 			}
 		} else if (!prep_mode) {
@@ -1369,7 +1298,7 @@ backup_files(const char *from, bool prep_mode)
 				 "%s/db.opt", node.filepath);
 			if (!(ret = backup_file_printf(
 					trim_dotslash(path), "%s", ""))) {
-				msg("Failed to create file %s", path);
+				msg("Failed to create file %s\n", path);
 				goto out;
 			}
 		}
@@ -1394,13 +1323,13 @@ backup_files(const char *from, bool prep_mode)
 		cmd << "rsync -t . --files-from=" << rsync_tmpfile_name
 		    << " " << xtrabackup_target_dir;
 
-		msg("Starting rsync as: %s", cmd.str().c_str());
+		msg_ts("Starting rsync as: %s\n", cmd.str().c_str());
 		if ((err = system(cmd.str().c_str()) && !prep_mode) != 0) {
-			msg("Error: rsync failed with error code %d", err);
+			msg_ts("Error: rsync failed with error code %d\n", err);
 			ret = false;
 			goto out;
 		}
-		msg("rsync finished successfully.");
+		msg_ts("rsync finished successfully.\n");
 
 		if (!prep_mode && !opt_no_lock) {
 			char path[FN_REFLEN];
@@ -1416,7 +1345,7 @@ backup_files(const char *from, bool prep_mode)
 
 			rsync_tmpfile = fopen(rsync_tmpfile_name, "r");
 			if (rsync_tmpfile == NULL) {
-				msg("Error: can't open file %s",
+				msg("Error: can't open file %s\n",
 					rsync_tmpfile_name);
 				ret = false;
 				goto out;
@@ -1432,7 +1361,7 @@ backup_files(const char *from, bool prep_mode)
 					snprintf(dst_path, sizeof(dst_path),
 						"%s/%s", xtrabackup_target_dir,
 						path);
-					msg("Removing %s", dst_path);
+					msg_ts("Removing %s\n", dst_path);
 					unlink(dst_path);
 				}
 			}
@@ -1442,7 +1371,7 @@ backup_files(const char *from, bool prep_mode)
 		}
 	}
 
-	msg("Finished %s non-InnoDB tables and files",
+	msg_ts("Finished %s non-InnoDB tables and files\n",
 	       prep_mode ? "a prep copy of" : "backing up");
 
 out:
@@ -1456,9 +1385,9 @@ out:
 	return(ret);
 }
 
-void backup_fix_ddl(CorruptedPages &);
+void backup_fix_ddl(void);
 
-lsn_t get_current_lsn(MYSQL *connection)
+static lsn_t get_current_lsn(MYSQL *connection)
 {
 	static const char lsn_prefix[] = "\nLog sequence number ";
 	lsn_t lsn = 0;
@@ -1481,7 +1410,7 @@ lsn_t get_current_lsn(MYSQL *connection)
 lsn_t server_lsn_after_lock;
 extern void backup_wait_for_lsn(lsn_t lsn);
 /** Start --backup */
-bool backup_start(CorruptedPages &corrupted_pages)
+bool backup_start()
 {
 	if (!opt_no_lock) {
 		if (opt_safe_slave_backup) {
@@ -1514,9 +1443,9 @@ bool backup_start(CorruptedPages &corrupted_pages)
 		rocksdb_create_checkpoint();
 	}
 
-	msg("Waiting for log copy thread to read lsn %llu", (ulonglong)server_lsn_after_lock);
+	msg_ts("Waiting for log copy thread to read lsn %llu\n", (ulonglong)server_lsn_after_lock);
 	backup_wait_for_lsn(server_lsn_after_lock);
-	backup_fix_ddl(corrupted_pages);
+	backup_fix_ddl();
 
 	// There is no need to stop slave thread before coping non-Innodb data when
 	// --no-lock option is used because --no-lock option requires that no DDL or
@@ -1547,6 +1476,7 @@ bool backup_start(CorruptedPages &corrupted_pages)
 		if (!write_galera_info(mysql_connection)) {
 			return(false);
 		}
+		write_current_binlog_file(mysql_connection);
 	}
 
 	if (opt_binlog_info == BINLOG_INFO_ON) {
@@ -1556,7 +1486,7 @@ bool backup_start(CorruptedPages &corrupted_pages)
 	}
 
 	if (have_flush_engine_logs && !opt_no_lock) {
-		msg("Executing FLUSH NO_WRITE_TO_BINLOG ENGINE LOGS...");
+		msg_ts("Executing FLUSH NO_WRITE_TO_BINLOG ENGINE LOGS...\n");
 		xb_mysql_query(mysql_connection,
 			"FLUSH NO_WRITE_TO_BINLOG ENGINE LOGS", false);
 	}
@@ -1580,21 +1510,22 @@ void backup_release()
 	}
 
 	if (opt_safe_slave_backup && sql_thread_started) {
-		msg("Starting slave SQL thread");
+		msg("Starting slave SQL thread\n");
 		xb_mysql_query(mysql_connection,
 				"START SLAVE SQL_THREAD", false);
 	}
 }
 
-static const char *default_buffer_pool_file = "ib_buffer_pool";
-
 /** Finish after backup_start() and backup_release() */
 bool backup_finish()
 {
 	/* Copy buffer pool dump or LRU dump */
-	if (!opt_rsync && opt_galera_info) {
+	if (!opt_rsync) {
 		if (buffer_pool_filename && file_exists(buffer_pool_filename)) {
-			copy_file(ds_data, buffer_pool_filename, default_buffer_pool_file, 0);
+			const char *dst_name;
+
+			dst_name = trim_dotslash(buffer_pool_filename);
+			copy_file(ds_data, buffer_pool_filename, dst_name, 0);
 		}
 		if (file_exists("ib_lru_dump")) {
 			copy_file(ds_data, "ib_lru_dump", "ib_lru_dump", 0);
@@ -1605,12 +1536,12 @@ bool backup_finish()
 		rocksdb_backup_checkpoint();
 	}
 
-	msg("Backup created in directory '%s'", xtrabackup_target_dir);
+	msg_ts("Backup created in directory '%s'\n", xtrabackup_target_dir);
 	if (mysql_binlog_position != NULL) {
-		msg("MySQL binlog position: %s", mysql_binlog_position);
+		msg("MySQL binlog position: %s\n", mysql_binlog_position);
 	}
 	if (mysql_slave_position && opt_slave_info) {
-		msg("MySQL slave binlog position: %s",
+		msg("MySQL slave binlog position: %s\n",
 			mysql_slave_position);
 	}
 
@@ -1618,57 +1549,14 @@ bool backup_finish()
 		return(false);
 	}
 
-	if (!write_xtrabackup_info(mysql_connection, XTRABACKUP_INFO,
-				    opt_history != 0, true)) {
+	if (!write_xtrabackup_info(mysql_connection, XTRABACKUP_INFO, opt_history != 0)) {
 		return(false);
 	}
 
 	return(true);
 }
 
-
-/*
-  Drop all empty database directories in the base backup
-  that do not exists in the icremental backup.
-
-  This effectively re-plays all DROP DATABASE statements happened
-  in between base backup and incremental backup creation time.
-
-  Note, only checking if base_dir/db/ is empty is not enough,
-  because inc_dir/db/db.opt might have been dropped for some reasons,
-  which may also result into empty base_dir/db/.
-
-  Only the fact that at the same time:
-  - base_dir/db/ exists
-  - inc_dir/db/ does not exist
-  means that DROP DATABASE happened.
-*/
-static void
-ibx_incremental_drop_databases(const char *base_dir,
-                               const char *inc_dir)
-{
-	datadir_node_t node;
-	datadir_node_init(&node);
-	datadir_iter_t *it = datadir_iter_new(base_dir);
-
-	while (datadir_iter_next(it, &node)) {
-		if (node.is_empty_dir) {
-			char path[FN_REFLEN];
-			snprintf(path, sizeof(path), "%s/%s",
-				 inc_dir, node.filepath_rel);
-			if (!directory_exists(path, false)) {
-				msg("Removing %s", node.filepath);
-				rmdir(node.filepath);
-			}
-		}
-
-	}
-	datadir_iter_free(it);
-	datadir_node_free(&node);
-}
-
-
-static bool
+bool
 ibx_copy_incremental_over_full()
 {
 	const char *ext_list[] = {"frm", "isl", "MYD", "MYI", "MAD", "MAI",
@@ -1686,7 +1574,6 @@ ibx_copy_incremental_over_full()
 	char path[FN_REFLEN];
 	int i;
 
-        DBUG_ASSERT(!opt_galera_info);
 	datadir_node_init(&node);
 
 	/* If we were applying an incremental change set, we need to make
@@ -1714,14 +1601,27 @@ ibx_copy_incremental_over_full()
 
 			if (!(ret = copy_file(ds_data, node.filepath,
 						node.filepath_rel, 1))) {
-				msg("Failed to copy file %s",
+				msg("Failed to copy file %s\n",
 					node.filepath);
 				goto cleanup;
 			}
 		}
 
-		if (!(ret = backup_files_from_datadir(xtrabackup_incremental_dir)))
-			goto cleanup;
+		/* copy buffer pool dump */
+		if (innobase_buffer_pool_filename) {
+			const char *src_name;
+
+			src_name = trim_dotslash(innobase_buffer_pool_filename);
+
+			snprintf(path, sizeof(path), "%s/%s",
+				xtrabackup_incremental_dir,
+				src_name);
+
+			if (file_exists(path)) {
+				copy_file(ds_data, path,
+					  innobase_buffer_pool_filename, 0);
+			}
+		}
 
 		/* copy supplementary files */
 
@@ -1739,23 +1639,8 @@ ibx_copy_incremental_over_full()
 			}
 		}
 
-		if (directory_exists(ROCKSDB_BACKUP_DIR, false)) {
-			if (my_rmtree(ROCKSDB_BACKUP_DIR, MYF(0))) {
-				die("Can't remove " ROCKSDB_BACKUP_DIR);
-			}
-		}
-		snprintf(path, sizeof(path), "%s/" ROCKSDB_BACKUP_DIR, xtrabackup_incremental_dir);
-		if (directory_exists(path, false)) {
-			if (my_mkdir(ROCKSDB_BACKUP_DIR, 0777, MYF(0))) {
-				die("my_mkdir failed for " ROCKSDB_BACKUP_DIR);
-			}
-			copy_or_move_dir(path, ROCKSDB_BACKUP_DIR, true, true);
-		}
-		ibx_incremental_drop_databases(xtrabackup_target_dir,
-					       xtrabackup_incremental_dir);
 	}
 
-
 cleanup:
 	if (it != NULL) {
 		datadir_iter_free(it);
@@ -1823,7 +1708,7 @@ apply_log_finish()
 bool
 copy_back()
 {
-	bool ret = false;
+	bool ret;
 	datadir_iter_t *it = NULL;
 	datadir_node_t node;
 	char *dst_dir;
@@ -1856,7 +1741,7 @@ copy_back()
 	/* cd to backup directory */
 	if (my_setwd(xtrabackup_target_dir, MYF(MY_WME)))
 	{
-		msg("Can't my_setwd %s", xtrabackup_target_dir);
+		msg("cannot my_setwd %s\n", xtrabackup_target_dir);
 		return(false);
 	}
 
@@ -1869,7 +1754,7 @@ copy_back()
 	srv_sys_space.set_path(".");
 
 	if (!srv_sys_space.parse_params(innobase_data_file_path, true)) {
-		msg("syntax error in innodb_data_file_path");
+		msg("syntax error in innodb_data_file_path\n");
 		return(false);
 	}
 
@@ -1983,12 +1868,12 @@ copy_back()
 			snprintf(path, sizeof(path), "%s/%s",
 				mysql_data_home, node.filepath_rel);
 
-			msg("Creating directory %s", path);
+			msg_ts("[%02u] Creating directory %s\n", 1, path);
 
 			if (mkdirp(path, 0777, MYF(0)) < 0) {
 				char errbuf[MYSYS_STRERROR_SIZE];
 				my_strerror(errbuf, sizeof(errbuf), my_errno);
-				msg("Can not create directory %s: %s",
+				msg("Can not create directory %s: %s\n",
 					path, errbuf);
 				ret = false;
 
@@ -1996,7 +1881,7 @@ copy_back()
 
 			}
 
-			msg(" ...done.");
+			msg_ts("[%02u] ...done.", 1);
 
 			continue;
 		}
@@ -2018,11 +1903,6 @@ copy_back()
 			continue;
 		}
 
-                /* skip buffer pool dump */
-                if (!strcmp(filename, default_buffer_pool_file)) {
-                        continue;
-                }
-
 		/* skip innodb data files */
 		is_ibdata_file = false;
 		for (Tablespace::const_iterator iter(srv_sys_space.begin()),
@@ -2045,11 +1925,24 @@ copy_back()
 
 	/* copy buffer pool dump */
 
-        if (file_exists(default_buffer_pool_file) &&
-            innobase_buffer_pool_filename) {
-		copy_or_move_file(default_buffer_pool_file,
-				  innobase_buffer_pool_filename,
-				  mysql_data_home, 0);
+	if (innobase_buffer_pool_filename) {
+		const char *src_name;
+		char path[FN_REFLEN];
+
+		src_name = trim_dotslash(innobase_buffer_pool_filename);
+
+		snprintf(path, sizeof(path), "%s/%s",
+			mysql_data_home,
+			src_name);
+
+		/* could be already copied with other files
+		from data directory */
+		if (file_exists(src_name) &&
+			!file_exists(innobase_buffer_pool_filename)) {
+			copy_or_move_file(src_name,
+					  innobase_buffer_pool_filename,
+					  mysql_data_home, 0);
+		}
 	}
 
 	rocksdb_copy_back();
@@ -2098,14 +1991,14 @@ decrypt_decompress_file(const char *filepath, uint thread_n)
 
  	if (needs_action) {
 
-		msg(thread_n,"%s\n", message.str().c_str());
+		msg_ts("[%02u] %s\n", thread_n, message.str().c_str());
 
 	 	if (system(cmd.str().c_str()) != 0) {
 	 		return(false);
 	 	}
 
 		if (opt_remove_original) {
-			msg(thread_n, "Removing %s", filepath);
+			msg_ts("[%02u] removing %s\n", thread_n, filepath);
 			if (my_delete(filepath, MYF(MY_WME)) != 0) {
 				return(false);
 			}
@@ -2168,7 +2061,7 @@ decrypt_decompress()
 	/* cd to backup directory */
 	if (my_setwd(xtrabackup_target_dir, MYF(MY_WME)))
 	{
-		msg("Can't my_setwd %s", xtrabackup_target_dir);
+		msg("cannot my_setwd %s\n", xtrabackup_target_dir);
 		return(false);
 	}
 
@@ -2212,26 +2105,20 @@ static bool backup_files_from_datadir(const char *dir_path)
 		if (info.type != OS_FILE_TYPE_FILE)
 			continue;
 
-		const char *pname = strrchr(info.name, OS_PATH_SEPARATOR);
+		const char *pname = strrchr(info.name, IF_WIN('\\', '/'));
 		if (!pname)
 			pname = info.name;
 
-		if (!starts_with(pname, "aws-kms-key") &&
-			!starts_with(pname, "aria_log"))
-			/* For ES exchange the above line with the following code:
-			(!xtrabackup_prepare || !xtrabackup_incremental_dir ||
-				!starts_with(pname, "aria_log")))
-			*/
-			continue;
-
-		if (xtrabackup_prepare && xtrabackup_incremental_dir &&
-			file_exists(info.name))
-			unlink(info.name);
-
-		std::string full_path(dir_path);
-		full_path.append(1, OS_PATH_SEPARATOR).append(info.name);
-		if (!(ret = copy_file(ds_data, full_path.c_str() , info.name, 1)))
-			break;
+		/* Copy aria log files, and aws keys for encryption plugins.*/
+		const char *prefixes[] = { "aria_log", "aws-kms-key" };
+		for (size_t i = 0; i < array_elements(prefixes); i++) {
+			if (starts_with(pname, prefixes[i])) {
+				ret = copy_file(ds_data, info.name, info.name, 1);
+				if (!ret) {
+					break;
+				}
+			}
+		}
 	}
 	os_file_closedir(dir);
 	return ret;
@@ -2324,7 +2211,8 @@ static void copy_or_move_dir(const char *from, const char *to, bool do_copy, boo
 			rc = make_hardlink(from_path, to_path);
 			if (rc)
 			{
-				msg("Creating hardlink from %s to %s",from_path, to_path);
+				msg_ts("[%02u] Creating hardlink from %s to %s\n",
+					1, from_path, to_path);
 			}
 			else
 			{
@@ -2340,7 +2228,7 @@ static void copy_or_move_dir(const char *from, const char *to, bool do_copy, boo
 					to, 1));
 		}
 		if (!rc)
-			die("copy or move file failed");
+			exit(EXIT_FAILURE);
 	}
 	datadir_iter_free(it);
 	datadir_node_free(&node);
@@ -2358,14 +2246,14 @@ static void copy_or_move_dir(const char *from, const char *to, bool do_copy, boo
 */
 static void rocksdb_lock_checkpoint()
 {
-	msg("Obtaining rocksdb checkpoint lock.");
+	msg_ts("Obtaining rocksdb checkpoint lock.\n");
 	MYSQL_RES *res =
 		xb_mysql_query(mysql_connection, "SELECT GET_LOCK('mariabackup_rocksdb_checkpoint',3600)", true, true);
 
 	MYSQL_ROW r = mysql_fetch_row(res);
 	if (r && r[0] && strcmp(r[0], "1"))
 	{
-		msg("Could not obtain rocksdb checkpont lock.");
+		msg_ts("Could not obtain rocksdb checkpont lock\n");
 		exit(EXIT_FAILURE);
 	}
 	mysql_free_result(res);
@@ -2419,7 +2307,7 @@ static void rocksdb_create_checkpoint()
 
 	if (!access(rocksdb_checkpoint_dir, 0))
 	{
-		msg("Removing rocksdb checkpoint from previous backup attempt.");
+		msg_ts("Removing rocksdb checkpoint from previous backup attempt.\n");
 		rocksdb_remove_checkpoint_directory();
 	}
 
@@ -2435,14 +2323,15 @@ static void rocksdb_create_checkpoint()
 */
 static void rocksdb_backup_checkpoint()
 {
-	msg("Backing up rocksdb files.");
+	msg_ts("Backing up rocksdb files.\n");
 	char rocksdb_backup_dir[FN_REFLEN];
 	snprintf(rocksdb_backup_dir, sizeof(rocksdb_backup_dir), "%s/" ROCKSDB_BACKUP_DIR , xtrabackup_target_dir);
 	bool backup_to_directory = xtrabackup_backup && xtrabackup_stream_fmt == XB_STREAM_FMT_NONE;
 	if (backup_to_directory) 
 	{
 		if (my_mkdir(rocksdb_backup_dir, 0777, MYF(0))){
-			die("Can't create rocksdb backup directory %s", rocksdb_backup_dir);
+			msg_ts("Can't create rocksdb backup directory %s\n", rocksdb_backup_dir);
+			exit(EXIT_FAILURE);
 		}
 	}
 	copy_or_move_dir(rocksdb_checkpoint_dir, ROCKSDB_BACKUP_DIR, true, backup_to_directory);
@@ -2458,8 +2347,7 @@ static void rocksdb_copy_back() {
 		return;
 	char rocksdb_home_dir[FN_REFLEN];
         if (xb_rocksdb_datadir && is_abs_path(xb_rocksdb_datadir)) {
-		strncpy(rocksdb_home_dir, xb_rocksdb_datadir, sizeof rocksdb_home_dir - 1);
-		rocksdb_home_dir[sizeof rocksdb_home_dir - 1] = '\0';
+		strncpy(rocksdb_home_dir, xb_rocksdb_datadir, sizeof(rocksdb_home_dir));
 	} else {
 	   snprintf(rocksdb_home_dir, sizeof(rocksdb_home_dir), "%s/%s", mysql_data_home, 
 		xb_rocksdb_datadir?trim_dotslash(xb_rocksdb_datadir): ROCKSDB_BACKUP_DIR);
