@@ -16,7 +16,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 *******************************************************/
 
@@ -35,7 +35,6 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA
 #include "common.h"
 #include "read_filt.h"
 #include "xtrabackup.h"
-#include "backup_debug.h"
 
 /* Size of read buffer in pages (640 pages = 10M for 16K sized pages) */
 #define XB_FIL_CUR_PAGES 640
@@ -148,8 +147,7 @@ xb_fil_cur_open(
 
 	cursor->space_id = node->space->id;
 
-	strncpy(cursor->abs_path, node->name, (sizeof cursor->abs_path) - 1);
-	cursor->abs_path[(sizeof cursor->abs_path) - 1] = '\0';
+	strncpy(cursor->abs_path, node->name, sizeof(cursor->abs_path));
 
 	/* Get the relative path for the destination tablespace name, i.e. the
 	one that can be appended to the backup root directory. Non-system
@@ -157,8 +155,7 @@ xb_fil_cur_open(
 	We want to make "local" copies for the backup. */
 	strncpy(cursor->rel_path,
 		xb_get_relative_path(cursor->abs_path, cursor->is_system()),
-		(sizeof cursor->rel_path) - 1);
-	cursor->rel_path[(sizeof cursor->rel_path) - 1] = '\0';
+		sizeof(cursor->rel_path));
 
 	/* In the backup mode we should already have a tablespace handle created
 	by fil_ibd_load() unless it is a system
@@ -176,8 +173,9 @@ xb_fil_cur_open(
 			/* The following call prints an error message */
 			os_file_get_last_error(TRUE);
 
-			msg(thread_n, "mariabackup: error: cannot open "
-			    "tablespace %s", cursor->abs_path);
+			msg("[%02u] mariabackup: error: cannot open "
+			    "tablespace %s\n",
+			    thread_n, cursor->abs_path);
 
 			return(XB_FIL_CUR_SKIP);
 		}
@@ -218,8 +216,8 @@ xb_fil_cur_open(
 		cursor->statinfo.st_size = (ulonglong)max_file_size;
 	}
 	if (err) {
-		msg(thread_n, "mariabackup: error: cannot fstat %s",
-		    cursor->abs_path);
+		msg("[%02u] mariabackup: error: cannot fstat %s\n",
+		    thread_n, cursor->abs_path);
 
 		xb_fil_cur_close(cursor);
 
@@ -253,7 +251,7 @@ xb_fil_cur_open(
 	if (!node->space->crypt_data
 	    && os_file_read(IORequestRead,
 			    node->handle, cursor->buf, 0,
-			    page_size.physical()) == DB_SUCCESS) {
+			    page_size.physical())) {
 		mutex_enter(&fil_system.mutex);
 		if (!node->space->crypt_data) {
 			node->space->crypt_data
@@ -263,6 +261,8 @@ xb_fil_cur_open(
 		mutex_exit(&fil_system.mutex);
 	}
 
+	/*msg("crypt: %s,%u\n", node->name, node->space->crypt_data->type);*/
+
 	cursor->space_size = (ulint)(cursor->statinfo.st_size
 				     / page_size.physical());
 
@@ -324,7 +324,7 @@ static bool page_is_corrupted(const byte *page, ulint page_no,
 	page_no first. */
 	if (page_no
 	    && mach_read_from_4(page + FIL_PAGE_FILE_FLUSH_LSN_OR_KEY_VERSION)
-	    && (opt_encrypted_backup
+	    && (opt_backup_encrypted
 		|| (space->crypt_data
 		    && space->crypt_data->type != CRYPT_SCHEME_UNENCRYPTED))) {
 
@@ -339,9 +339,11 @@ static bool page_is_corrupted(const byte *page, ulint page_no,
 
 		memcpy(tmp_page, page, page_size);
 
+		bool decrypted = false;
 		if (!space->crypt_data
 		    || space->crypt_data->type == CRYPT_SCHEME_UNENCRYPTED
-		    || !fil_space_decrypt(space, tmp_frame, tmp_page)) {
+		    || !fil_space_decrypt(space, tmp_frame, tmp_page,
+					  &decrypted)) {
 			return true;
 		}
 
@@ -372,15 +374,16 @@ static bool page_is_corrupted(const byte *page, ulint page_no,
 	return buf_page_is_corrupted(true, page, cursor->page_size, space);
 }
 
-/** Reads and verifies the next block of pages from the source
+/************************************************************************
+Reads and verifies the next block of pages from the source
 file. Positions the cursor after the last read non-corrupted page.
-@param[in,out] cursor source file cursor
-@param[out] corrupted_pages adds corrupted pages if
-opt_log_innodb_page_corruption is set
+
 @return XB_FIL_CUR_SUCCESS if some have been read successfully, XB_FIL_CUR_EOF
 if there are no more pages to read and XB_FIL_CUR_ERROR on error. */
-xb_fil_cur_result_t xb_fil_cur_read(xb_fil_cur_t*	cursor,
-                                    CorruptedPages &corrupted_pages)
+xb_fil_cur_result_t
+xb_fil_cur_read(
+/*============*/
+	xb_fil_cur_t*	cursor)	/*!< in/out: source file cursor */
 {
 	byte*			page;
 	ulint			i;
@@ -409,8 +412,14 @@ xb_fil_cur_result_t xb_fil_cur_read(xb_fil_cur_t*	cursor,
 	    && offset + to_read == cursor->statinfo.st_size) {
 
 		if (to_read < (ib_int64_t) page_size) {
-			msg(cursor->thread_n, "Warning: junk at the end of "
-			    "%s, offset = %llu, to_read = %llu",cursor->abs_path, (ulonglong) offset, (ulonglong) to_read);
+			msg("[%02u] mariabackup: Warning: junk at the end of "
+			    "%s:\n", cursor->thread_n, cursor->abs_path);
+			msg("[%02u] mariabackup: Warning: offset = %llu, "
+			    "to_read = %llu\n",
+			    cursor->thread_n,
+			    (unsigned long long) offset,
+			    (unsigned long long) to_read);
+
 			return(XB_FIL_CUR_EOF);
 		}
 
@@ -439,8 +448,8 @@ read_retry:
 	cursor->buf_offset = offset;
 	cursor->buf_page_no = (ulint)(offset / page_size);
 
-	if (os_file_read(IORequestRead, cursor->file, cursor->buf, offset,
-			  (ulint) to_read) != DB_SUCCESS) {
+	if (!os_file_read(IORequestRead, cursor->file, cursor->buf, offset,
+			  (ulint) to_read)) {
 		ret = XB_FIL_CUR_ERROR;
 		goto func_exit;
 	}
@@ -454,40 +463,23 @@ read_retry:
 			retry_count--;
 
 			if (retry_count == 0) {
-				const char *ignore_corruption_warn = opt_log_innodb_page_corruption ?
-					" WARNING!!! The corruption is ignored due to"
-					" log-innodb-page-corruption option, the backup can contain"
-					" corrupted data." : "";
-				msg(cursor->thread_n,
+				msg("[%02u] mariabackup: "
 				    "Error: failed to read page after "
 				    "10 retries. File %s seems to be "
-				    "corrupted.%s", cursor->abs_path, ignore_corruption_warn);
+				    "corrupted.\n", cursor->thread_n,
+				    cursor->abs_path);
+				ret = XB_FIL_CUR_ERROR;
 				buf_page_print(page, cursor->page_size);
-				if (opt_log_innodb_page_corruption) {
-					corrupted_pages.add_page(cursor->node->name, cursor->node->space->id,
-						page_no);
-					retry_count = 1;
-				}
-				else {
-					ret = XB_FIL_CUR_ERROR;
-					break;
-				}
-			}
-			else {
-				msg(cursor->thread_n, "Database page corruption detected at page "
-				    ULINTPF ", retrying...",
-				    page_no);
-				os_thread_sleep(100000);
-				goto read_retry;
+				break;
 			}
+			msg("[%02u] mariabackup: "
+			    "Database page corruption detected at page "
+			    ULINTPF ", retrying...\n", cursor->thread_n,
+			    page_no);
+
+			os_thread_sleep(100000);
+			goto read_retry;
 		}
-		DBUG_EXECUTE_FOR_KEY("add_corrupted_page_for", cursor->node->space->name,
-			{
-				ulint corrupted_page_no = strtoul(dbug_val, NULL, 10);
-				if (page_no == corrupted_page_no)
-					corrupted_pages.add_page(cursor->node->name, cursor->node->space->id,
-						corrupted_page_no);
-			});
 		cursor->buf_read += page_size;
 		cursor->buf_npages++;
 	}
