@@ -14,7 +14,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 *******************************************************/
 
@@ -71,7 +71,7 @@ extern char		*xtrabackup_databases_file;
 extern char		*xtrabackup_tables_exclude;
 extern char		*xtrabackup_databases_exclude;
 
-extern uint		xtrabackup_compress;
+extern ibool		xtrabackup_compress;
 
 extern my_bool		xtrabackup_backup;
 extern my_bool		xtrabackup_prepare;
@@ -86,10 +86,14 @@ extern int		xtrabackup_parallel;
 
 extern my_bool		xb_close_files;
 extern const char	*xtrabackup_compress_alg;
-
-extern uint		xtrabackup_compress_threads;
-extern ulonglong	xtrabackup_compress_chunk_size;
-
+#ifdef __cplusplus
+extern "C"{
+#endif
+  extern uint		xtrabackup_compress_threads;
+  extern ulonglong	xtrabackup_compress_chunk_size;
+#ifdef __cplusplus
+}
+#endif
 extern my_bool		xtrabackup_export;
 extern char		*xtrabackup_extra_lsndir;
 extern ulint		xtrabackup_log_copy_interval;
@@ -108,7 +112,7 @@ extern my_bool		opt_no_backup_locks;
 extern my_bool		opt_decompress;
 extern my_bool		opt_remove_original;
 extern my_bool		opt_extended_validation;
-extern my_bool		opt_encrypted_backup;
+extern my_bool		opt_backup_encrypted;
 extern my_bool		opt_lock_ddl_per_table;
 
 extern char		*opt_incremental_history_name;
@@ -140,7 +144,7 @@ extern uint		opt_safe_slave_backup_timeout;
 
 extern const char	*opt_history;
 
-enum binlog_info_enum { BINLOG_INFO_OFF, BINLOG_INFO_ON,
+enum binlog_info_enum { BINLOG_INFO_OFF, BINLOG_INFO_LOCKLESS, BINLOG_INFO_ON,
 			BINLOG_INFO_AUTO};
 
 extern ulong opt_binlog_info;
@@ -194,14 +198,4 @@ void mdl_lock_init();
 void mdl_lock_table(ulint space_id);
 void mdl_unlock_all();
 bool ends_with(const char *str, const char *suffix);
-
-typedef void (*insert_entry_func_t)(const char*);
-
-/* Scan string and load filter entries from it.
-@param[in] list string representing a list
-@param[in] delimiters delimiters of entries
-@param[in] ins callback to add entry */
-void xb_load_list_string(char *list, const char *delimiters,
-                         insert_entry_func_t ins);
-void register_ignore_db_dirs_filter(const char *name);
 #endif /* XB_XTRABACKUP_H */
