@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /* Some general useful functions */
 
@@ -40,7 +40,7 @@
 #include "ha_partition.h"
 
 
-partition_info *partition_info::get_clone(THD *thd, bool empty_data_and_index_file)
+partition_info *partition_info::get_clone(THD *thd)
 {
   MEM_ROOT *mem_root= thd->mem_root;
   DBUG_ENTER("partition_info::get_clone");
@@ -60,22 +60,21 @@ partition_info *partition_info::get_clone(THD *thd, bool empty_data_and_index_fi
   {
     List_iterator<partition_element> subpart_it(part->subpartitions);
     partition_element *subpart;
-    partition_element *part_clone= new (mem_root) partition_element(*part);
+    partition_element *part_clone= new (mem_root) partition_element();
     if (!part_clone)
       DBUG_RETURN(NULL);
+
+    memcpy(part_clone, part, sizeof(partition_element));
     part_clone->subpartitions.empty();
     while ((subpart= (subpart_it++)))
     {
-      partition_element *subpart_clone= new (mem_root) partition_element(*subpart);
+      partition_element *subpart_clone= new (mem_root) partition_element();
       if (!subpart_clone)
         DBUG_RETURN(NULL);
-      if (empty_data_and_index_file)
-        subpart_clone->data_file_name= subpart_clone->index_file_name= NULL;
+
+      memcpy(subpart_clone, subpart, sizeof(partition_element));
       part_clone->subpartitions.push_back(subpart_clone, mem_root);
     }
-
-    if (empty_data_and_index_file)
-      part_clone->data_file_name= part_clone->index_file_name= NULL;
     clone->partitions.push_back(part_clone, mem_root);
     part_clone->list_val_list.empty();
     List_iterator<part_elem_value> list_val_it(part->list_val_list);
@@ -452,8 +451,6 @@ bool partition_info::set_up_default_partitions(THD *thd, handler *file,
     const char *error_string;
     if (part_type == RANGE_PARTITION)
       error_string= "RANGE";
-    else if (part_type == VERSIONING_PARTITION)
-      error_string= "SYSTEM_TIME";
     else
       error_string= "LIST";
     my_error(ER_PARTITIONS_MUST_BE_DEFINED_ERROR, MYF(0), error_string);
@@ -833,13 +830,8 @@ bool partition_info::has_unique_name(partition_element *element)
   DBUG_RETURN(TRUE);
 }
 
-int partition_info::vers_set_hist_part(THD *thd)
+void partition_info::vers_set_hist_part(THD *thd)
 {
-  if (table->pos_in_table_list &&
-      table->pos_in_table_list->partition_names)
-  {
-    return HA_ERR_PARTITION_LIST;
-  }
   if (vers_info->limit)
   {
     ha_partition *hp= (ha_partition*)(table->file);
@@ -847,11 +839,9 @@ int partition_info::vers_set_hist_part(THD *thd)
     List_iterator<partition_element> it(partitions);
     while (next != vers_info->hist_part)
       next= it++;
-    DBUG_ASSERT(bitmap_is_set(&read_partitions, next->id));
     ha_rows records= hp->part_records(next);
     while ((next= it++) != vers_info->now_part)
     {
-      DBUG_ASSERT(bitmap_is_set(&read_partitions, next->id));
       ha_rows next_records= hp->part_records(next);
       if (next_records == 0)
         break;
@@ -864,13 +854,13 @@ int partition_info::vers_set_hist_part(THD *thd)
         goto warn;
       vers_info->hist_part= next;
     }
-    return 0;
+    return;
   }
 
   if (vers_info->interval.is_set())
   {
     if (vers_info->hist_part->range_value > thd->query_start())
-      return 0;
+      return;
 
     partition_element *next= NULL;
     List_iterator<partition_element> it(partitions);
@@ -881,15 +871,15 @@ int partition_info::vers_set_hist_part(THD *thd)
     {
       vers_info->hist_part= next;
       if (next->range_value > thd->query_start())
-        return 0;
+        return;
     }
+    goto warn;
   }
-  return 0;
+  return;
 warn:
   my_error(WARN_VERS_PART_FULL, MYF(ME_WARNING|ME_ERROR_LOG),
-           table->s->db.str, table->s->table_name.str,
+           table->s->db.str, table->s->error_table_name(),
            vers_info->hist_part->partition_name);
-  return 0;
 }
 
 
@@ -904,16 +894,15 @@ bool partition_info::vers_setup_expression(THD * thd, uint32 alter_add)
 
   DBUG_ASSERT(part_type == VERSIONING_PARTITION);
   DBUG_ASSERT(table->versioned(VERS_TIMESTAMP));
+  DBUG_ASSERT(num_columns == 1);
 
   if (!alter_add)
   {
     Field *row_end= table->vers_end_field();
+    part_field_list.push_back(row_end->field_name.str, thd->mem_root);
+    DBUG_ASSERT(part_field_list.elements == 1);
     // needed in handle_list_of_fields()
     row_end->flags|= GET_FIXED_FIELDS_FLAG;
-    Name_resolution_context *context= &thd->lex->current_select->context;
-    Item *row_end_item= new (thd->mem_root) Item_field(thd, context, row_end);
-    Item *row_end_ts= new (thd->mem_root) Item_func_unix_timestamp(thd, row_end_item);
-    set_part_expr(thd, row_end_ts, false);
   }
 
   if (alter_add)
@@ -922,12 +911,12 @@ bool partition_info::vers_setup_expression(THD * thd, uint32 alter_add)
     partition_element *el;
     for(uint32 id= 0; ((el= it++)); id++)
     {
-      DBUG_ASSERT(el->type != partition_element::CONVENTIONAL);
+      DBUG_ASSERT(el->type() != partition_element::CONVENTIONAL);
       /* Newly added element is inserted before AS_OF_NOW. */
-      if (el->id == UINT_MAX32 || el->type == partition_element::CURRENT)
+      if (el->id == UINT_MAX32 || el->type() == partition_element::CURRENT)
       {
         el->id= id;
-        if (el->type == partition_element::CURRENT)
+        if (el->type() == partition_element::CURRENT)
           break;
       }
     }
@@ -1354,13 +1343,13 @@ bool partition_info::check_partition_info(THD *thd, handlerton **eng_type,
       }
       if (part_type == VERSIONING_PARTITION)
       {
-        if (part_elem->type == partition_element::HISTORY)
+        if (part_elem->type() == partition_element::HISTORY)
         {
           hist_parts++;
         }
         else
         {
-          DBUG_ASSERT(part_elem->type == partition_element::CURRENT);
+          DBUG_ASSERT(part_elem->type() == partition_element::CURRENT);
           now_parts++;
         }
       }
@@ -1443,11 +1432,12 @@ void partition_info::print_no_partition_found(TABLE *table_arg, myf errflag)
   TABLE_LIST table_list;
   THD *thd= current_thd;
 
-  table_list.reset();
+  bzero(&table_list, sizeof(table_list));
   table_list.db= table_arg->s->db;
   table_list.table_name= table_arg->s->table_name;
 
-  if (check_single_table_access(thd, SELECT_ACL, &table_list, TRUE))
+  if (check_single_table_access(thd,
+                                SELECT_ACL, &table_list, TRUE))
   {
     my_message(ER_NO_PARTITION_FOR_GIVEN_VALUE,
                ER_THD(thd, ER_NO_PARTITION_FOR_GIVEN_VALUE_SILENT), errflag);
@@ -1458,13 +1448,13 @@ void partition_info::print_no_partition_found(TABLE *table_arg, myf errflag)
       buf_ptr= (char*)"from column_list";
     else
     {
-      MY_BITMAP *old_map= dbug_tmp_use_all_columns(table_arg, &table_arg->read_set);
+      my_bitmap_map *old_map= dbug_tmp_use_all_columns(table_arg, table_arg->read_set);
       if (part_expr->null_value)
         buf_ptr= (char*)"NULL";
       else
         longlong10_to_str(err_value, buf,
                      part_expr->unsigned_flag ? 10 : -10);
-      dbug_tmp_restore_column_map(&table_arg->read_set, old_map);
+      dbug_tmp_restore_column_map(table_arg->read_set, old_map);
     }
     my_error(ER_NO_PARTITION_FOR_GIVEN_VALUE, errflag, buf_ptr);
   }
@@ -1484,8 +1474,15 @@ void partition_info::print_no_partition_found(TABLE *table_arg, myf errflag)
     FALSE                     Success
 */
 
-bool partition_info::set_part_expr(THD *thd, Item *item_ptr, bool is_subpart)
+bool partition_info::set_part_expr(THD *thd, char *start_token, Item *item_ptr,
+                                   char *end_token, bool is_subpart)
 {
+  size_t expr_len= end_token - start_token;
+  char *func_string= (char*) thd->memdup(start_token, expr_len);
+
+  if (unlikely(!func_string))
+    return TRUE;
+
   if (is_subpart)
   {
     list_of_subpart_fields= FALSE;
@@ -2658,9 +2655,12 @@ part_column_list_val *partition_info::add_column_value(THD *thd)
   return NULL;
 }
 
-bool partition_info::set_part_expr(THD *thd, Item *item_ptr, bool is_subpart)
+bool partition_info::set_part_expr(THD *thd, char *start_token, Item *item_ptr,
+                                   char *end_token, bool is_subpart)
 {
+  (void)start_token;
   (void)item_ptr;
+  (void)end_token;
   (void)is_subpart;
   return FALSE;
 }
@@ -2698,8 +2698,9 @@ bool check_partition_dirs(partition_info *part_info)
 bool partition_info::vers_init_info(THD * thd)
 {
   part_type= VERSIONING_PARTITION;
-  list_of_part_fields= true;
-  column_list= false;
+  list_of_part_fields= TRUE;
+  column_list= TRUE;
+  num_columns= 1;
   vers_info= new (thd->mem_root) Vers_part_info;
   if (unlikely(!vers_info))
     return true;
