@@ -14,14 +14,14 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 
 #ifdef USE_PRAGMA_INTERFACE 
 #pragma interface			/* gcc class implementation */
 #endif
 
-#include "sql_list.h"                           /* Sql_alloc */
+#include "sql_alloc.h"                       /* Sql_alloc */
 #include "my_rnd.h"                          /* rand_struct */
 
 class SQL_CRYPT :public Sql_alloc
