@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
 
 
 /* create and drop of databases */
@@ -1089,12 +1089,8 @@ static bool find_db_tables_and_rm_known_files(THD *thd, MY_DIR *dirp,
     table_list->table_name= *table;
     table_list->open_type= OT_BASE_ONLY;
 
-    /*
-      On the case-insensitive file systems table is opened
-      with the lowercased file name. So we should lowercase
-      as well to look up the cache properly.
-    */
-    if (lower_case_file_system)
+    /* To be able to correctly look up the table in the table cache. */
+    if (lower_case_table_names)
       table_list->table_name.length= my_casedn_str(files_charset_info,
                                                    (char*) table_list->table_name.str);
 
@@ -1441,12 +1437,12 @@ static void backup_current_db_name(THD *thd,
   a stack pointer set by Stored Procedures was used by replication after
   the stack address was long gone.
 
-  @return error code (ER_XXX)
-    @retval 0 Success
-    @retval >0  Error
+  @return Operation status
+    @retval FALSE Success
+    @retval TRUE  Error
 */
 
-uint mysql_change_db(THD *thd, const LEX_CSTRING *new_db_name,
+bool mysql_change_db(THD *thd, const LEX_CSTRING *new_db_name,
                      bool force_switch)
 {
   LEX_CSTRING new_db_file_name;
@@ -1477,7 +1473,7 @@ uint mysql_change_db(THD *thd, const LEX_CSTRING *new_db_name,
     {
       my_message(ER_NO_DB_ERROR, ER_THD(thd, ER_NO_DB_ERROR), MYF(0));
 
-      DBUG_RETURN(ER_NO_DB_ERROR);
+      DBUG_RETURN(TRUE);
     }
   }
   DBUG_PRINT("enter",("name: '%s'", new_db_name->str));
@@ -1503,7 +1499,7 @@ uint mysql_change_db(THD *thd, const LEX_CSTRING *new_db_name,
   new_db_file_name.length= new_db_name->length;
 
   if (new_db_file_name.str == NULL)
-    DBUG_RETURN(ER_OUT_OF_RESOURCES);                             /* the error is set */
+    DBUG_RETURN(TRUE);                             /* the error is set */
 
   /*
     NOTE: if check_db_name() fails, we should throw an error in any case,
@@ -1523,7 +1519,7 @@ uint mysql_change_db(THD *thd, const LEX_CSTRING *new_db_name,
     if (force_switch)
       mysql_change_db_impl(thd, NULL, 0, thd->variables.collation_server);
 
-    DBUG_RETURN(ER_WRONG_DB_NAME);
+    DBUG_RETURN(TRUE);
   }
 
   DBUG_PRINT("info",("Use database: %s", new_db_file_name.str));
@@ -1553,7 +1549,7 @@ uint mysql_change_db(THD *thd, const LEX_CSTRING *new_db_name,
     general_log_print(thd, COM_INIT_DB, ER_THD(thd, ER_DBACCESS_DENIED_ERROR),
                       sctx->priv_user, sctx->priv_host, new_db_file_name.str);
     my_free(const_cast<char*>(new_db_file_name.str));
-    DBUG_RETURN(ER_DBACCESS_DENIED_ERROR);
+    DBUG_RETURN(TRUE);
   }
 #endif
 
@@ -1587,7 +1583,7 @@ uint mysql_change_db(THD *thd, const LEX_CSTRING *new_db_name,
 
       /* The operation failed. */
 
-      DBUG_RETURN(ER_BAD_DB_ERROR);
+      DBUG_RETURN(TRUE);
     }
   }
 
@@ -1603,7 +1599,7 @@ uint mysql_change_db(THD *thd, const LEX_CSTRING *new_db_name,
 done:
   SESSION_TRACKER_CHANGED(thd, CURRENT_SCHEMA_TRACKER, NULL);
   SESSION_TRACKER_CHANGED(thd, SESSION_STATE_CHANGE_TRACKER, NULL);
-  DBUG_RETURN(0);
+  DBUG_RETURN(FALSE);
 }
 
 
@@ -1851,7 +1847,7 @@ bool mysql_upgrade_db(THD *thd, const LEX_CSTRING *old_db)
 
   /* Step9: Let's do "use newdb" if we renamed the current database */
   if (change_to_newdb)
-    error|= mysql_change_db(thd, & new_db, FALSE) != 0;
+    error|= mysql_change_db(thd, & new_db, FALSE);
 
 exit:
   DBUG_RETURN(error);
