@@ -1,5 +1,5 @@
 /* Copyright (c) 2000, 2010, Oracle and/or its affiliates.
-   Copyright (c) 2009, 2020, MariaDB Corporation.
+   Copyright (c) 2009, 2013 Monty Program Ab.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,12 +12,13 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 
 /* Functions to handle date and time */
 
 #include "mariadb.h"
+#include "sql_priv.h"
 #include "sql_time.h"
 #include "tztime.h"                      // struct Time_zone
 #include "sql_class.h"                   // THD
@@ -68,71 +69,62 @@ int append_interval(String *str, interval_type int_type, const INTERVAL &interva
   size_t len;
   switch (int_type) {
   case INTERVAL_YEAR:
-    len= my_snprintf(buf,sizeof(buf),"%lu", interval.year);
+    len= my_snprintf(buf,sizeof(buf),"%u", interval.year);
     break;
   case INTERVAL_QUARTER:
   case INTERVAL_MONTH:
-    len= my_snprintf(buf,sizeof(buf),"%lu", interval.month);
+    len= my_snprintf(buf,sizeof(buf),"%u", interval.month);
     int_type=INTERVAL_MONTH;
     break;
   case INTERVAL_WEEK:
   case INTERVAL_DAY:
-    len= my_snprintf(buf,sizeof(buf),"%lu", interval.day);
+    len= my_snprintf(buf,sizeof(buf),"%u", interval.day);
     int_type=INTERVAL_DAY;
     break;
   case INTERVAL_HOUR:
-    len= my_snprintf(buf,sizeof(buf),"%lu", interval.hour);
+    len= my_snprintf(buf,sizeof(buf),"%u", interval.hour);
     break;
   case INTERVAL_MINUTE:
-    len= my_snprintf(buf,sizeof(buf),"%llu", interval.minute);
+    len= my_snprintf(buf,sizeof(buf),"%u", interval.minute);
     break;
   case INTERVAL_SECOND:
-    len= my_snprintf(buf,sizeof(buf),"%llu", interval.second);
+    len= my_snprintf(buf,sizeof(buf),"%u", interval.second);
     break;
   case INTERVAL_MICROSECOND:
-    len= my_snprintf(buf,sizeof(buf),"%llu", interval.second_part);
+    len= my_snprintf(buf,sizeof(buf),"%u", interval.second_part);
     break;
   case INTERVAL_YEAR_MONTH:
-    len= my_snprintf(buf,sizeof(buf),"'%lu-%02lu'",
-                     interval.year, interval.month);
+    len= my_snprintf(buf,sizeof(buf),"%u-%02u", interval.day, interval.month);
     break;
   case INTERVAL_DAY_HOUR:
-    len= my_snprintf(buf,sizeof(buf),"'%lu %lu'", interval.day, interval.hour);
+    len= my_snprintf(buf,sizeof(buf),"%u %u", interval.day, interval.hour);
     break;
   case INTERVAL_DAY_MINUTE:
-    len= my_snprintf(buf,sizeof(buf),"'%lu %lu:%02llu'",
-                     interval.day, interval.hour, interval.minute);
+    len= my_snprintf(buf,sizeof(buf),"%u %u:%02u", interval.day, interval.hour, interval.minute);
     break;
   case INTERVAL_DAY_SECOND:
-    len= my_snprintf(buf,sizeof(buf),"'%lu %lu:%02llu:%02llu'",
-                     interval.day, interval.hour, interval.minute, interval.second);
+    len= my_snprintf(buf,sizeof(buf),"%u %u:%02u:%02u", interval.day, interval.hour, interval.minute, interval.second);
     break;
   case INTERVAL_HOUR_MINUTE:
-    len= my_snprintf(buf,sizeof(buf),"'%lu:%02llu'", interval.hour, interval.minute);
+    len= my_snprintf(buf,sizeof(buf),"%u:%02u", interval.hour, interval.minute);
     break;
   case INTERVAL_HOUR_SECOND:
-    len= my_snprintf(buf,sizeof(buf),"'%lu:%02llu:%02llu'",
-                     interval.hour, interval.minute, interval.second);
+    len= my_snprintf(buf,sizeof(buf),"%u:%02u:%02u", interval.hour, interval.minute, interval.second);
     break;
   case INTERVAL_MINUTE_SECOND:
-    len= my_snprintf(buf,sizeof(buf),"'%llu:%02llu'", interval.minute, interval.second);
+    len= my_snprintf(buf,sizeof(buf),"%u:%02u", interval.minute, interval.second);
     break;
   case INTERVAL_DAY_MICROSECOND:
-    len= my_snprintf(buf,sizeof(buf),"'%lu %lu:%02llu:%02llu.%06llu'",
-                     interval.day, interval.hour, interval.minute,
-                     interval.second, interval.second_part);
+    len= my_snprintf(buf,sizeof(buf),"%u %u:%02u:%02u.%06u", interval.day, interval.hour, interval.minute, interval.second, interval.second_part);
     break;
   case INTERVAL_HOUR_MICROSECOND:
-    len= my_snprintf(buf,sizeof(buf),"'%lu:%02llu:%02llu.%06llu'",
-                     interval.hour, interval.minute, interval.second,
-                     interval.second_part);
+    len= my_snprintf(buf,sizeof(buf),"%u:%02u:%02u.%06u", interval.hour, interval.minute, interval.second, interval.second_part);
     break;
   case INTERVAL_MINUTE_MICROSECOND:
-    len= my_snprintf(buf,sizeof(buf),"'%llu:%02llu.%06llu'",
-                     interval.minute, interval.second, interval.second_part);
+    len= my_snprintf(buf,sizeof(buf),"%u:%02u.%06u", interval.minute, interval.second, interval.second_part);
     break;
   case INTERVAL_SECOND_MICROSECOND:
-    len= my_snprintf(buf,sizeof(buf),"%llu.%06llu", interval.second, interval.second_part);
+    len= my_snprintf(buf,sizeof(buf),"%u.%06u", interval.second, interval.second_part);
     break;
   default:
     DBUG_ASSERT(0);
@@ -305,7 +297,7 @@ check_date_with_warn(const MYSQL_TIME *ltime, ulonglong fuzzy_date,
   {
     ErrConvTime str(ltime);
     make_truncated_value_warning(current_thd, Sql_condition::WARN_LEVEL_WARN,
-                                 &str, ts_type, NULL, NULL, NULL);
+                                 &str, ts_type, 0, 0);
     return true;
   }
   return false;
@@ -322,7 +314,7 @@ adjust_time_range_with_warn(MYSQL_TIME *ltime, uint dec)
     return true;
   if (warnings)
     make_truncated_value_warning(current_thd, Sql_condition::WARN_LEVEL_WARN,
-                                 &str, MYSQL_TIMESTAMP_TIME, NULL, NULL, NULL);
+                                 &str, MYSQL_TIMESTAMP_TIME, 0, NullS);
   return false;
 }
 
@@ -411,8 +403,7 @@ str_to_datetime_with_warn(CHARSET_INFO *cs,
                                  ret_val ? Sql_condition::WARN_LEVEL_WARN :
                                  Sql_condition::time_warn_level(status.warnings),
                                  str, length, flags & TIME_TIME_ONLY ?
-                                 MYSQL_TIMESTAMP_TIME : l_time->time_type, 
-                                 NULL, NULL, NULL);
+                                 MYSQL_TIMESTAMP_TIME : l_time->time_type, 0, NullS);
   DBUG_EXECUTE_IF("str_to_datetime_warn",
                   push_warning(thd, Sql_condition::WARN_LEVEL_NOTE,
                                ER_YES, str););
@@ -441,7 +432,7 @@ static bool number_to_time_with_warn(bool neg, ulonglong nr, ulong sec_part,
   int was_cut;
   longlong res;
   enum_mysql_timestamp_type ts_type;
-  bool have_warnings= false;
+  bool have_warnings;
 
   if (fuzzydate & TIME_TIME_ONLY)
   {
@@ -469,8 +460,7 @@ static bool number_to_time_with_warn(bool neg, ulonglong nr, ulong sec_part,
     make_truncated_value_warning(current_thd,
                                  Sql_condition::WARN_LEVEL_WARN, str,
                                  res < 0 ? MYSQL_TIMESTAMP_ERROR : ts_type,
-                                 s ? s->db.str : NULL,
-                                 s ? s->table_name.str : NULL, field_name);
+                                 s, field_name);
   }
   return res < 0;
 }
@@ -486,7 +476,7 @@ bool double_to_datetime_with_warn(double value, MYSQL_TIME *ltime,
   if (neg)
     value= -value;
 
-  if (value > static_cast<double>(LONGLONG_MAX))
+  if (value > LONGLONG_MAX)
     value= static_cast<double>(LONGLONG_MAX);
 
   longlong nr= static_cast<ulonglong>(floor(value));
@@ -943,8 +933,7 @@ void make_truncated_value_warning(THD *thd,
                                   Sql_condition::enum_warning_level level,
                                   const ErrConv *sval,
 				  timestamp_type time_type,
-                                  const char *db_name, const char *table_name,
-                                  const char *field_name)
+                                  const TABLE_SHARE *s, const char *field_name)
 {
   char warn_buff[MYSQL_ERRMSG_SIZE];
   const char *type_str;
@@ -964,6 +953,9 @@ void make_truncated_value_warning(THD *thd,
   }
   if (field_name)
   {
+    const char *db_name= s->db.str;
+    const char *table_name= s->error_table_name();
+
     if (!db_name)
       db_name= "";
     if (!table_name)
@@ -1301,7 +1293,7 @@ make_date_with_warn(MYSQL_TIME *ltime, ulonglong fuzzy_date,
     /* e.g. negative time */
     ErrConvTime str(ltime);
     make_truncated_value_warning(current_thd, Sql_condition::WARN_LEVEL_WARN,
-                                 &str, ts_type, NULL, NULL, NULL);
+                                 &str, ts_type, 0, 0);
     return true;
   }
   if ((ltime->time_type= ts_type) == MYSQL_TIMESTAMP_DATE)
@@ -1465,7 +1457,7 @@ time_to_datetime_with_warn(THD *thd,
   {
     ErrConvTime str(from);
     make_truncated_value_warning(thd, Sql_condition::WARN_LEVEL_WARN,
-                                 &str, MYSQL_TIMESTAMP_DATETIME, NULL, NULL, NULL); 
+                                 &str, MYSQL_TIMESTAMP_DATETIME, 0, 0); 
     return true;
   }
   return false;
