@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
 
 
 #define MYSQL_LEX 1
@@ -34,11 +34,6 @@
 #include "sql_handler.h"                        // mysql_ha_rm_tables
 #include "sp_cache.h"                     // sp_invalidate_cache
 #include <mysys_err.h>
-#ifdef WITH_WSREP
-#include "debug_sync.h"
-#endif /* WITH_WSREP */
-#include <my_time.h>
-#include <mysql_time.h>
 
 LEX_CSTRING *make_lex_string(LEX_CSTRING *lex_str,
                              const char* str, size_t length,
@@ -214,7 +209,7 @@ static File_option triggers_file_parameters[]=
   },
   {
     { STRING_WITH_LEN("created") },
-    my_offsetof(class Table_triggers_list, hr_create_times),
+    my_offsetof(class Table_triggers_list, create_times),
     FILE_OPTIONS_ULLLIST
   },
   { { 0, 0 }, 0, FILE_OPTIONS_STRING }
@@ -343,7 +338,7 @@ public:
 
 Trigger::~Trigger()
 {
-  sp_head::destroy(body);
+  delete body;
 }
 
 
@@ -523,7 +518,7 @@ bool mysql_create_or_drop_trigger(THD *thd, TABLE_LIST *tables, bool create)
 
 #ifdef WITH_WSREP
   if (thd->wsrep_exec_mode == LOCAL_STATE)
-    WSREP_TO_ISOLATION_BEGIN(WSREP_MYSQL_DB, NULL, tables);
+    WSREP_TO_ISOLATION_BEGIN(WSREP_MYSQL_DB, NULL, NULL);
 #endif
 
   /* We should have only one table in table list. */
@@ -585,16 +580,6 @@ bool mysql_create_or_drop_trigger(THD *thd, TABLE_LIST *tables, bool create)
       goto end;
   }
 
-#ifdef WITH_WSREP
-  DBUG_EXECUTE_IF("sync.mdev_20225",
-                  {
-                    const char act[]=
-                      "now "
-                      "wait_for signal.mdev_20225_continue";
-                    DBUG_ASSERT(!debug_sync_set_action(thd,
-                                                       STRING_WITH_LEN(act)));
-                  };);
-#endif /* WITH_WSREP */
   result= (create ?
            table->triggers->create_trigger(thd, tables, &stmt_query):
            table->triggers->drop_trigger(thd, tables, &stmt_query));
@@ -898,10 +883,6 @@ bool Table_triggers_list::create_trigger(THD *thd, TABLE_LIST *tables,
   if (!(trigger= new (&table->mem_root) Trigger(this, 0)))
     goto err_without_cleanup;
 
-  /* Time with in microseconds */
-  trigger->hr_create_time= make_hr_time(thd->query_start(),
-                                        thd->query_start_sec_part());
-
   /* Create trigger_name.TRN file to ensure trigger name is unique */
   if (sql_create_definition_file(NULL, &trigname_file, &trigname_file_type,
                                  (uchar*)&trigname, trigname_file_parameters))
@@ -910,6 +891,8 @@ bool Table_triggers_list::create_trigger(THD *thd, TABLE_LIST *tables,
   /* Populate the trigger object */
 
   trigger->sql_mode= thd->variables.sql_mode;
+  /* Time with 2 decimals, like in MySQL 5.7 */
+  trigger->create_time= ((ulonglong) thd->query_start())*100 + thd->query_start_sec_part()/10000;
   build_trig_stmt_query(thd, tables, stmt_query, &trigger_definition,
                         &trigger->definer, trg_definer_holder);
 
@@ -977,7 +960,7 @@ void Table_triggers_list::empty_lists()
   client_cs_names.empty();
   connection_cl_names.empty();
   db_cl_names.empty();
-  hr_create_times.empty();
+  create_times.empty();
 }
 
 
@@ -1013,7 +996,7 @@ bool Trigger::add_to_file_list(void* param_arg)
       base->client_cs_names.push_back(&client_cs_name, mem_root) ||
       base->connection_cl_names.push_back(&connection_cl_name, mem_root) ||
       base->db_cl_names.push_back(&db_cl_name, mem_root) ||
-      base->hr_create_times.push_back(&hr_create_time.val, mem_root))
+      base->create_times.push_back(&create_time, mem_root))
     return 1;
   return 0;
 }
@@ -1395,8 +1378,7 @@ bool Table_triggers_list::check_n_load(THD *thd, const LEX_CSTRING *db,
       List_iterator_fast<LEX_CSTRING> it_client_cs_name(trigger_list->client_cs_names);
       List_iterator_fast<LEX_CSTRING> it_connection_cl_name(trigger_list->connection_cl_names);
       List_iterator_fast<LEX_CSTRING> it_db_cl_name(trigger_list->db_cl_names);
-      List_iterator_fast<ulonglong>
-        it_create_times(trigger_list->hr_create_times);
+      List_iterator_fast<ulonglong> it_create_times(trigger_list->create_times);
       LEX *old_lex= thd->lex;
       LEX lex;
       sp_rcontext *save_spcont= thd->spcont;
@@ -1482,13 +1464,7 @@ bool Table_triggers_list::check_n_load(THD *thd, const LEX_CSTRING *db,
 
         trigger->sql_mode= sql_mode;
         trigger->definition= *trg_create_str;
-        trigger->hr_create_time.val= trg_create_time ? *trg_create_time : 0;
-        /*
-          Fix time if in 100th of second (comparison with max uint * 100
-          (max possible timestamp in the old format))
-        */
-        if (trigger->hr_create_time.val < 429496729400ULL)
-          trigger->hr_create_time.val*= 10000;
+        trigger->create_time= trg_create_time ? *trg_create_time : 0;
         trigger->name= sp ? sp->m_name : empty_clex_str;
         trigger->on_table_name.str= (char*) lex.raw_trg_on_table_name_begin;
         trigger->on_table_name.length= (lex.raw_trg_on_table_name_end -
@@ -1830,7 +1806,7 @@ bool Table_triggers_list::drop_all_triggers(THD *thd, const LEX_CSTRING *db,
   bool result= 0;
   DBUG_ENTER("Triggers::drop_all_triggers");
 
-  table.reset();
+  bzero(&table, sizeof(table));
   init_sql_alloc(&table.mem_root, "Triggers::drop_all_triggers", 8192, 0,
                  MYF(0));
 
@@ -2082,7 +2058,7 @@ bool Table_triggers_list::change_table_name(THD *thd, const LEX_CSTRING *db,
   Trigger *err_trigger;
   DBUG_ENTER("Triggers::change_table_name");
 
-  table.reset();
+  bzero(&table, sizeof(table));
   init_sql_alloc(&table.mem_root, "Triggers::change_table_name", 8192, 0,
                  MYF(0));
 
@@ -2216,7 +2192,8 @@ bool Table_triggers_list::process_triggers(THD *thd,
     This trigger must have been processed by the pre-locking
     algorithm.
   */
-  DBUG_ASSERT(trigger_table->pos_in_table_list->trg_event_map & trg2bit(event));
+  DBUG_ASSERT(trigger_table->pos_in_table_list->trg_event_map &
+              static_cast<uint>(1 << static_cast<int>(event)));
 
   thd->reset_sub_statement_state(&statement_state, SUB_STMT_TRIGGER);
 
@@ -2268,7 +2245,8 @@ add_tables_and_routines_for_triggers(THD *thd,
 
   for (int i= 0; i < (int)TRG_EVENT_MAX; i++)
   {
-    if (table_list->trg_event_map & trg2bit(static_cast<trg_event_type>(i)))
+    if (table_list->trg_event_map &
+        static_cast<uint8>(1 << static_cast<int>(i)))
     {
       for (int j= 0; j < (int)TRG_ACTION_MAX; j++)
       {
