@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 
 /*
@@ -52,9 +52,8 @@
    compare_record(TABLE*).
  */
 bool records_are_comparable(const TABLE *table) {
-  return !table->versioned(VERS_TRX_ID) &&
-          (((table->file->ha_table_flags() & HA_PARTIAL_COLUMN_READ) == 0) ||
-           bitmap_is_subset(table->write_set, table->read_set));
+  return ((table->file->ha_table_flags() & HA_PARTIAL_COLUMN_READ) == 0) ||
+    bitmap_is_subset(table->write_set, table->read_set);
 }
 
 
@@ -70,20 +69,17 @@ bool compare_record(const TABLE *table)
 {
   DBUG_ASSERT(records_are_comparable(table));
 
-  if (table->file->ha_table_flags() & HA_PARTIAL_COLUMN_READ ||
-      table->s->has_update_default_function)
+  if ((table->file->ha_table_flags() & HA_PARTIAL_COLUMN_READ) != 0)
   {
     /*
       Storage engine may not have read all columns of the record.  Fields
       (including NULL bits) not in the write_set may not have been read and
       can therefore not be compared.
-      Or ON UPDATE DEFAULT NOW() could've changed field values, including
-      NULL bits.
     */ 
     for (Field **ptr= table->field ; *ptr != NULL; ptr++)
     {
       Field *field= *ptr;
-      if (field->has_explicit_value() && !field->vcol_info)
+      if (bitmap_is_set(table->write_set, field->field_index))
       {
         if (field->real_maybe_null())
         {
@@ -115,9 +111,8 @@ bool compare_record(const TABLE *table)
   /* Compare updated fields */
   for (Field **ptr= table->field ; *ptr ; ptr++)
   {
-    Field *field= *ptr;
-    if (field->has_explicit_value() && !field->vcol_info &&
-	field->cmp_binary_offset(table->s->rec_buff_length))
+    if (bitmap_is_set(table->write_set, (*ptr)->field_index) &&
+	(*ptr)->cmp_binary_offset(table->s->rec_buff_length))
       return TRUE;
   }
   return FALSE;
@@ -185,10 +180,10 @@ static bool check_fields(THD *thd, List<Item> &items, bool update_view)
   return FALSE;
 }
 
-bool TABLE::vers_check_update(List<Item> &items)
+static bool check_has_vers_fields(TABLE *table, List<Item> &items)
 {
   List_iterator<Item> it(items);
-  if (!versioned_write())
+  if (!table->versioned())
     return false;
 
   while (Item *item= it++)
@@ -196,11 +191,8 @@ bool TABLE::vers_check_update(List<Item> &items)
     if (Item_field *item_field= item->field_for_view_update())
     {
       Field *field= item_field->field;
-      if (field->table == this && !field->vers_update_unversioned())
-      {
-        no_cache= true;
+      if (field->table == table && !field->vers_update_unversioned())
         return true;
-      }
     }
   }
   return false;
@@ -244,7 +236,7 @@ static void prepare_record_for_error_message(int error, TABLE *table)
 
   /* Create unique_map with all fields used by that index. */
   my_bitmap_init(&unique_map, unique_map_buf, table->s->fields, FALSE);
-  table->mark_index_columns(keynr, &unique_map);
+  table->mark_columns_used_by_index(keynr, &unique_map);
 
   /* Subtract read_set and write_set. */
   bitmap_subtract(&unique_map, table->read_set);
@@ -264,14 +256,6 @@ static void prepare_record_for_error_message(int error, TABLE *table)
   bitmap_union(table->read_set, &unique_map);
   /* Tell the engine about the new set. */
   table->file->column_bitmaps_signal();
-
-  if ((error= table->file->ha_index_or_rnd_end()) ||
-      (error= table->file->ha_rnd_init(0)))
-  {
-    table->file->print_error(error, MYF(0));
-    DBUG_VOID_RETURN;
-  }
-
   /* Read record that is identified by table->file->ref. */
   (void) table->file->ha_rnd_pos(table->record[1], table->file->ref);
   /* Copy the newly read columns into the new record. */
@@ -295,6 +279,7 @@ static void prepare_record_for_error_message(int error, TABLE *table)
     order_num		number of elemen in ORDER BY clause
     order		ORDER BY clause list
     limit		limit clause
+    handle_duplicates	how to handle duplicates
 
   RETURN
     0  - OK
@@ -309,13 +294,12 @@ int mysql_update(THD *thd,
 		 List<Item> &values,
                  COND *conds,
                  uint order_num, ORDER *order,
-                 ha_rows limit,
-                 bool ignore,
+		 ha_rows limit,
+		 enum enum_duplicates handle_duplicates, bool ignore,
                  ha_rows *found_return, ha_rows *updated_return)
 {
   bool		using_limit= limit != HA_POS_ERROR;
-  bool          safe_update= (thd->variables.option_bits & OPTION_SAFE_UPDATES)
-                             && !thd->lex->describe;
+  bool          safe_update= thd->variables.option_bits & OPTION_SAFE_UPDATES;
   bool          used_key_is_modified= FALSE, transactional_table;
   bool          will_batch= FALSE;
   bool		can_compare_record;
@@ -365,15 +349,13 @@ int mysql_update(THD *thd,
     DBUG_ASSERT(update_source_table || table_list->view != 0);
     DBUG_PRINT("info", ("Switch to multi-update"));
     /* pass counter value */
-    thd->lex->table_count_update= table_count;
+    thd->lex->table_count= table_count;
     /* convert to multiupdate */
     DBUG_RETURN(2);
   }
   if (lock_tables(thd, table_list, table_count, 0))
     DBUG_RETURN(1);
 
-  (void) read_statistics_for_tables_if_needed(thd, table_list);
-
   THD_STAGE_INFO(thd, stage_init_update);
   if (table_list->handle_derived(thd->lex, DT_MERGE_FOR_INSERT))
     DBUG_RETURN(1);
@@ -400,8 +382,6 @@ int mysql_update(THD *thd,
   want_privilege= (table_list->view ? UPDATE_ACL :
                    table_list->grant.want_privilege);
 #endif
-  promote_select_describe_flag_if_needed(thd->lex);
-
   if (mysql_prepare_update(thd, table_list, &conds, order_num, order))
     DBUG_RETURN(1);
 
@@ -422,7 +402,7 @@ int mysql_update(THD *thd,
   {
     DBUG_RETURN(1);
   }
-  bool has_vers_fields= table->vers_check_update(fields);
+  bool has_vers_fields= check_has_vers_fields(table, fields);
   if (check_key_in_view(thd, table_list))
   {
     my_error(ER_NON_UPDATABLE_TABLE, MYF(0), table_list->alias.str, "UPDATE");
@@ -481,8 +461,6 @@ int mysql_update(THD *thd,
     query_plan.set_no_partitions();
     if (thd->lex->describe || thd->lex->analyze_stmt)
       goto produce_explain_and_leave;
-    if (thd->is_error())
-      DBUG_RETURN(1);
 
     my_ok(thd);				// No matching records
     DBUG_RETURN(0);
@@ -518,7 +496,7 @@ int mysql_update(THD *thd,
   }
 
   /* If running in safe sql mode, don't allow updates without keys */
-  if (!select || !select->quick)
+  if (table->quick_keys.is_clear_all())
   {
     thd->set_status_no_index_used();
     if (safe_update && !using_limit)
@@ -652,20 +630,15 @@ int mysql_update(THD *thd,
       !table->check_virtual_columns_marked_for_write())
   {
     DBUG_PRINT("info", ("Trying direct update"));
-    bool use_direct_update= !select || !select->cond;
-    if (!use_direct_update &&
-        (select->cond->used_tables() & ~RAND_TABLE_BIT) == table->map)
+    if (select && select->cond &&
+        (select->cond->used_tables() == table->map))
     {
       DBUG_ASSERT(!table->file->pushed_cond);
       if (!table->file->cond_push(select->cond))
-      {
-        use_direct_update= TRUE;
         table->file->pushed_cond= select->cond;
-      }
     }
 
-    if (use_direct_update &&
-        !table->file->info_push(INFO_KIND_UPDATE_FIELDS, &fields) &&
+    if (!table->file->info_push(INFO_KIND_UPDATE_FIELDS, &fields) &&
         !table->file->info_push(INFO_KIND_UPDATE_VALUES, &values) &&
         !table->file->direct_update_rows_init(&fields))
     {
@@ -892,6 +865,11 @@ update_begin:
   THD_STAGE_INFO(thd, stage_updating);
   while (!(error=info.read_record()) && !thd->killed)
   {
+    if (table->versioned() && !table->vers_end_field()->is_max())
+    {
+      continue;
+    }
+
     explain->tracker.on_record_read();
     thd->inc_examined_row_count(1);
     if (!select || select->skip_record(thd) > 0)
@@ -910,6 +888,11 @@ update_begin:
 
       if (!can_compare_record || compare_record(table))
       {
+        if (table->default_field && table->update_default_fields(1, ignore))
+        {
+          error= 1;
+          break;
+        }
         if ((res= table_list->view_check_option(thd, ignore)) !=
             VIEW_CHECK_OK)
         {
@@ -969,27 +952,24 @@ update_begin:
         }
         else if (likely(!error))
         {
-          if (has_vers_fields && table->versioned(VERS_TRX_ID))
-            updated_sys_ver++;
-          updated++;
+          if (has_vers_fields && table->versioned())
+          {
+            if (table->versioned(VERS_TIMESTAMP))
+            {
+              store_record(table, record[2]);
+              error= vers_insert_history_row(table);
+              restore_record(table, record[2]);
+            }
+            if (likely(!error))
+              updated_sys_ver++;
+          }
+          if (likely(!error))
+            updated++;
         }
 
         if (unlikely(error) &&
             (!ignore || table->file->is_fatal_error(error, HA_CHECK_ALL)))
         {
-          goto error;
-        }
-      }
-
-      if (likely(!error) && has_vers_fields && table->versioned(VERS_TIMESTAMP))
-      {
-        store_record(table, record[2]);
-        table->mark_columns_per_binlog_row_image();
-        error= vers_insert_history_row(table);
-        restore_record(table, record[2]);
-        if (unlikely(error))
-        {
-error:
           /*
             If (ignore && error is ignorable) we don't have to
             do anything; otherwise...
@@ -1000,11 +980,10 @@ error:
             flags|= ME_FATALERROR; /* Other handler errors are fatal */
 
           prepare_record_for_error_message(error, table);
-          table->file->print_error(error,MYF(flags));
-          error= 1;
-          break;
-        }
-        updated_sys_ver++;
+	  table->file->print_error(error,MYF(flags));
+	  error= 1;
+	  break;
+	}
       }
 
       if (table->triggers &&
@@ -1168,7 +1147,7 @@ update_end:
 
       if (thd->binlog_query(THD::ROW_QUERY_TYPE,
                             thd->query(), thd->query_length(),
-                            transactional_table, FALSE, FALSE, errcode) > 0)
+                            transactional_table, FALSE, FALSE, errcode))
       {
         error=1;				// Rollback update
       }
@@ -1278,15 +1257,6 @@ bool mysql_prepare_update(THD *thd, TABLE_LIST *table_list,
 
   thd->lex->allow_sum_func.clear_all();
 
-  DBUG_ASSERT(table_list->table);
-  // conds could be cached from previous SP call
-  DBUG_ASSERT(!table_list->vers_conditions.need_setup() ||
-              !*conds || thd->stmt_arena->is_stmt_execute());
-  if (select_lex->vers_setup_conds(thd, table_list))
-    DBUG_RETURN(TRUE);
-
-  *conds= select_lex->where;
-
   /*
     We do not call DT_MERGE_FOR_INSERT because it has no sense for simple
     (not multi-) update
@@ -1295,7 +1265,8 @@ bool mysql_prepare_update(THD *thd, TABLE_LIST *table_list,
     DBUG_RETURN(TRUE);
 
   if (setup_tables_and_check_access(thd, &select_lex->context, 
-                                    &select_lex->top_join_list, table_list,
+                                    &select_lex->top_join_list,
+                                    table_list,
                                     select_lex->leaf_tables,
                                     FALSE, UPDATE_ACL, SELECT_ACL, TRUE) ||
       setup_conds(thd, table_list, select_lex->leaf_tables, conds) ||
@@ -1535,78 +1506,102 @@ static bool multi_update_check_table_access(THD *thd, TABLE_LIST *table,
 }
 
 
-class Multiupdate_prelocking_strategy : public DML_prelocking_strategy
-{
-  bool done;
-  bool has_prelocking_list;
-public:
-  void reset(THD *thd);
-  bool handle_end(THD *thd);
-};
+/*
+  make update specific preparation and checks after opening tables
 
-void Multiupdate_prelocking_strategy::reset(THD *thd)
-{
-  done= false;
-  has_prelocking_list= thd->lex->requires_prelocking();
-}
+  SYNOPSIS
+    mysql_multi_update_prepare()
+    thd         thread handler
 
-/**
-  Determine what tables could be updated in the multi-update
-
-  For these tables we'll need to open triggers and continue prelocking
-  until all is open.
+  RETURN
+    FALSE OK
+    TRUE  Error
 */
-bool Multiupdate_prelocking_strategy::handle_end(THD *thd)
+
+int mysql_multi_update_prepare(THD *thd)
 {
-  DBUG_ENTER("Multiupdate_prelocking_strategy::handle_end");
-  if (done)
-    DBUG_RETURN(0);
-
   LEX *lex= thd->lex;
-  SELECT_LEX *select_lex= &lex->select_lex;
-  TABLE_LIST *table_list= lex->query_tables, *tl;
+  TABLE_LIST *table_list= lex->query_tables;
+  TABLE_LIST *tl;
+  List<Item> *fields= &lex->select_lex.item_list;
+  table_map tables_for_update;
+  bool update_view= 0;
+  /*
+    if this multi-update was converted from usual update, here is table
+    counter else junk will be assigned here, but then replaced with real
+    count in open_tables()
+  */
+  uint  table_count= lex->table_count;
+  const bool using_lock_tables= thd->locked_tables_mode != LTM_NONE;
+  bool original_multiupdate= (thd->lex->sql_command == SQLCOM_UPDATE_MULTI);
+  DBUG_ENTER("mysql_multi_update_prepare");
 
-  done= true;
+  /* following need for prepared statements, to run next time multi-update */
+  thd->lex->sql_command= SQLCOM_UPDATE_MULTI;
 
-  if (mysql_handle_derived(lex, DT_INIT) ||
-      mysql_handle_derived(lex, DT_MERGE_FOR_INSERT) ||
-      mysql_handle_derived(lex, DT_PREPARE))
-    DBUG_RETURN(1);
+  /*
+    Open tables and create derived ones, but do not lock and fill them yet.
 
+    During prepare phase acquire only S metadata locks instead of SW locks to
+    keep prepare of multi-UPDATE compatible with concurrent LOCK TABLES WRITE
+    and global read lock.
+  */
+  if ((original_multiupdate &&
+       open_tables(thd, &table_list, &table_count,
+                   (thd->stmt_arena->is_stmt_prepare() ?
+                    MYSQL_OPEN_FORCE_SHARED_MDL : 0))) ||
+      mysql_handle_derived(lex, DT_INIT))
+    DBUG_RETURN(TRUE);
   /*
     setup_tables() need for VIEWs. JOIN::prepare() will call setup_tables()
     second time, but this call will do nothing (there are check for second
     call in setup_tables()).
   */
 
-  if (setup_tables_and_check_access(thd, &select_lex->context,
-      &select_lex->top_join_list, table_list, select_lex->leaf_tables,
-      FALSE, UPDATE_ACL, SELECT_ACL, TRUE))
-    DBUG_RETURN(1);
+  //We need to merge for insert prior to prepare.
+  if (mysql_handle_derived(lex, DT_MERGE_FOR_INSERT))
+    DBUG_RETURN(TRUE);
+
+  if (mysql_handle_derived(lex, DT_PREPARE))
+    DBUG_RETURN(TRUE);
+
+  if (setup_tables_and_check_access(thd, &lex->select_lex.context,
+                                    &lex->select_lex.top_join_list,
+                                    table_list,
+                                    lex->select_lex.leaf_tables, FALSE,
+                                    UPDATE_ACL, SELECT_ACL, FALSE))
+    DBUG_RETURN(TRUE);
+
+  if (lex->select_lex.handle_derived(thd->lex, DT_MERGE))  
+    DBUG_RETURN(TRUE);
 
-  List<Item> *fields= &lex->select_lex.item_list;
   if (setup_fields_with_no_wrap(thd, Ref_ptr_array(),
                                 *fields, MARK_COLUMNS_WRITE, 0, 0))
-    DBUG_RETURN(1);
+    DBUG_RETURN(TRUE);
 
-  // Check if we have a view in the list ...
   for (tl= table_list; tl ; tl= tl->next_local)
+  {
     if (tl->view)
+    {
+      update_view= 1;
       break;
-  // ... and pass this knowlage in check_fields call
-  if (check_fields(thd, *fields, tl != NULL ))
-    DBUG_RETURN(1);
+    }
+  }
 
-  table_map tables_for_update= thd->table_map_for_update= get_table_map(fields);
+  if (check_fields(thd, *fields, update_view))
+  {
+    DBUG_RETURN(TRUE);
+  }
 
-  if (unsafe_key_update(select_lex->leaf_tables, tables_for_update))
-    DBUG_RETURN(1);
+  thd->table_map_for_update= tables_for_update= get_table_map(fields);
+
+  if (unsafe_key_update(lex->select_lex.leaf_tables, tables_for_update))
+    DBUG_RETURN(true);
 
   /*
     Setup timestamp handling and locking mode
   */
-  List_iterator<TABLE_LIST> ti(select_lex->leaf_tables);
-  const bool using_lock_tables= thd->locked_tables_mode != LTM_NONE;
+  List_iterator<TABLE_LIST> ti(lex->select_lex.leaf_tables);
   while ((tl= ti++))
   {
     TABLE *table= tl->table;
@@ -1621,7 +1616,7 @@ bool Multiupdate_prelocking_strategy::handle_end(THD *thd)
       {
         my_error(ER_NON_UPDATABLE_TABLE, MYF(0),
                  tl->top_table()->alias.str, "UPDATE");
-        DBUG_RETURN(1);
+        DBUG_RETURN(TRUE);
       }
 
       DBUG_PRINT("info",("setting table `%s` for update",
@@ -1630,11 +1625,6 @@ bool Multiupdate_prelocking_strategy::handle_end(THD *thd)
         If table will be updated we should not downgrade lock for it and
         leave it as is.
       */
-      tl->updating= 1;
-      if (tl->belong_to_view)
-        tl->belong_to_view->updating= 1;
-      if (extend_table_list(thd, tl, this, has_prelocking_list))
-        DBUG_RETURN(1);
     }
     else
     {
@@ -1657,6 +1647,7 @@ bool Multiupdate_prelocking_strategy::handle_end(THD *thd)
         tl->lock_type= lock_type;
       else
         tl->set_lock_type(thd, lock_type);
+      tl->updating= 0;
     }
   }
 
@@ -1665,7 +1656,6 @@ bool Multiupdate_prelocking_strategy::handle_end(THD *thd)
     Note that unlike in the above loop we need to iterate here not only
     through all leaf tables but also through all view hierarchy.
   */
-
   for (tl= table_list; tl; tl= tl->next_local)
   {
     bool not_used= false;
@@ -1678,78 +1668,26 @@ bool Multiupdate_prelocking_strategy::handle_end(THD *thd)
   /* check single table update for view compound from several tables */
   for (tl= table_list; tl; tl= tl->next_local)
   {
-    TABLE_LIST *for_update= 0;
     if (tl->is_jtbm())
       continue;
-    if (tl->is_merged_derived() &&
-        tl->check_single_table(&for_update, tables_for_update, tl))
+    if (tl->is_merged_derived())
     {
-      my_error(ER_VIEW_MULTIUPDATE, MYF(0), tl->view_db.str, tl->view_name.str);
-      DBUG_RETURN(1);
+      TABLE_LIST *for_update= 0;
+      if (tl->check_single_table(&for_update, tables_for_update, tl))
+      {
+	my_error(ER_VIEW_MULTIUPDATE, MYF(0),
+		 tl->view_db.str, tl->view_name.str);
+	DBUG_RETURN(-1);
+      }
     }
   }
 
-  DBUG_RETURN(0);
-}
-
-/*
-  make update specific preparation and checks after opening tables
-
-  SYNOPSIS
-    mysql_multi_update_prepare()
-    thd         thread handler
-
-  RETURN
-    FALSE OK
-    TRUE  Error
-*/
-
-int mysql_multi_update_prepare(THD *thd)
-{
-  LEX *lex= thd->lex;
-  TABLE_LIST *table_list= lex->query_tables;
-  TABLE_LIST *tl;
-  Multiupdate_prelocking_strategy prelocking_strategy;
-  uint table_count= lex->table_count_update;
-  DBUG_ENTER("mysql_multi_update_prepare");
-
-  /*
-    Open tables and create derived ones, but do not lock and fill them yet.
-
-    During prepare phase acquire only S metadata locks instead of SW locks to
-    keep prepare of multi-UPDATE compatible with concurrent LOCK TABLES WRITE
-    and global read lock.
-
-    Don't evaluate any subqueries even if constant, because
-    tables aren't locked yet.
-  */
-  lex->context_analysis_only|= CONTEXT_ANALYSIS_ONLY_DERIVED;
-  if (thd->lex->sql_command == SQLCOM_UPDATE_MULTI)
-  {
-    if (open_tables(thd, &table_list, &table_count,
-        thd->stmt_arena->is_stmt_prepare() ? MYSQL_OPEN_FORCE_SHARED_MDL : 0,
-        &prelocking_strategy))
-      DBUG_RETURN(TRUE);
-  }
-  else
-  {
-    /* following need for prepared statements, to run next time multi-update */
-    thd->lex->sql_command= SQLCOM_UPDATE_MULTI;
-    prelocking_strategy.reset(thd);
-    if (prelocking_strategy.handle_end(thd))
-      DBUG_RETURN(TRUE);
-  }
-
   /* now lock and fill tables */
   if (!thd->stmt_arena->is_stmt_prepare() &&
       lock_tables(thd, table_list, table_count, 0))
   {
     DBUG_RETURN(TRUE);
   }
-
-  lex->context_analysis_only&= ~CONTEXT_ANALYSIS_ONLY_DERIVED;
-
-  (void) read_statistics_for_tables_if_needed(thd, table_list);
   /* @todo: downgrade the metadata locks here. */
 
   /*
@@ -1757,9 +1695,8 @@ int mysql_multi_update_prepare(THD *thd)
     skip all tables of UPDATE SELECT itself
   */
   lex->select_lex.exclude_from_table_unique_test= TRUE;
-
   /* We only need SELECT privilege for columns in the values list */
-  List_iterator<TABLE_LIST> ti(lex->select_lex.leaf_tables);
+  ti.rewind();
   while ((tl= ti++))
   {
     if (tl->is_jtbm())
@@ -1792,39 +1729,36 @@ int mysql_multi_update_prepare(THD *thd)
   Setup multi-update handling and call SELECT to do the join
 */
 
-bool mysql_multi_update(THD *thd, TABLE_LIST *table_list, List<Item> *fields,
-                        List<Item> *values, COND *conds, ulonglong options,
+bool mysql_multi_update(THD *thd,
+                        TABLE_LIST *table_list,
+                        List<Item> *fields,
+                        List<Item> *values,
+                        COND *conds,
+                        ulonglong options,
                         enum enum_duplicates handle_duplicates,
-                        bool ignore, SELECT_LEX_UNIT *unit,
-                        SELECT_LEX *select_lex, multi_update **result)
+                        bool ignore,
+                        SELECT_LEX_UNIT *unit,
+                        SELECT_LEX *select_lex,
+                        multi_update **result)
 {
   bool res;
   DBUG_ENTER("mysql_multi_update");
-
+  
   if (!(*result= new (thd->mem_root) multi_update(thd, table_list,
                                  &thd->lex->select_lex.leaf_tables,
-                                 fields, values, handle_duplicates, ignore)))
+                                 fields, values,
+                                 handle_duplicates, ignore)))
   {
     DBUG_RETURN(TRUE);
   }
 
-  if ((*result)->init(thd))
-    DBUG_RETURN(1);
-
   thd->abort_on_warning= !ignore && thd->is_strict_mode();
   List<Item> total_list;
 
-  if (setup_tables(thd, &select_lex->context, &select_lex->top_join_list,
-                   table_list, select_lex->leaf_tables, FALSE, FALSE))
-    DBUG_RETURN(1);
-
-  if (select_lex->vers_setup_conds(thd, table_list))
-    DBUG_RETURN(1);
-
   res= mysql_select(thd,
                     table_list, select_lex->with_wild, total_list, conds,
-                    select_lex->order_list.elements,
-                    select_lex->order_list.first, NULL, NULL, NULL,
+                    select_lex->order_list.elements, select_lex->order_list.first,
+                    (ORDER *)NULL, (Item *) NULL, (ORDER *)NULL,
                     options | SELECT_NO_JOIN_CACHE | SELECT_NO_UNLOCK |
                     OPTION_SETUP_TABLES_DONE,
                     *result, unit, select_lex);
@@ -1859,24 +1793,6 @@ multi_update::multi_update(THD *thd_arg, TABLE_LIST *table_list,
 }
 
 
-bool multi_update::init(THD *thd)
-{
-  table_map tables_to_update= get_table_map(fields);
-  List_iterator_fast<TABLE_LIST> li(*leaves);
-  TABLE_LIST *tbl;
-  while ((tbl =li++))
-  {
-    if (tbl->is_jtbm())
-      continue;
-    if (!(tbl->table->map & tables_to_update))
-      continue;
-    if (updated_leaves.push_back(tbl, thd->mem_root))
-      return true;
-  }
-  return false;
-}
-
-
 /*
   Connect fields with tables and create list of tables that are updated
 */
@@ -1893,7 +1809,7 @@ int multi_update::prepare(List<Item> &not_used_values,
   List_iterator_fast<Item> value_it(*values);
   uint i, max_fields;
   uint leaf_table_count= 0;
-  List_iterator<TABLE_LIST> ti(updated_leaves);
+  List_iterator<TABLE_LIST> ti(*leaves);
   DBUG_ENTER("multi_update::prepare");
 
   if (prepared)
@@ -1977,9 +1893,10 @@ int multi_update::prepare(List<Item> &not_used_values,
       if (!tl)
 	DBUG_RETURN(1);
       update.link_in_list(tl, &tl->next_local);
-      table_ref->shared= tl->shared= table_count++;
+      tl->shared= table_count++;
       table->no_keyread=1;
       table->covering_keys.clear_all();
+      table->pos_in_table_list= tl;
       table->prepare_triggers_for_update_stmt_or_event();
       table->reset_default_fields();
     }
@@ -2149,11 +2066,6 @@ multi_update::initialize_tables(JOIN *join)
   if (unlikely((thd->variables.option_bits & OPTION_SAFE_UPDATES) &&
                error_if_full_join(join)))
     DBUG_RETURN(1);
-  if (join->implicit_grouping)
-  {
-    my_error(ER_INVALID_GROUP_FUNC_USE, MYF(0));
-    DBUG_RETURN(1);
-  }
   main_table=join->join_tab->table;
   table_to_update= 0;
 
@@ -2183,7 +2095,7 @@ multi_update::initialize_tables(JOIN *join)
       if (safe_update_on_fly(thd, join->join_tab, table_ref, all_tables))
       {
 	table_to_update= table;			// Update table on the fly
-        has_vers_fields= table->vers_check_update(*fields);
+        has_vers_fields= check_has_vers_fields(table, *fields);
 	continue;
       }
     }
@@ -2385,7 +2297,6 @@ int multi_update::send_data(List<Item> &not_used_values)
 
   for (cur_table= update_tables; cur_table; cur_table= cur_table->next_local)
   {
-    int error= 0;
     TABLE *table= cur_table->table;
     uint offset= cur_table->shared;
     /*
@@ -2403,6 +2314,11 @@ int multi_update::send_data(List<Item> &not_used_values)
     if (table->status & (STATUS_NULL_ROW | STATUS_UPDATED))
       continue;
 
+    if (table->versioned() && !table->vers_end_field()->is_max())
+    {
+      continue;
+    }
+
     if (table == table_to_update)
     {
       /*
@@ -2429,6 +2345,11 @@ int multi_update::send_data(List<Item> &not_used_values)
       found++;
       if (!can_compare_record || compare_record(table))
       {
+	int error;
+
+        if (table->default_field &&
+            unlikely(table->update_default_fields(1, ignore)))
+          DBUG_RETURN(1);
 
         if ((error= cur_table->view_check_option(thd, ignore)) !=
             VIEW_CHECK_OK)
@@ -2455,7 +2376,20 @@ int multi_update::send_data(List<Item> &not_used_values)
           updated--;
           if (!ignore ||
               table->file->is_fatal_error(error, HA_CHECK_ALL))
-            goto error;
+          {
+            /*
+              If (ignore && error == is ignorable) we don't have to
+              do anything; otherwise...
+            */
+            myf flags= 0;
+
+            if (table->file->is_fatal_error(error, HA_CHECK_ALL))
+              flags|= ME_FATALERROR; /* Other handler errors are fatal */
+
+            prepare_record_for_error_message(error, table);
+            table->file->print_error(error,MYF(flags));
+            DBUG_RETURN(1);
+          }
         }
         else
         {
@@ -2464,8 +2398,19 @@ int multi_update::send_data(List<Item> &not_used_values)
             error= 0;
             updated--;
           }
-          else if (has_vers_fields && table->versioned(VERS_TRX_ID))
+          else if (has_vers_fields && table->versioned())
           {
+            if (table->versioned(VERS_TIMESTAMP))
+            {
+              store_record(table, record[2]);
+              if (vers_insert_history_row(table))
+              {
+                restore_record(table, record[2]);
+                error= 1;
+                break;
+              }
+              restore_record(table, record[2]);
+            }
             updated_sys_ver++;
           }
           /* non-transactional or transactional table got modified   */
@@ -2479,17 +2424,6 @@ int multi_update::send_data(List<Item> &not_used_values)
           }
         }
       }
-      if (has_vers_fields && table->versioned(VERS_TIMESTAMP))
-      {
-        store_record(table, record[2]);
-        if (unlikely(error= vers_insert_history_row(table)))
-        {
-          restore_record(table, record[2]);
-          goto error;
-        }
-        restore_record(table, record[2]);
-        updated_sys_ver++;
-      }
       if (table->triggers &&
           unlikely(table->triggers->process_triggers(thd, TRG_EVENT_UPDATE,
                                                      TRG_ACTION_AFTER, TRUE)))
@@ -2497,12 +2431,10 @@ int multi_update::send_data(List<Item> &not_used_values)
     }
     else
     {
+      int error;
       TABLE *tmp_table= tmp_tables[offset];
       if (copy_funcs(tmp_table_param[offset].items_to_copy, thd))
         DBUG_RETURN(1);
-      /* rowid field is NULL if join tmp table has null row from outer join */
-      if (tmp_table->field[0]->is_null())
-        continue;
       /* Store regular updated fields in the row. */
       DBUG_ASSERT(1 + unupdated_check_opt_tables.elements ==
                   tmp_table_param[offset].func_count);
@@ -2531,22 +2463,7 @@ int multi_update::send_data(List<Item> &not_used_values)
         }
       }
     }
-    continue;
-error:
-    DBUG_ASSERT(error > 0);
-    /*
-      If (ignore && error == is ignorable) we don't have to
-      do anything; otherwise...
-    */
-    myf flags= 0;
-
-    if (table->file->is_fatal_error(error, HA_CHECK_ALL))
-      flags|= ME_FATALERROR; /* Other handler errors are fatal */
-
-    prepare_record_for_error_message(error, table);
-    table->file->print_error(error,MYF(flags));
-    DBUG_RETURN(1);
-  } // for (cur_table)
+  }
   DBUG_RETURN(0);
 }
 
@@ -2663,7 +2580,7 @@ int multi_update::do_updates()
     if (table->vfield)
       empty_record(table);
 
-    has_vers_fields= table->vers_check_update(*fields);
+    has_vers_fields= check_has_vers_fields(table, *fields);
 
     check_opt_it.rewind();
     while(TABLE *tbl= check_opt_it++)
@@ -2722,7 +2639,6 @@ int multi_update::do_updates()
       uint field_num= 0;
       do
       {
-        DBUG_ASSERT(!tmp_table->field[field_num]->is_null());
         if (unlikely((local_error=
                       tbl->file->ha_rnd_pos(tbl->record[0],
                                             (uchar *) tmp_table->
@@ -2751,10 +2667,6 @@ int multi_update::do_updates()
         copy_field_ptr->to_field->set_has_explicit_value();
       }
 
-      table->evaluate_update_default_function();
-      if (table->vfield &&
-          table->update_virtual_fields(table->file, VCOL_UPDATE_FOR_WRITE))
-        goto err2;
       if (table->triggers &&
           table->triggers->process_triggers(thd, TRG_EVENT_UPDATE,
                                             TRG_ACTION_BEFORE, TRUE))
@@ -2763,6 +2675,12 @@ int multi_update::do_updates()
       if (!can_compare_record || compare_record(table))
       {
         int error;
+        if (table->default_field &&
+            (error= table->update_default_fields(1, ignore)))
+          goto err2;
+        if (table->vfield &&
+            table->update_virtual_fields(table->file, VCOL_UPDATE_FOR_WRITE))
+          goto err2;
         if ((error= cur_table->view_check_option(thd, ignore)) !=
             VIEW_CHECK_OK)
         {
@@ -2942,7 +2860,7 @@ bool multi_update::send_eof()
 
       if (thd->binlog_query(THD::ROW_QUERY_TYPE, thd->query(),
                             thd->query_length(), transactional_tables, FALSE,
-                            FALSE, errcode) > 0)
+                            FALSE, errcode))
 	local_error= 1;				// Rollback update
       thd->set_current_stmt_binlog_format(save_binlog_format);
     }
