@@ -1,4 +1,4 @@
-/* Copyright 2008-2021 Codership Oy <http://www.codership.com>
+/* Copyright 2008-2015 Codership Oy <http://www.codership.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301 USA */
 
 #include "sql_plugin.h"                         /* wsrep_plugins_pre_init() */
 #include <mysqld.h>
@@ -37,6 +37,7 @@
 #include <cstdio>
 #include <cstdlib>
 #include "log_event.h"
+#include <slave.h>
 
 wsrep_t *wsrep                  = NULL;
 /*
@@ -78,7 +79,6 @@ my_bool wsrep_convert_LOCK_to_trx;              // Convert locking sessions to t
 my_bool wsrep_auto_increment_control;           // Control auto increment variables
 my_bool wsrep_drupal_282555_workaround;         // Retry autoinc insert after dupkey
 my_bool wsrep_certify_nonPK;                    // Certify, even when no primary key
-ulong   wsrep_certification_rules      = WSREP_CERTIFICATION_RULES_STRICT;
 my_bool wsrep_recovery;                         // Recovery
 my_bool wsrep_replicate_myisam;                 // Enable MyISAM replication
 my_bool wsrep_log_conflicts;
@@ -133,11 +133,7 @@ mysql_mutex_t LOCK_wsrep_desync;
 mysql_mutex_t LOCK_wsrep_config_state;
 
 int wsrep_replaying= 0;
-ulong  wsrep_running_threads = 0; // # of currently running wsrep
-				  // # threads
-ulong  wsrep_running_applier_threads = 0; // # of running applier threads
-ulong  wsrep_running_rollbacker_threads = 0; // # of running
-					     // # rollbacker threads
+ulong  wsrep_running_threads = 0; // # of currently running wsrep threads
 ulong  my_bind_addr;
 
 #ifdef HAVE_PSI_INTERFACE
@@ -181,19 +177,7 @@ static PSI_file_info wsrep_files[]=
 {
   { &key_file_wsrep_gra_log, "wsrep_gra_log", 0}
 };
-
-PSI_thread_key key_wsrep_sst_joiner, key_wsrep_sst_donor,
-  key_wsrep_rollbacker, key_wsrep_applier;
-
-static PSI_thread_info wsrep_threads[]=
-{
- {&key_wsrep_sst_joiner, "wsrep_sst_joiner_thread", PSI_FLAG_GLOBAL},
- {&key_wsrep_sst_donor, "wsrep_sst_donor_thread", PSI_FLAG_GLOBAL},
- {&key_wsrep_rollbacker, "wsrep_rollbacker_thread", PSI_FLAG_GLOBAL},
- {&key_wsrep_applier, "wsrep_applier_thread", PSI_FLAG_GLOBAL}
-};
-
-#endif /* HAVE_PSI_INTERFACE */
+#endif
 
 my_bool wsrep_inited                   = 0; // initialized ?
 
@@ -637,6 +621,7 @@ int wsrep_init()
   {
     // enable normal operation in case no provider is specified
     wsrep_ready_set(TRUE);
+    wsrep_inited= 1;
     global_system_variables.wsrep_on = 0;
     wsrep_init_args args;
     args.logger_cb = wsrep_log_cb;
@@ -647,15 +632,10 @@ int wsrep_init()
     {
       DBUG_PRINT("wsrep",("wsrep::init() failed: %d", rcode));
       WSREP_ERROR("wsrep::init() failed: %d, must shutdown", rcode);
-      wsrep_ready_set(FALSE);
       wsrep->free(wsrep);
       free(wsrep);
       wsrep = NULL;
     }
-    else
-    {
-      wsrep_inited= 1;
-    }
     return rcode;
   }
   else
@@ -813,7 +793,6 @@ void wsrep_thr_init()
   mysql_mutex_register("sql", wsrep_mutexes, array_elements(wsrep_mutexes));
   mysql_cond_register("sql", wsrep_conds, array_elements(wsrep_conds));
   mysql_file_register("sql", wsrep_files, array_elements(wsrep_files));
-  mysql_thread_register("sql", wsrep_threads, array_elements(wsrep_threads));
 #endif
 
   mysql_mutex_init(key_LOCK_wsrep_ready, &LOCK_wsrep_ready, MY_MUTEX_INIT_FAST);
@@ -829,29 +808,16 @@ void wsrep_thr_init()
   mysql_mutex_init(key_LOCK_wsrep_slave_threads, &LOCK_wsrep_slave_threads, MY_MUTEX_INIT_FAST);
   mysql_mutex_init(key_LOCK_wsrep_desync, &LOCK_wsrep_desync, MY_MUTEX_INIT_FAST);
   mysql_mutex_init(key_LOCK_wsrep_config_state, &LOCK_wsrep_config_state, MY_MUTEX_INIT_FAST);
-
   DBUG_VOID_RETURN;
 }
 
-/* This is wrapper for wsrep_break_lock in thr_lock.c */
-static int wsrep_thr_abort_thd(void *bf_thd_ptr, void *victim_thd_ptr, my_bool signal)
-{
-  THD* victim_thd= (THD *) victim_thd_ptr;
-  /* We need to lock THD::LOCK_thd_data to protect victim
-  from concurrent usage or disconnect or delete. */
-  wsrep_thd_LOCK(victim_thd);
-  int res= wsrep_abort_thd(bf_thd_ptr, victim_thd_ptr, signal);
-  return res;
-}
-
-
 void wsrep_init_startup (bool first)
 {
   if (wsrep_init()) unireg_abort(1);
 
   wsrep_thr_lock_init(
      (wsrep_thd_is_brute_force_fun)wsrep_thd_is_BF,
-     (wsrep_abort_thd_fun)wsrep_thr_abort_thd,
+     (wsrep_abort_thd_fun)wsrep_abort_thd,
      wsrep_debug, wsrep_convert_LOCK_to_trx,
      (wsrep_on_fun)wsrep_on);
 
@@ -859,7 +825,7 @@ void wsrep_init_startup (bool first)
     Pre-initialize global_system_variables.table_plugin with a dummy engine
     (placeholder) required during the initialization of wsrep threads (THDs).
     (see: plugin_thdvar_init())
-    Note: This only needs to be done for rsync & mariabackup based SST methods.
+    Note: This only needs to be done for rsync & xtrabackup based SST methods.
     In case of mysqldump SST method, the wsrep threads are created after the
     server plugins & global system variables are initialized.
   */
@@ -1561,39 +1527,6 @@ static bool wsrep_can_run_in_toi(THD *thd, const char *db, const char *table,
     {
       return false;
     }
-    /*
-      If mariadb master has replicated a CTAS, we should not replicate the create table
-      part separately as TOI, but to replicate both create table and following inserts
-      as one write set.
-      Howver, if CTAS creates empty table, we should replicate the create table alone
-      as TOI. We have to do relay log event lookup to see if row events follow the
-      create table event.
-    */
-    if (thd->slave_thread && !(thd->rgi_slave->gtid_ev_flags2 & Gtid_log_event::FL_STANDALONE))
-    {
-      /* this is CTAS, either empty or populated table */
-      ulonglong event_size = 0;
-      enum Log_event_type ev_type= wsrep_peak_event(thd->rgi_slave, &event_size);
-      switch (ev_type)
-      {
-      case QUERY_EVENT:
-        /* CTAS with empty table, we replicate create table as TOI */
-        break;
-
-      case TABLE_MAP_EVENT:
-        WSREP_DEBUG("replicating CTAS of empty table as TOI");
-        // fall through
-      case WRITE_ROWS_EVENT:
-        /* CTAS with populated table, we replicate later at commit time */
-        WSREP_DEBUG("skipping create table of CTAS replication");
-        return false;
-
-      default:
-        WSREP_WARN("unexpected async replication event: %d", ev_type);
-      }
-      return true;
-    }
-    /* no next async replication event */
     return true;
 
   case SQLCOM_CREATE_VIEW:
@@ -1615,6 +1548,7 @@ static bool wsrep_can_run_in_toi(THD *thd, const char *db, const char *table,
 
   case SQLCOM_CREATE_TRIGGER:
 
+    DBUG_ASSERT(!table_list);
     DBUG_ASSERT(first_table);
 
     if (thd->find_temporary_table(first_table))
@@ -1623,14 +1557,6 @@ static bool wsrep_can_run_in_toi(THD *thd, const char *db, const char *table,
     }
     return true;
 
-  case SQLCOM_DROP_TRIGGER:
-    DBUG_ASSERT(table_list);
-    if (thd->find_temporary_table(table_list))
-    {
-      return false;
-    }
-    return true;
-
   default:
     if (table && !thd->find_temporary_table(db, table))
     {
@@ -1651,6 +1577,25 @@ static bool wsrep_can_run_in_toi(THD *thd, const char *db, const char *table,
   }
 }
 
+static const char* wsrep_get_query_or_msg(const THD* thd)
+{
+  switch(thd->lex->sql_command)
+  {
+    case SQLCOM_CREATE_USER:
+      return "CREATE USER";
+    case SQLCOM_GRANT:
+      return "GRANT";
+    case SQLCOM_REVOKE:
+      return "REVOKE";
+    case SQLCOM_SET_OPTION:
+      if (thd->lex->definer)
+	return "SET PASSWORD";
+      /* fallthrough */
+    default:
+      return thd->query();
+   }
+}
+
 /*
   returns: 
    0: statement was replicated as TOI
@@ -1674,7 +1619,7 @@ static int wsrep_TOI_begin(THD *thd, const char *db_, const char *table_,
   }
 
   WSREP_DEBUG("TO BEGIN: %lld, %d : %s", (long long)wsrep_thd_trx_seqno(thd),
-              thd->wsrep_exec_mode, wsrep_thd_query(thd));
+              thd->wsrep_exec_mode, wsrep_get_query_or_msg(thd));
 
   switch (thd->lex->sql_command)
   {
@@ -1697,11 +1642,6 @@ static int wsrep_TOI_begin(THD *thd, const char *db_, const char *table_,
   case SQLCOM_DROP_TABLE:
     buf_err= wsrep_drop_table_query(thd, &buf, &buf_len);
     break;
-  case SQLCOM_KILL:
-    WSREP_DEBUG("KILL as TOI: %s", thd->query());
-    buf_err= wsrep_to_buf_helper(thd, thd->query(), thd->query_length(),
-                                 &buf, &buf_len);
-    break;
   case SQLCOM_CREATE_ROLE:
     if (sp_process_definer(thd))
     {
@@ -1759,13 +1699,13 @@ static void wsrep_TOI_end(THD *thd) {
   wsrep_to_isolation--;
 
   WSREP_DEBUG("TO END: %lld, %d: %s", (long long)wsrep_thd_trx_seqno(thd),
-              thd->wsrep_exec_mode, wsrep_thd_query(thd));
+              thd->wsrep_exec_mode, wsrep_get_query_or_msg(thd));
 
   wsrep_set_SE_checkpoint(thd->wsrep_trx_meta.gtid.uuid,
                           thd->wsrep_trx_meta.gtid.seqno);
   WSREP_DEBUG("TO END: %lld, update seqno",
               (long long)wsrep_thd_trx_seqno(thd));
-
+  
   if (WSREP_OK == (ret = wsrep->to_execute_end(wsrep, thd->thread_id))) {
     WSREP_DEBUG("TO END: %lld", (long long)wsrep_thd_trx_seqno(thd));
   }
@@ -1813,7 +1753,7 @@ static int wsrep_RSU_begin(THD *thd, const char *db_, const char *table_)
     }
 
     my_error(ER_LOCK_DEADLOCK, MYF(0));
-    return(-1);
+    return(1);
   }
 
   wsrep_seqno_t seqno = wsrep->pause(wsrep);
@@ -2022,14 +1962,14 @@ bool wsrep_grant_mdl_exception(MDL_context *requestor_ctx,
                   request_thd, granted_thd);
     ticket->wsrep_report(wsrep_debug);
 
-    wsrep_thd_LOCK(granted_thd);
+    mysql_mutex_lock(&granted_thd->LOCK_thd_data);
     if (granted_thd->wsrep_exec_mode == TOTAL_ORDER ||
         granted_thd->wsrep_exec_mode == REPL_RECV)
     {
       WSREP_MDL_LOG(INFO, "MDL BF-BF conflict", schema, schema_len,
                     request_thd, granted_thd);
       ticket->wsrep_report(true);
-      wsrep_thd_UNLOCK(granted_thd);
+      mysql_mutex_unlock(&granted_thd->LOCK_thd_data);
       ret= true;
     }
     else if (granted_thd->lex->sql_command == SQLCOM_FLUSH ||
@@ -2037,7 +1977,7 @@ bool wsrep_grant_mdl_exception(MDL_context *requestor_ctx,
     {
       WSREP_DEBUG("BF thread waiting for FLUSH");
       ticket->wsrep_report(wsrep_debug);
-      wsrep_thd_UNLOCK(granted_thd);
+      mysql_mutex_unlock(&granted_thd->LOCK_thd_data);
       ret= false;
     }
     else
@@ -2062,10 +2002,8 @@ bool wsrep_grant_mdl_exception(MDL_context *requestor_ctx,
         ticket->wsrep_report(true);
       }
 
-      /* This will call wsrep_abort_transaction so we should hold
-      THD::LOCK_thd_data to protect victim from concurrent usage
-      or disconnect or delete. */
-      wsrep_abort_thd((void *) request_thd, (void *) granted_thd, true);
+      mysql_mutex_unlock(&granted_thd->LOCK_thd_data);
+      wsrep_abort_thd((void *) request_thd, (void *) granted_thd, 1);
       ret= false;
     }
   }
@@ -2081,8 +2019,7 @@ bool wsrep_grant_mdl_exception(MDL_context *requestor_ctx,
 pthread_handler_t start_wsrep_THD(void *arg)
 {
   THD *thd;
-  wsrep_thread_args* args= (wsrep_thread_args*)arg;
-  wsrep_thd_processor_fun processor= args->processor;
+  wsrep_thd_processor_fun processor= (wsrep_thd_processor_fun)arg;
 
   if (my_thread_init() || (!(thd= new THD(next_thread_id(), true))))
   {
@@ -2134,7 +2071,7 @@ pthread_handler_t start_wsrep_THD(void *arg)
     need to know the start of the stack so that we could check for
     stack overruns.
   */
-  DBUG_PRINT("wsrep", ("handle_one_connection called by thread %lld",
+  DBUG_PRINT("wsrep", ("handle_one_connection called by thread %lld\n",
                        (long long)thd->thread_id));
   /* now that we've called my_thread_init(), it is safe to call DBUG_* */
 
@@ -2158,19 +2095,6 @@ pthread_handler_t start_wsrep_THD(void *arg)
 
   mysql_mutex_lock(&LOCK_thread_count);
   wsrep_running_threads++;
-
-  switch (args->thread_type) {
-    case WSREP_APPLIER_THREAD:
-      wsrep_running_applier_threads++;
-      break;
-    case WSREP_ROLLBACKER_THREAD:
-      wsrep_running_rollbacker_threads++;
-      break;
-    default:
-      WSREP_ERROR("Incorrect wsrep thread type: %d", args->thread_type);
-      break;
-  }
-
   mysql_cond_broadcast(&COND_thread_count);
   mysql_mutex_unlock(&LOCK_thread_count);
 
@@ -2179,25 +2103,7 @@ pthread_handler_t start_wsrep_THD(void *arg)
   close_connection(thd, 0);
 
   mysql_mutex_lock(&LOCK_thread_count);
-  DBUG_ASSERT(wsrep_running_threads > 0);
   wsrep_running_threads--;
-
-  switch (args->thread_type) {
-    case WSREP_APPLIER_THREAD:
-      DBUG_ASSERT(wsrep_running_applier_threads > 0);
-      wsrep_running_applier_threads--;
-      break;
-    case WSREP_ROLLBACKER_THREAD:
-      DBUG_ASSERT(wsrep_running_rollbacker_threads > 0);
-      wsrep_running_rollbacker_threads--;
-      break;
-    default:
-      WSREP_ERROR("Incorrect wsrep thread type: %d", args->thread_type);
-      break;
-  }
-
-  my_free(args);
-
   WSREP_DEBUG("wsrep running threads now: %lu", wsrep_running_threads);
   mysql_cond_broadcast(&COND_thread_count);
   mysql_mutex_unlock(&LOCK_thread_count);
@@ -2226,8 +2132,6 @@ pthread_handler_t start_wsrep_THD(void *arg)
 error:
   WSREP_ERROR("Failed to create/initialize system thread");
 
-  my_free(args);
-
   /* Abort if its the first applier/rollbacker thread. */
   if (!mysqld_server_initialized)
     unireg_abort(1);
@@ -2240,7 +2144,6 @@ error:
 static bool abort_replicated(THD *thd)
 {
   bool ret_code= false;
-  wsrep_thd_LOCK(thd);
   if (thd->wsrep_query_state== QUERY_COMMITTING)
   {
     WSREP_DEBUG("aborting replicated trx: %llu", (ulonglong)(thd->real_id));
@@ -2248,8 +2151,6 @@ static bool abort_replicated(THD *thd)
     (void)wsrep_abort_thd(thd, thd, TRUE);
     ret_code= true;
   }
-  else
-    wsrep_thd_UNLOCK(thd);
   return ret_code;
 }
 
@@ -2296,8 +2197,6 @@ static bool have_client_connections()
                        (longlong) tmp->thread_id));
     if (is_client_connection(tmp) && tmp->killed == KILL_CONNECTION)
     {
-      WSREP_DEBUG("Informing thread %lld that it's time to die",
-                  (longlong)tmp->thread_id);
       (void)abort_replicated(tmp);
       return true;
     }
@@ -2337,7 +2236,6 @@ static my_bool have_committing_connections()
 
     if (is_committing_connection(tmp))
     {
-      mysql_mutex_unlock(&LOCK_thread_count);
       return TRUE;
     }
   }
@@ -2382,8 +2280,6 @@ void wsrep_close_client_connections(my_bool wait_to_end, THD *except_caller_thd)
   {
     DBUG_PRINT("quit",("Informing thread %lld that it's time to die",
                        (longlong) tmp->thread_id));
-    WSREP_DEBUG("Informing thread %lld that it's time to die",
-                (longlong)tmp->thread_id);
     /* We skip slave threads & scheduler on this first loop through. */
     if (!is_client_connection(tmp))
       continue;
@@ -2400,19 +2296,21 @@ void wsrep_close_client_connections(my_bool wait_to_end, THD *except_caller_thd)
       continue;
     }
 
-    /* replicated transactions must be skipped and aborted
-    with wsrep_abort_thd. */
+    /* replicated transactions must be skipped */
     if (abort_replicated(tmp))
       continue;
 
     WSREP_DEBUG("closing connection %lld", (longlong) tmp->thread_id);
 
     /*
-      instead of wsrep_close_thread() we do now  soft kill by
-      THD::awake(). Here also victim needs to be protected from
-      concurrent usage or disconnect or delete.
-    */
+      instead of wsrep_close_thread() we do now  soft kill by THD::awake
+     */
+    mysql_mutex_lock(&tmp->LOCK_thd_data);
+
     tmp->awake(KILL_CONNECTION);
+
+    mysql_mutex_unlock(&tmp->LOCK_thd_data);
+
   }
   mysql_mutex_unlock(&LOCK_thread_count);
 
@@ -2427,6 +2325,7 @@ void wsrep_close_client_connections(my_bool wait_to_end, THD *except_caller_thd)
   I_List_iterator<THD> it2(threads);
   while ((tmp=it2++))
   {
+#ifndef __bsdi__				// Bug in BSDI kernel
     if (is_client_connection(tmp) &&
         !abort_replicated(tmp)    &&
         !is_replaying_connection(tmp) &&
@@ -2435,6 +2334,7 @@ void wsrep_close_client_connections(my_bool wait_to_end, THD *except_caller_thd)
       WSREP_INFO("killing local connection: %lld", (longlong) tmp->thread_id);
       close_connection(tmp,0);
     }
+#endif
   }
 
   DBUG_PRINT("quit",("Waiting for threads to die (count=%u)",thread_count));
@@ -2604,25 +2504,13 @@ extern "C" void wsrep_thd_set_exec_mode(THD *thd, enum wsrep_exec_mode mode)
 extern "C" void wsrep_thd_set_query_state(
 	THD *thd, enum wsrep_query_state state)
 {
-  /* async slave thread should never flag IDLE state, as it may
-     give rollbacker thread chance to interfere and rollback async slave
-     transaction.
-     in fact, async slave thread is never idle as it reads complete
-     transactions from relay log and applies them, as a whole.
-     BF abort happens voluntarily by async slave thread.
-  */
-  if (thd->slave_thread && state == QUERY_IDLE) {
-    WSREP_DEBUG("Skipping IDLE state change for slave SQL");
-    return;
-  }
   thd->wsrep_query_state= state;
 }
 
 
 void wsrep_thd_set_conflict_state(THD *thd, enum wsrep_conflict_state state)
 {
-  mysql_mutex_assert_owner(&thd->LOCK_thd_data);
-  thd->wsrep_conflict_state= state;
+  if (WSREP(thd)) thd->wsrep_conflict_state= state;
 }
 
 
@@ -2690,29 +2578,15 @@ wsrep_ws_handle_t* wsrep_thd_ws_handle(THD *thd)
 void wsrep_thd_LOCK(THD *thd)
 {
   mysql_mutex_lock(&thd->LOCK_thd_data);
-  mysql_mutex_lock(&thd->LOCK_thd_kill);
 }
 
 
 void wsrep_thd_UNLOCK(THD *thd)
 {
-  mysql_mutex_unlock(&thd->LOCK_thd_kill);
   mysql_mutex_unlock(&thd->LOCK_thd_data);
 }
 
 
-void wsrep_thd_kill_LOCK(THD *thd)
-{
-  mysql_mutex_lock(&thd->LOCK_thd_kill);
-}
-
-
-void wsrep_thd_kill_UNLOCK(THD *thd)
-{
-  mysql_mutex_unlock(&thd->LOCK_thd_kill);
-}
-
-
 extern "C" time_t wsrep_thd_query_start(THD *thd) 
 {
   return thd->query_start();
@@ -2736,28 +2610,9 @@ extern "C" query_id_t wsrep_thd_query_id(THD *thd)
 }
 
 
-const char *wsrep_thd_query(THD *thd)
+char *wsrep_thd_query(THD *thd)
 {
-  if (thd)
-  {
-    switch(thd->lex->sql_command)
-    {
-    case SQLCOM_CREATE_USER:
-      return "CREATE USER";
-    case SQLCOM_GRANT:
-      return "GRANT";
-    case SQLCOM_REVOKE:
-      return "REVOKE";
-    case SQLCOM_SET_OPTION:
-      if (thd->lex->definer)
-	return "SET PASSWORD";
-      /* fallthrough */
-    default:
-      if (thd->query())
-        return thd->query();
-    }
-  }
-  return "NULL";
+  return (thd) ? thd->query() : NULL;
 }
 
 
@@ -2777,12 +2632,7 @@ extern "C" void wsrep_thd_awake(THD *thd, my_bool signal)
 {
   if (signal)
   {
-    /* Here we should hold THD::LOCK_thd_data to
-    protect from concurrent usage and
-    THD::LOCK_thd_kill from disconnect or delete */
-    mysql_mutex_assert_owner(&thd->LOCK_thd_data);
-    mysql_mutex_assert_owner(&thd->LOCK_thd_kill);
-    thd->awake_no_mutex(KILL_QUERY);
+    thd->awake(KILL_QUERY);
   }
   else
   {
@@ -2808,18 +2658,16 @@ extern "C" bool wsrep_thd_ignore_table(THD *thd)
 extern int
 wsrep_trx_order_before(THD *thd1, THD *thd2)
 {
-  const longlong trx1_seqno= wsrep_thd_trx_seqno(thd1);
-  const longlong trx2_seqno= wsrep_thd_trx_seqno(thd2);
-  WSREP_DEBUG("BF conflict, order: %lld %lld\n",
-              trx1_seqno, trx2_seqno);
-
-  if (trx1_seqno == WSREP_SEQNO_UNDEFINED ||
-      trx2_seqno == WSREP_SEQNO_UNDEFINED)
-    return 1; /* trx is not yet replicated */
-  else if (trx1_seqno < trx2_seqno)
-    return 1;
-
-  return 0;
+    if (wsrep_thd_trx_seqno(thd1) < wsrep_thd_trx_seqno(thd2)) {
+        WSREP_DEBUG("BF conflict, order: %lld %lld\n",
+                    (long long)wsrep_thd_trx_seqno(thd1),
+                    (long long)wsrep_thd_trx_seqno(thd2));
+        return 1;
+    }
+    WSREP_DEBUG("waiting for BF, trx order: %lld %lld\n",
+                (long long)wsrep_thd_trx_seqno(thd1),
+                (long long)wsrep_thd_trx_seqno(thd2));
+    return 0;
 }
 
 
@@ -2862,7 +2710,7 @@ bool wsrep_create_like_table(THD* thd, TABLE_LIST* table,
   if (create_info->tmp_table())
   {
     /* CREATE TEMPORARY TABLE LIKE must be skipped from replication */
-    WSREP_DEBUG("CREATE TEMPORARY TABLE LIKE... skipped replication\n %s",
+    WSREP_DEBUG("CREATE TEMPORARY TABLE LIKE... skipped replication\n %s", 
                 thd->query());
   }
   else if (!(thd->find_temporary_table(src_table)))
@@ -2872,17 +2720,21 @@ bool wsrep_create_like_table(THD* thd, TABLE_LIST* table,
   }
   else
   {
-    /* Non-MERGE tables ignore this call. */
-    if (src_table->table->file->extra(HA_EXTRA_ADD_CHILDREN_LIST))
-      return (true);
-
+    /* here we have CREATE TABLE LIKE <temporary table> 
+       the temporary table definition will be needed in slaves to
+       enable the create to succeed
+     */
+    TABLE_LIST tbl;
+    bzero((void*) &tbl, sizeof(tbl));
+    tbl.db= src_table->db;
+    tbl.table_name= tbl.alias= src_table->table_name;
+    tbl.table= src_table->table;
     char buf[2048];
     String query(buf, sizeof(buf), system_charset_info);
     query.length(0);  // Have to zero it since constructor doesn't
 
-    int result __attribute__((unused))=
-      show_create_table(thd, src_table, &query, NULL, WITH_DB_NAME);
-    WSREP_DEBUG("TMP TABLE: %s ret_code %d", query.ptr(), result);
+    (void)  show_create_table(thd, &tbl, &query, NULL, WITH_DB_NAME);
+    WSREP_DEBUG("TMP TABLE: %s", query.ptr());
 
     thd->wsrep_TOI_pre_query=     query.ptr();
     thd->wsrep_TOI_pre_query_len= query.length();
@@ -2891,14 +2743,11 @@ bool wsrep_create_like_table(THD* thd, TABLE_LIST* table,
 
     thd->wsrep_TOI_pre_query=      NULL;
     thd->wsrep_TOI_pre_query_len= 0;
-
-    /* Non-MERGE tables ignore this call. */
-    src_table->table->file->extra(HA_EXTRA_DETACH_CHILDREN);
   }
 
   return(false);
 
-wsrep_error_label:
+WSREP_ERROR_LABEL:
   thd->wsrep_TOI_pre_query= NULL;
   return (true);
 }
@@ -2943,14 +2792,7 @@ static int wsrep_create_trigger_query(THD *thd, uchar** buf, size_t* buf_len)
     definer_host.length= 0;
   }
 
-  const LEX_CSTRING command[2]=
-      {{ C_STRING_WITH_LEN("CREATE ") },
-       { C_STRING_WITH_LEN("CREATE OR REPLACE ") }};
-
-  if (thd->lex->create_info.or_replace())
-    stmt_query.append(command[1]);
-  else
-    stmt_query.append(command[0]);
+  stmt_query.append(STRING_WITH_LEN("CREATE "));
 
   append_definer(thd, &stmt_query, &definer_user, &definer_host);
 
