@@ -3,7 +3,7 @@
 /* -------------                                                       */
 /*  Version 3.0                                                        */
 /*                                                                     */
-/*  Author Olivier BERTRAND          2007 - 2020                       */
+/*  Author Olivier BERTRAND          2007 - 2017                       */
 /*                                                                     */
 /*  This program are the XML tables classes using MS-DOM or libxml2.   */
 /***********************************************************************/
@@ -15,12 +15,12 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <errno.h>
-#if defined(_WIN32)
+#if defined(__WIN__)
 #include <io.h>
 #include <winsock2.h>
 //#include <windows.h>
 #include <comdef.h>
-#else   // !_WIN32
+#else   // !__WIN__
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -28,7 +28,7 @@
 //#include <ctype.h>
 #include "osutil.h"
 #define _O_RDONLY O_RDONLY
-#endif  // !_WIN32
+#endif  // !__WIN__
 #include "resource.h"                        // for IDS_COLUMNS
 
 #define INCLUDE_TDBXML
@@ -53,17 +53,15 @@
 
 extern "C" char version[];
 
-#if defined(_WIN32) && defined(DOMDOC_SUPPORT)
+#if defined(__WIN__) && defined(DOMDOC_SUPPORT)
 #define XMLSUP "MS-DOM"
-#else   // !_WIN32
+#else   // !__WIN__
 #define XMLSUP "libxml2"
-#endif  // !_WIN32
+#endif  // !__WIN__
 
 #define TYPE_UNKNOWN     12        /* Must be greater than other types */
 #define XLEN(M)  sizeof(M) - strlen(M) - 1	       /* To avoid overflow*/
 
-int GetDefaultDepth(void);
-
 /***********************************************************************/
 /* Class and structure used by XMLColumns.                             */
 /***********************************************************************/
@@ -148,21 +146,13 @@ PQRYRES XMLColumns(PGLOBAL g, char *db, char *tab, PTOS topt, bool info)
   /*  Open the input file.                                             */
   /*********************************************************************/
   if (!(fn = GetStringTableOption(g, topt, "Filename", NULL))) {
-    if (topt->http) // REST table can have default filename
-      fn = GetStringTableOption(g, topt, "Subtype", NULL);
-    
-    if (!fn) {
-      strcpy(g->Message, MSG(MISSING_FNAME));
-      return NULL;
-    } else
-      topt->subtype = NULL;
-
+    strcpy(g->Message, MSG(MISSING_FNAME));
+    return NULL;
+  } else {
+    lvl = GetIntegerTableOption(g, topt, "Level", 0);
+    lvl = (lvl < 0) ? 0 : (lvl > 16) ? 16 : lvl;
   } // endif fn
 
-  lvl = GetIntegerTableOption(g, topt, "Level", GetDefaultDepth());
-  lvl = GetIntegerTableOption(g, topt, "Depth", lvl);
-  lvl = (lvl < 0) ? 0 : (lvl > 16) ? 16 : lvl;
-
   if (trace(1))
     htrc("File %s lvl=%d\n", topt->filename, lvl);
 
@@ -173,18 +163,15 @@ PQRYRES XMLColumns(PGLOBAL g, char *db, char *tab, PTOS topt, bool info)
 		return NULL;
 
   tdp->Tabname = tab;
-	tdp->Tabname = (char*)GetStringTableOption(g, topt, "Tabname", tab);
-	tdp->Rowname = (char*)GetStringTableOption(g, topt, "Rownode", NULL);
 	tdp->Zipped = GetBooleanTableOption(g, topt, "Zipped", false);
 	tdp->Entry = GetStringTableOption(g, topt, "Entry", NULL);
-	tdp->Skip = GetBooleanTableOption(g, topt, "Skipnull", false);
 
   if (!(op = GetStringTableOption(g, topt, "Xmlsup", NULL)))
-#if defined(_WIN32)
+#if defined(__WIN__)
     tdp->Usedom = true;
-#else   // !_WIN32
+#else   // !__WIN__
     tdp->Usedom = false;
-#endif  // !_WIN32
+#endif  // !__WIN__
   else
     tdp->Usedom = (toupper(*op) == 'M' || toupper(*op) == 'D');
 
@@ -250,9 +237,7 @@ PQRYRES XMLColumns(PGLOBAL g, char *db, char *tab, PTOS topt, bool info)
 
      more:
       if (vp->atp) {
-				size_t z = sizeof(colname) - 1;
-        strncpy(colname, vp->atp->GetName(g), z);
-				colname[z] = 0;
+        strncpy(colname, vp->atp->GetName(g), sizeof(colname));
 				strncat(xcol->Name, colname, XLEN(xcol->Name));
 
         switch (vp->atp->GetText(g, buf, sizeof(buf))) {
@@ -295,9 +280,7 @@ PQRYRES XMLColumns(PGLOBAL g, char *db, char *tab, PTOS topt, bool info)
             if (!vp->atp)
 							node = vp->nl->GetItem(g, vp->k++, tdp->Usedom ? node : NULL);
 
-						if (!j)
-							strncat(fmt, colname, XLEN(fmt));
-
+						strncat(fmt, colname, XLEN(fmt));
 						strncat(fmt, "/", XLEN(fmt));
 						strncat(xcol->Name, "_", XLEN(xcol->Name));
 						j++;
@@ -319,7 +302,6 @@ PQRYRES XMLColumns(PGLOBAL g, char *db, char *tab, PTOS topt, bool info)
           case RC_INFO:
             PushWarning(g, txmp);
           case RC_OK:
-						xcol->Cbn = !strlen(buf);
             break;
           default:
             goto err;
@@ -345,9 +327,9 @@ PQRYRES XMLColumns(PGLOBAL g, char *db, char *tab, PTOS topt, bool info)
 
         xcp->Len = MY_MAX(xcp->Len, xcol->Len);
         xcp->Scale = MY_MAX(xcp->Scale, xcol->Scale);
-        xcp->Cbn |= (xcol->Cbn || !xcol->Len);
+        xcp->Cbn |= xcol->Cbn;
         xcp->Found = true;
-      } else if(xcol->Len || !tdp->Skip) {
+      } else {
         // New column
         xcp = new(g) XMCOL(g, xcol, fmt, i);
         length[0] = MY_MAX(length[0], strlen(xcol->Name));
@@ -362,8 +344,7 @@ PQRYRES XMLColumns(PGLOBAL g, char *db, char *tab, PTOS topt, bool info)
         n++;
       } // endif xcp
 
-			if (xcp)
-				pxcp = xcp;
+      pxcp = xcp;
 
       if (vp->atp)
         vp->atp = vp->atp->GetNext(g);
@@ -464,7 +445,6 @@ XMLDEF::XMLDEF(void)
   Usedom = false;
 	Zipped = false;
 	Mulentries = false;
-	Skip = false;
   } // end of XMLDEF constructor
 
 /***********************************************************************/
@@ -512,7 +492,7 @@ bool XMLDEF::DefineAM(PGLOBAL g, LPCSTR am, int poff)
       defcol = "TD";
       break;
     default:
-      snprintf(g->Message, sizeof(g->Message), MSG(INV_COL_TYPE), buf);
+      sprintf(g->Message, MSG(INV_COL_TYPE), buf);
       return true;
     } // endswitch typname
 
@@ -525,7 +505,7 @@ bool XMLDEF::DefineAM(PGLOBAL g, LPCSTR am, int poff)
   XmlDB = GetStringCatInfo(g, "XmlDB", NULL);
   Nslist = GetStringCatInfo(g, "Nslist", NULL);
   DefNs = GetStringCatInfo(g, "DefNs", NULL);
-  Limit = GetIntCatInfo("Limit", 50);
+  Limit = GetIntCatInfo("Limit", 10);
   Xpand = GetBoolCatInfo("Expand", false);
   Header = GetIntCatInfo("Header", 0);
   GetCharCatInfo("Xmlsup", "*", buf, sizeof(buf));
@@ -533,11 +513,11 @@ bool XMLDEF::DefineAM(PGLOBAL g, LPCSTR am, int poff)
   // Note that if no support is specified, the default is MS-DOM
   // on Windows and libxml2 otherwise
   if (*buf == '*')
-#if defined(_WIN32)
+#if defined(__WIN__)
     Usedom = true;
-#else   // !_WIN32
+#else   // !__WIN__
     Usedom = false;
-#endif  // !_WIN32
+#endif  // !__WIN__
   else
     Usedom = (toupper(*buf) == 'M' || toupper(*buf) == 'D');
 
@@ -777,7 +757,7 @@ int TDBXML::LoadTableFile(PGLOBAL g, char *filename)
 
     // Initialize the implementation
     if (Docp->Initialize(g, Entry, Zipped)) {
-      snprintf(g->Message, sizeof(g->Message), MSG(INIT_FAILED), (Usedom) ? "DOM" : "libxml2");
+      sprintf(g->Message, MSG(INIT_FAILED), (Usedom) ? "DOM" : "libxml2");
       return RC_FX;
       } // endif init
 
@@ -823,155 +803,138 @@ bool TDBXML::Initialize(PGLOBAL g)
   if (Void)
     return false;
 
-  if (Columns) {
+  if (Columns && !Bufdone) {
     // Allocate the buffers that will contain node values
     for (colp = (PXMLCOL)Columns; colp; colp = (PXMLCOL)colp->GetNext())
-			if (!colp->IsSpecial()) {            // Not a pseudo column
-				if (!Bufdone && colp->AllocBuf(g, Mode == MODE_INSERT))
-					return true;
-
-				colp->Nx = colp->Sx = -1;
-			} // endif Special
+      if (!colp->IsSpecial())            // Not a pseudo column
+        if (colp->AllocBuf(g, Mode == MODE_INSERT))
+          return true;
 
     Bufdone = true;
     } // endif Bufdone
 
 #if !defined(UNIX)
-	if (!Root) try {
+  if (!Root) try {
 #else
-	if (!Root) {
+  if (!Root) {
 #endif
-		char tabpath[64], filename[_MAX_PATH];
+    char tabpath[64], filename[_MAX_PATH];
 
-		//  We used the file name relative to recorded datapath
-		PlugSetPath(filename, Xfile, GetPath());
+    //  We used the file name relative to recorded datapath
+    PlugSetPath(filename, Xfile, GetPath());
 
-		// Load or re-use the table file
-		rc = LoadTableFile(g, filename);
+    // Load or re-use the table file
+    rc = LoadTableFile(g, filename);
 
-		if (rc == RC_OK) {
-			// Get root node
-			if (!(Root = Docp->GetRoot(g))) {
-				// This should never happen as load should have failed
-				strcpy(g->Message, MSG(EMPTY_DOC));
-				goto error;
-			} // endif Root
+    if (rc == RC_OK) {
+      // Get root node
+      if (!(Root = Docp->GetRoot(g))) {
+        // This should never happen as load should have failed
+        strcpy(g->Message, MSG(EMPTY_DOC));
+        goto error;
+        } // endif Root
 
-		// If tabname is not an Xpath,
-		// construct one that will find it anywhere
-			if (!strchr(Tabname, '/'))
-				strcat(strcpy(tabpath, "//"), Tabname);
-			else
-				strcpy(tabpath, Tabname);
+      // If tabname is not an Xpath,
+      // construct one that will find it anywhere
+      if (!strchr(Tabname, '/'))
+        strcat(strcpy(tabpath, "//"), Tabname);
+      else
+        strcpy(tabpath, Tabname);
 
-			// Evaluate table xpath
-			if ((TabNode = Root->SelectSingleNode(g, tabpath))) {
-				if (TabNode->GetType() != XML_ELEMENT_NODE) {
-					snprintf(g->Message, sizeof(g->Message), MSG(BAD_NODE_TYPE), TabNode->GetType());
-					goto error;
-				} // endif Type
+      // Evaluate table xpath
+      if ((TabNode = Root->SelectSingleNode(g, tabpath))) {
+        if (TabNode->GetType() != XML_ELEMENT_NODE) {
+          sprintf(g->Message, MSG(BAD_NODE_TYPE), TabNode->GetType());
+          goto error;
+          } // endif Type
 
-			} else if (Mode == MODE_INSERT && XmlDB) {
-				// We are adding a new table to a multi-table file
+      } else if (Mode == MODE_INSERT && XmlDB) {
+        // We are adding a new table to a multi-table file
 
-				// If XmlDB is not an Xpath,
-				// construct one that will find it anywhere
-				if (!strchr(XmlDB, '/'))
-					strcat(strcpy(tabpath, "//"), XmlDB);
-				else
-					strcpy(tabpath, XmlDB);
+        // If XmlDB is not an Xpath,
+        // construct one that will find it anywhere
+        if (!strchr(XmlDB, '/'))
+          strcat(strcpy(tabpath, "//"), XmlDB);
+        else
+          strcpy(tabpath, XmlDB);
 
-				if (!(DBnode = Root->SelectSingleNode(g, tabpath))) {
-					// DB node does not exist yet; we cannot create it
-					// because we don't know where it should be placed
-					snprintf(g->Message, sizeof(g->Message), MSG(MISSING_NODE), XmlDB, Xfile);
-					goto error;
-				} // endif DBnode
+        if (!(DBnode = Root->SelectSingleNode(g, tabpath))) {
+          // DB node does not exist yet; we cannot create it
+          // because we don't know where it should be placed
+          sprintf(g->Message, MSG(MISSING_NODE), XmlDB, Xfile);
+          goto error;
+          } // endif DBnode
 
-				if (!(TabNode = DBnode->AddChildNode(g, Tabname))) {
-					snprintf(g->Message, sizeof(g->Message), MSG(FAIL_ADD_NODE), Tabname);
-					goto error;
-				} // endif TabNode
+        if (!(TabNode = DBnode->AddChildNode(g, Tabname))) {
+          sprintf(g->Message, MSG(FAIL_ADD_NODE), Tabname);
+          goto error;
+          } // endif TabNode
 
-				DBnode->AddText(g, "\n");
-			} else {
-				TabNode = Root;              // Try this ?
-				Tabname = TabNode->GetName(g);
-			} // endif's
+        DBnode->AddText(g, "\n");
+      } else
+        TabNode = Root;              // Try this ?
 
-		} else if (rc == RC_NF || rc == RC_EF) {
-			// The XML file does not exist or is void
-			if (Mode == MODE_INSERT) {
-				// New Document
-				char buf[64];
+    } else if (rc == RC_NF || rc == RC_EF) {
+      // The XML file does not exist or is void
+      if (Mode == MODE_INSERT) {
+        // New Document
+        char buf[64];
 
-				// Create the XML node
-				if (Docp->NewDoc(g, "1.0")) {
-					strcpy(g->Message, MSG(NEW_DOC_FAILED));
-					goto error;
-				} // endif NewDoc
+        // Create the XML node
+        if (Docp->NewDoc(g, "1.0")) {
+          strcpy(g->Message, MSG(NEW_DOC_FAILED));
+          goto error;
+          } // endif NewDoc
 
-			//  Now we can link the Xblock
-				To_Xb = Docp->LinkXblock(g, Mode, rc, filename);
+        //  Now we can link the Xblock
+        To_Xb = Docp->LinkXblock(g, Mode, rc, filename);
 
-				// Add a CONNECT comment node
-				strcpy(buf, " Created by the MariaDB CONNECT Storage Engine");
-				Docp->AddComment(g, buf);
+        // Add a CONNECT comment node
+        strcpy(buf, " Created by the MariaDB CONNECT Storage Engine");
+        Docp->AddComment(g, buf);
 
-				if (XmlDB) {
-					// This is a multi-table file
-					DBnode = Root = Docp->NewRoot(g, XmlDB);
-					DBnode->AddText(g, "\n");
-					TabNode = DBnode->AddChildNode(g, Tabname);
-					DBnode->AddText(g, "\n");
-				} else
-					TabNode = Root = Docp->NewRoot(g, Tabname);
+        if (XmlDB) {
+          // This is a multi-table file
+          DBnode = Root = Docp->NewRoot(g, XmlDB);
+          DBnode->AddText(g, "\n");
+          TabNode = DBnode->AddChildNode(g, Tabname);
+          DBnode->AddText(g, "\n");
+        } else
+          TabNode = Root = Docp->NewRoot(g, Tabname);
 
-				if (TabNode == NULL || Root == NULL) {
-					strcpy(g->Message, MSG(XML_INIT_ERROR));
-					goto error;
-				} else if (SetTabNode(g))
-					goto error;
+        if (TabNode == NULL || Root == NULL) {
+          strcpy(g->Message, MSG(XML_INIT_ERROR));
+          goto error;
+        } else if (SetTabNode(g))
+          goto error;
 
-			} else {
-				snprintf(g->Message, sizeof(g->Message), MSG(FILE_UNFOUND), Xfile);
+      } else {
+        sprintf(g->Message, MSG(FILE_UNFOUND), Xfile);
 
-				if (Mode == MODE_READ) {
-					PushWarning(g, this);
-					Void = true;
-				} // endif Mode
+        if (Mode == MODE_READ) {
+          PushWarning(g, this);
+          Void = true;
+          } // endif Mode
 
-				goto error;
-			} // endif Mode
+        goto error;
+      } // endif Mode
 
-		} else if (rc == RC_INFO) {
-			// Loading failed
-			snprintf(g->Message, sizeof(g->Message), MSG(LOADING_FAILED), Xfile);
-			goto error;
-		} else // (rc == RC_FX)
-			goto error;
+    } else if (rc == RC_INFO) {
+      // Loading failed
+      sprintf(g->Message, MSG(LOADING_FAILED), Xfile);
+      goto error;
+    } else // (rc == RC_FX)
+      goto error;
 
-		if (!Rowname) {
-			for (PXNODE n = TabNode->GetChild(g); n; n = n->GetNext(g))
-				if (n->GetType() == XML_ELEMENT_NODE) {
-					Rowname = n->GetName(g);
-					break;
-				} // endif Type
+    // Get row node list
+    if (Rowname)
+      Nlist = TabNode->SelectNodes(g, Rowname);
+    else
+      Nlist = TabNode->GetChildElements(g);
 
-			if (!Rowname)
-				Rowname = TabNode->GetName(g);
-		} // endif Rowname
-
-			// Get row node list
-		if (strcmp(Rowname, Tabname))
-			Nlist = TabNode->SelectNodes(g, Rowname);
-		else
-			Nrow = 1;
-
-
-		Docp->SetNofree(true);       // For libxml2
-#if defined(_WIN32)
-	} catch (_com_error e) {
+    Docp->SetNofree(true);       // For libxml2
+#if defined(__WIN__)
+  } catch(_com_error e) {
     // We come here if a DOM command threw an error
     char   buf[128];
 
@@ -979,12 +942,12 @@ bool TDBXML::Initialize(PGLOBAL g)
                              buf, sizeof(buf), NULL, NULL);
 
     if (rc)
-      snprintf(g->Message, sizeof(g->Message), "%s: %s", MSG(COM_ERROR), buf);
+      sprintf(g->Message, "%s: %s", MSG(COM_ERROR), buf);
     else
-      snprintf(g->Message, sizeof(g->Message), "%s hr=%x", MSG(COM_ERROR), e.Error());
+      sprintf(g->Message, "%s hr=%x", MSG(COM_ERROR), e.Error());
 
     goto error;
-#endif   // _WIN32
+#endif   // __WIN__
 #if !defined(UNIX)
   } catch(...) {
     // Other errors
@@ -1138,7 +1101,7 @@ int TDBXML::RowNumber(PGLOBAL g, bool b)
     /*******************************************************************/
     /*  Don't know how to retrieve RowID for expanded XML tables.      */
     /*******************************************************************/
-    snprintf(g->Message, sizeof(g->Message), MSG(NO_ROWID_FOR_AM),
+    sprintf(g->Message, MSG(NO_ROWID_FOR_AM),
                         GetAmName(g, GetAmType()));
     return 0;          // Means error
   } else
@@ -1258,14 +1221,10 @@ int TDBXML::ReadDB(PGLOBAL g)
       htrc("TDBXML ReadDB: Irow=%d RowNode=%p\n", Irow, RowNode);
 
     // Get the new row node
-		if (Nlist) {
-			if ((RowNode = Nlist->GetItem(g, Irow, RowNode)) == NULL) {
-				snprintf(g->Message, sizeof(g->Message), MSG(MISSING_ROWNODE), Irow);
-				return RC_FX;
-			} // endif RowNode
-
-		} else
-			RowNode = TabNode;
+    if ((RowNode = Nlist->GetItem(g, Irow, RowNode)) == NULL) {
+      sprintf(g->Message, MSG(MISSING_ROWNODE), Irow);
+      return RC_FX;
+      } // endif RowNode
 
     if (Colname && Coltype == 2)
       Clist = RowNode->SelectNodes(g, Colname, Clist);
@@ -1320,12 +1279,11 @@ int TDBXML::WriteDB(PGLOBAL g)
 /***********************************************************************/
 int TDBXML::DeleteDB(PGLOBAL g, int irc)
   {
-	// TODO: Handle null Nlist
   if (irc == RC_FX) {
     // Delete all rows
     for (Irow = 0; Irow < Nrow; Irow++)
       if ((RowNode = Nlist->GetItem(g, Irow, RowNode)) == NULL) {
-        snprintf(g->Message, sizeof(g->Message), MSG(MISSING_ROWNODE), Irow);
+        sprintf(g->Message, MSG(MISSING_ROWNODE), Irow);
         return RC_FX;
       } else {
         TabNode->DeleteChild(g, RowNode);
@@ -1511,17 +1469,13 @@ bool XMLCOL::ParseXpath(PGLOBAL g, bool mode)
   if (!mode)
     // Take care of an eventual extra column node a la html
     if (Tdbp->Colname) {
-      char *p = strstr(Tdbp->Colname, "%d");
-      if (p)
-        snprintf(pbuf, len + 3, "%.*s%d%s/", (int) (p - Tdbp->Colname), Tdbp->Colname,
-            Rank + (Tdbp->Usedom ? 0 : 1), p + 2);
-      else
-        snprintf(pbuf, len + 3, "%s/", Tdbp->Colname);
-    } // endif Colname
+      sprintf(pbuf, Tdbp->Colname, Rank + ((Tdbp->Usedom) ? 0 : 1));
+      strcat(pbuf, "/");
+      } // endif Colname
 
   if (Xname) {
     if (Type == 2) {
-      snprintf(g->Message, sizeof(g->Message), MSG(BAD_COL_XPATH), Name, Tdbp->Name);
+      sprintf(g->Message, MSG(BAD_COL_XPATH), Name, Tdbp->Name);
       return true;
     } else
       strcat(pbuf, Xname);
@@ -1551,7 +1505,7 @@ bool XMLCOL::ParseXpath(PGLOBAL g, bool mode)
       if (mode) {
         // For Update or Insert the Xpath must be explicit
         if (strchr("@/.*", *p)) {
-          snprintf(g->Message, sizeof(g->Message), MSG(XPATH_NOT_SUPP), Name);
+          sprintf(g->Message, MSG(XPATH_NOT_SUPP), Name);
           return true;
         } else
           Nodes[i] = p;
@@ -1562,7 +1516,7 @@ bool XMLCOL::ParseXpath(PGLOBAL g, bool mode)
       } // endfor i, p
 
     if (*p == '/' || *p == '.') {
-      snprintf(g->Message, sizeof(g->Message), MSG(XPATH_NOT_SUPP), Name);
+      sprintf(g->Message, MSG(XPATH_NOT_SUPP), Name);
       return true;
     } else if (*p == '@') {
       p++;                           // Remove the @ if mode
@@ -1623,7 +1577,7 @@ bool XMLCOL::ParseXpath(PGLOBAL g, bool mode)
 bool XMLCOL::SetBuffer(PGLOBAL g, PVAL value, bool ok, bool check)
   {
   if (!(To_Val = value)) {
-    snprintf(g->Message, sizeof(g->Message), MSG(VALUE_ERROR), Name);
+    sprintf(g->Message, MSG(VALUE_ERROR), Name);
     return true;
   } else if (Buf_Type == value->GetType()) {
     // Values are of the (good) column type
@@ -1642,7 +1596,7 @@ bool XMLCOL::SetBuffer(PGLOBAL g, PVAL value, bool ok, bool check)
   } else {
     // Values are not of the (good) column type
     if (check) {
-      snprintf(g->Message, sizeof(g->Message), MSG(TYPE_VALUE_ERR), Name,
+      sprintf(g->Message, MSG(TYPE_VALUE_ERR), Name,
               GetTypeName(Buf_Type), GetTypeName(value->GetType()));
       return true;
       } // endif check
@@ -1695,7 +1649,7 @@ void XMLCOL::ReadColumn(PGLOBAL g)
   if (ValNode) {
     if (ValNode->GetType() != XML_ELEMENT_NODE &&
         ValNode->GetType() != XML_ATTRIBUTE_NODE) {
-      snprintf(g->Message, sizeof(g->Message), MSG(BAD_VALNODE), ValNode->GetType(), Name);
+      sprintf(g->Message, MSG(BAD_VALNODE), ValNode->GetType(), Name);
 			throw (int)TYPE_AM_XML;
 		} // endif type
 
@@ -1803,9 +1757,6 @@ void XMLCOL::WriteColumn(PGLOBAL g)
     else if (Tdbp->Clist)
       ColNode = NULL;
 
-    // refresh CList in case its Listp was freed in SelectSingleNode above
-    if (Tdbp->Clist)
-      Tdbp->RowNode->SelectNodes(g, Tdbp->Colname, Tdbp->Clist);
     } // endfor i
 
   /*********************************************************************/
@@ -1861,7 +1812,7 @@ void XMLCOL::WriteColumn(PGLOBAL g)
   p = Value->GetCharString(buf);
 
   if (strlen(p) > (unsigned)Long) {
-    snprintf(g->Message, sizeof(g->Message), MSG(VALUE_TOO_LONG), p, Name, Long);
+    sprintf(g->Message, MSG(VALUE_TOO_LONG), p, Name, Long);
 		throw (int)TYPE_AM_XML;
 	} else
     strcpy(Valbuf, p);
@@ -1902,7 +1853,7 @@ void XMULCOL::ReadColumn(PGLOBAL g)
 
       if (N > Tdbp->Limit) {
         N = Tdbp->Limit;
-        snprintf(g->Message, sizeof(g->Message), "Multiple values limited to %d", Tdbp->Limit);
+        sprintf(g->Message, "Multiple values limited to %d", Tdbp->Limit);
         PushWarning(g, Tdbp);
         } // endif N
 
@@ -1911,7 +1862,7 @@ void XMULCOL::ReadColumn(PGLOBAL g)
 
         if (ValNode->GetType() != XML_ELEMENT_NODE &&
             ValNode->GetType() != XML_ATTRIBUTE_NODE) {
-          snprintf(g->Message, sizeof(g->Message), MSG(BAD_VALNODE), ValNode->GetType(), Name);
+          sprintf(g->Message, MSG(BAD_VALNODE), ValNode->GetType(), Name);
 					throw (int)TYPE_AM_XML;
 				} // endif type
 
@@ -2046,7 +1997,7 @@ void XMULCOL::WriteColumn(PGLOBAL g)
         len = Nlx->GetLength();
 
         if (len > 1 && !Tdbp->Xpand) {
-          snprintf(g->Message, sizeof(g->Message), MSG(BAD_VAL_UPDATE), Name);
+          sprintf(g->Message, MSG(BAD_VAL_UPDATE), Name);
 					throw (int)TYPE_AM_XML;
 				} else
           ValNode = Nlx->GetItem(g, Tdbp->Nsub, Vxnp);
@@ -2117,7 +2068,7 @@ void XMULCOL::WriteColumn(PGLOBAL g)
   p = Value->GetCharString(buf);
 
   if (strlen(p) > (unsigned)Long) {
-    snprintf(g->Message, sizeof(g->Message), MSG(VALUE_TOO_LONG), p, Name, Long);
+    sprintf(g->Message, MSG(VALUE_TOO_LONG), p, Name, Long);
 		throw (int)TYPE_AM_XML;
 	} else
     strcpy(Valbuf, p);
@@ -2244,7 +2195,7 @@ void XPOSCOL::WriteColumn(PGLOBAL g)
   p = Value->GetCharString(buf);
 
   if (strlen(p) > (unsigned)Long) {
-    snprintf(g->Message, sizeof(g->Message), MSG(VALUE_TOO_LONG), p, Name, Long);
+    sprintf(g->Message, MSG(VALUE_TOO_LONG), p, Name, Long);
 		throw (int)TYPE_AM_XML;
 	} else
     strcpy(Valbuf, p);
