@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2013, 2022, MariaDB Corporation.
+Copyright (c) 2013, 2018, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -13,7 +13,7 @@ FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
-51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -32,10 +32,6 @@ Created 10/25/1995 Heikki Tuuri
 #include "log0recv.h"
 #include "dict0types.h"
 #include "page0size.h"
-#include "ilist.h"
-
-struct unflushed_spaces_tag_t;
-struct rotation_list_tag_t;
 
 // Forward declaration
 extern my_bool srv_use_doublewrite_buf;
@@ -77,12 +73,9 @@ fil_type_is_data(
 struct fil_node_t;
 
 /** Tablespace or log data space */
-struct fil_space_t : ilist_node<unflushed_spaces_tag_t>,
-                     ilist_node<rotation_list_tag_t>
-{
-	ulint		id;	/*!< space id */
-	hash_node_t	hash;	/*!< hash chain node */
+struct fil_space_t {
 	char*		name;	/*!< Tablespace name */
+	ulint		id;	/*!< space id */
 	lsn_t		max_lsn;
 				/*!< LSN of the most recent
 				fil_names_write_if_was_clean().
@@ -90,16 +83,25 @@ struct fil_space_t : ilist_node<unflushed_spaces_tag_t>,
 				Protected by log_sys.mutex.
 				If and only if this is nonzero, the
 				tablespace will be in named_spaces. */
-	/** Log sequence number of the latest MLOG_INDEX_LOAD record
-	that was found while parsing the redo log */
-	lsn_t		enable_lsn;
+	bool		stop_new_ops;
+				/*!< we set this true when we start
+				deleting a single-table tablespace.
+				When this is set following new ops
+				are not allowed:
+				* read IO request
+				* ibuf merge
+				* file flush
+				Note that we can still possibly have
+				new write operations because we don't
+				check this flag when doing flush
+				batches. */
 	/** whether undo tablespace truncation is in progress */
 	bool		is_being_truncated;
 #ifdef UNIV_DEBUG
 	ulint		redo_skipped_count;
 				/*!< reference count for operations who want
 				to skip redo log in the file space in order
-				to make modify_check() pass.
+				to make fsp_space_modify_check pass.
 				Uses my_atomic_loadlint() and friends. */
 #endif
 	fil_type_t	purpose;/*!< purpose */
@@ -118,8 +120,6 @@ struct fil_space_t : ilist_node<unflushed_spaces_tag_t>,
 				/*!< recovered tablespace size in pages;
 				0 if no size change was read from the redo log,
 				or if the size change was implemented */
-  /** the committed size of the tablespace in pages */
-  ulint committed_size;
 	ulint		flags;	/*!< FSP_SPACE_FLAGS and FSP_FLAGS_MEM_ flags;
 				see fsp0types.h,
 				fsp_flags_is_valid(),
@@ -130,24 +130,14 @@ struct fil_space_t : ilist_node<unflushed_spaces_tag_t>,
 	ulint		n_pending_flushes; /*!< this is positive when flushing
 				the tablespace to disk; dropping of the
 				tablespace is forbidden if this is positive */
-private:
-  /** Number of pending buffer pool operations accessing the
-  tablespace without holding a table lock or dict_operation_lock
-  S-latch that would prevent the table (and tablespace) from being
-  dropped. An example is change buffer merge.
-
-  The tablespace cannot be dropped while this is nonzero, or while
-  fil_node_t::n_pending is nonzero.
-
-  The most significant bit contains the STOP_NEW_OPS flag.
-
-  Protected by my_atomic. */
-  int32 n_pending_ops;
-
-  /** Flag in n_pending_ops that indicates that the tablespace is being
-  deleted, and no further operations should be performed */
-  static const int32 STOP_NEW_OPS= 1 << 31;
-public:
+	/** Number of pending buffer pool operations accessing the tablespace
+	without holding a table lock or dict_operation_lock S-latch
+	that would prevent the table (and tablespace) from being
+	dropped. An example is change buffer merge.
+	The tablespace cannot be dropped while this is nonzero,
+	or while fil_node_t::n_pending is nonzero.
+	Protected by fil_system.mutex and my_atomic_loadlint() and friends. */
+	ulint		n_pending_ops;
 	/** Number of pending block read or write operations
 	(when a write is imminent or a read has recently completed).
 	The tablespace object cannot be freed while this is nonzero,
@@ -155,23 +145,29 @@ public:
 	Note that fil_node_t::n_pending tracks actual pending I/O requests.
 	Protected by fil_system.mutex and my_atomic_loadlint() and friends. */
 	ulint		n_pending_ios;
+	hash_node_t	hash;	/*!< hash chain node */
+	hash_node_t	name_hash;/*!< hash chain the name_hash table */
 	rw_lock_t	latch;	/*!< latch protecting the file space storage
 				allocation */
+	UT_LIST_NODE_T(fil_space_t) unflushed_spaces;
+				/*!< list of spaces with at least one unflushed
+				file we have written to */
 	UT_LIST_NODE_T(fil_space_t) named_spaces;
 				/*!< list of spaces for which MLOG_FILE_NAME
 				records have been issued */
+	bool		is_in_unflushed_spaces;
+				/*!< true if this space is currently in
+				unflushed_spaces */
 	UT_LIST_NODE_T(fil_space_t) space_list;
 				/*!< list of all spaces */
+	/** other tablespaces needing key rotation */
+	UT_LIST_NODE_T(fil_space_t) rotation_list;
+	/** whether this tablespace needs key rotation */
+	bool		is_in_rotation_list;
 
 	/** MariaDB encryption data */
 	fil_space_crypt_t* crypt_data;
 
-	/** Checks that this tablespace in a list of unflushed tablespaces. */
-	bool is_in_unflushed_spaces;
-
-	/** Checks that this tablespace needs key rotation. */
-	bool is_in_default_encrypt;
-
 	/** True if the device this filespace is on supports atomic writes */
 	bool		atomic_write_supported;
 
@@ -181,14 +177,8 @@ public:
 
 	ulint		magic_n;/*!< FIL_SPACE_MAGIC_N */
 
-  /** Clamp a page number for batched I/O, such as read-ahead.
-  @param offset   page number limit
-  @return offset clamped to the tablespace size */
-  ulint max_page_number_for_io(ulint offset) const
-  {
-    const ulint limit= committed_size;
-    return limit > offset ? offset : limit;
-  }
+	/** @return whether the tablespace is about to be dropped */
+	bool is_stopping() const { return stop_new_ops;	}
 
 	/** @return whether doublewrite buffering is needed */
 	bool use_doublewrite() const
@@ -209,12 +199,6 @@ public:
 	fil_node_t* add(const char* name, pfs_os_file_t handle,
 			ulint size, bool is_raw, bool atomic_write,
 			ulint max_pages = ULINT_MAX);
-#ifdef UNIV_DEBUG
-	/** Assert that the mini-transaction is compatible with
-	updating an allocation bitmap page.
-	@param[in]	mtr	mini-transaction */
-	void modify_check(const mtr_t& mtr) const;
-#endif /* UNIV_DEBUG */
 
 	/** Try to reserve free extents.
 	@param[in]	n_free_now	current number of free extents
@@ -262,78 +246,37 @@ public:
 	/** Close each file. Only invoked on fil_system.temp_space. */
 	void close();
 
-  /** @return whether the tablespace is about to be dropped or is referenced */
-  int32 is_stopping_or_referenced()
-  {
-    return my_atomic_load32(&n_pending_ops);
-  }
+	/** Acquire a tablespace reference. */
+	void acquire() { my_atomic_addlint(&n_pending_ops, 1); }
+	/** Release a tablespace reference. */
+	void release()
+	{
+		ut_ad(referenced());
+		my_atomic_addlint(&n_pending_ops, ulint(-1));
+	}
+	/** @return whether references are being held */
+	bool referenced() { return my_atomic_loadlint(&n_pending_ops); }
+	/** @return whether references are being held */
+	bool referenced() const
+	{
+		return const_cast<fil_space_t*>(this)->referenced();
+	}
 
-  /** @return whether the tablespace is about to be dropped or is referenced */
-  int32 is_stopping_or_referenced() const
-  {
-    return const_cast<fil_space_t*>(this)->is_stopping_or_referenced();
-  }
-
-  /** @return whether the tablespace is about to be dropped */
-  bool is_stopping() const
-  {
-    return is_stopping_or_referenced() & STOP_NEW_OPS;
-  }
-
-  /** @return number of references being held */
-  int32 referenced() const
-  {
-    return is_stopping_or_referenced() & ~STOP_NEW_OPS;
-  }
-
-  /** Note that operations on the tablespace must stop or can resume */
-  void set_stopping(bool stopping)
-  {
-    /* Note: starting with 10.4 this should be std::atomic::fetch_xor() */
-    int32 n= stopping ? 0 : STOP_NEW_OPS;
-    while (!my_atomic_cas32_strong_explicit(&n_pending_ops, &n,
-                                            n ^ STOP_NEW_OPS,
-                                            MY_MEMORY_ORDER_ACQUIRE,
-                                            MY_MEMORY_ORDER_RELAXED))
-      ut_ad(!(n & STOP_NEW_OPS) == stopping);
-  }
-
-  MY_ATTRIBUTE((warn_unused_result))
-  /** @return whether a tablespace reference was successfully acquired */
-  bool acquire()
-  {
-    int32 n= 0;
-    while (!my_atomic_cas32_strong_explicit(&n_pending_ops, &n, n + 1,
-                                            MY_MEMORY_ORDER_ACQUIRE,
-                                            MY_MEMORY_ORDER_RELAXED))
-      if (UNIV_UNLIKELY(n & STOP_NEW_OPS))
-        return false;
-    return true;
-  }
-  /** Release a tablespace reference.
-  @return whether this was the last reference */
-  bool release()
-  {
-    int32 n= my_atomic_add32(&n_pending_ops, -1);
-    ut_ad(n & ~STOP_NEW_OPS);
-    return (n & ~STOP_NEW_OPS) == 1;
-  }
-
-  /** Acquire a tablespace reference for I/O. */
-  void acquire_for_io() { my_atomic_addlint(&n_pending_ios, 1); }
-  /** Release a tablespace reference for I/O. */
-  void release_for_io()
-  {
-    ut_d(ulint n=) my_atomic_addlint(&n_pending_ios, ulint(-1));
-    ut_ad(n);
-  }
-  /** @return whether I/O is pending */
-  bool pending_io() { return my_atomic_loadlint(&n_pending_ios); }
-  /** @return whether I/O is pending */
-  bool pending_io() const
-  {
-    return const_cast<fil_space_t*>(this)->pending_io();
-  }
+	/** Acquire a tablespace reference for I/O. */
+	void acquire_for_io() { my_atomic_addlint(&n_pending_ios, 1); }
+	/** Release a tablespace reference for I/O. */
+	void release_for_io()
+	{
+		ut_ad(pending_io());
+		my_atomic_addlint(&n_pending_ios, ulint(-1));
+	}
+	/** @return whether I/O is pending */
+	bool pending_io() { return my_atomic_loadlint(&n_pending_ios); }
+	/** @return whether I/O is pending */
+	bool pending_io() const
+	{
+		return const_cast<fil_space_t*>(this)->pending_io();
+	}
 };
 
 /** Value of fil_space_t::magic_n */
@@ -347,6 +290,10 @@ struct fil_node_t {
 	char*		name;
 	/** file handle (valid if is_open) */
 	pfs_os_file_t	handle;
+	/** event that groups and serializes calls to fsync;
+	os_event_set() and os_event_reset() are protected by
+	fil_system.mutex */
+	os_event_t	sync_event;
 	/** whether the file actually is a raw device or disk partition */
 	bool		is_raw_disk;
 	/** size of the file in database pages (0 if not known yet);
@@ -364,8 +311,10 @@ struct fil_node_t {
 	ulint		n_pending_flushes;
 	/** whether the file is currently being extended */
 	bool		being_extended;
-	/** whether this file had writes after lasy fsync() */
-	bool		needs_flush;
+	/** number of writes to the file since the system was started */
+	int64_t		modification_counter;
+	/** the modification_counter of the latest flush to disk */
+	int64_t		flush_counter;
 	/** link to other files in this tablespace */
 	UT_LIST_NODE_T(fil_node_t) chain;
 	/** link to the fil_system.LRU list (keeping track of open files) */
@@ -398,7 +347,7 @@ struct fil_node_t {
 /** Value of fil_node_t::magic_n */
 #define	FIL_NODE_MAGIC_N	89389
 
-/** Common InnoDB file extensions */
+/** Common InnoDB file extentions */
 enum ib_extention {
 	NO_EXT = 0,
 	IBD = 1,
@@ -627,6 +576,8 @@ struct fil_system_t {
   {
     UT_LIST_INIT(LRU, &fil_node_t::LRU);
     UT_LIST_INIT(space_list, &fil_space_t::space_list);
+    UT_LIST_INIT(rotation_list, &fil_space_t::rotation_list);
+    UT_LIST_INIT(unflushed_spaces, &fil_space_t::unflushed_spaces);
     UT_LIST_INIT(named_spaces, &fil_space_t::named_spaces);
   }
 
@@ -662,13 +613,15 @@ public:
 					not put to this list: they are opened
 					after the startup, and kept open until
 					shutdown */
-	sized_ilist<fil_space_t, unflushed_spaces_tag_t> unflushed_spaces;
-					/*!< list of those
+	UT_LIST_BASE_NODE_T(fil_space_t) unflushed_spaces;
+					/*!< base node for the list of those
 					tablespaces whose files contain
 					unflushed writes; those spaces have
 					at least one file node where
-					needs_flush == true */
+					modification_counter > flush_counter */
 	ulint		n_open;		/*!< number of files currently open */
+	int64_t		modification_counter;/*!< when we write to a file we
+					increment this by one */
 	ulint		max_assigned_id;/*!< maximum space id in the existing
 					tables, or assigned during the time
 					mysqld has been up; at an InnoDB
@@ -683,30 +636,14 @@ public:
 					record has been written since
 					the latest redo log checkpoint.
 					Protected only by log_sys.mutex. */
-
-	/** List of all file spaces need key rotation */
-	ilist<fil_space_t, rotation_list_tag_t> default_encrypt_tables;
+	UT_LIST_BASE_NODE_T(fil_space_t) rotation_list;
+					/*!< list of all file spaces needing
+					key rotation.*/
 
 	bool		space_id_reuse_warned;
 					/*!< whether fil_space_create()
 					has issued a warning about
 					potential space_id reuse */
-
-	/** Trigger a call to fil_node_t::read_page0()
-	@param[in]	id	tablespace identifier
-	@return	tablespace
-	@retval	NULL	if the tablespace does not exist or cannot be read */
-	fil_space_t* read_page0(ulint id);
-
-  /** Return the next tablespace from default_encrypt_tables list.
-  @param space   previous tablespace (NULL to start from the start)
-  @param recheck whether the removal condition needs to be rechecked after
-  the encryption parameters were changed
-  @param encrypt expected state of innodb_encrypt_tables
-  @return the next tablespace to process (n_pending_ops incremented)
-  @retval NULL if this was the last */
-  inline fil_space_t* default_encrypt_next(
-    fil_space_t *space, bool recheck, bool encrypt);
 };
 
 /** The tablespace memory cache. */
@@ -714,6 +651,15 @@ extern fil_system_t	fil_system;
 
 #include "fil0crypt.h"
 
+/** Returns the latch of a file space.
+@param[in]	id	space id
+@param[out]	flags	tablespace flags
+@return latch protecting storage allocation */
+rw_lock_t*
+fil_space_get_latch(
+	ulint	id,
+	ulint*	flags);
+
 /** Create a space memory object and put it to the fil_system hash table.
 Error messages are issued to the server log.
 @param[in]	name		tablespace name
@@ -794,8 +740,9 @@ database server shutdown. This should be called at a server startup after the
 space objects for the log and the system tablespace have been created. The
 purpose of this operation is to make sure we never run out of file descriptors
 if we need to read from the insert buffer or to write to the log. */
-dberr_t fil_open_log_and_system_tablespace_files()
-	MY_ATTRIBUTE((warn_unused_result));
+void
+fil_open_log_and_system_tablespace_files(void);
+/*==========================================*/
 /*******************************************************************//**
 Closes all open files. There must not be any pending i/o's or not flushed
 modifications in the files. */
@@ -872,6 +819,32 @@ when it could be dropped concurrently.
 fil_space_t*
 fil_space_acquire_for_io(ulint id);
 
+/** Return the next fil_space_t.
+Once started, the caller must keep calling this until it returns NULL.
+fil_space_acquire() and fil_space_t::release() are invoked here which
+blocks a concurrent operation from dropping the tablespace.
+@param[in,out]	prev_space	Pointer to the previous fil_space_t.
+If NULL, use the first fil_space_t on fil_system.space_list.
+@return pointer to the next fil_space_t.
+@retval NULL if this was the last  */
+fil_space_t*
+fil_space_next(
+	fil_space_t*	prev_space)
+	MY_ATTRIBUTE((warn_unused_result));
+
+/** Return the next fil_space_t from key rotation list.
+Once started, the caller must keep calling this until it returns NULL.
+fil_space_acquire() and fil_space_t::release() are invoked here which
+blocks a concurrent operation from dropping the tablespace.
+@param[in,out]	prev_space	Pointer to the previous fil_space_t.
+If NULL, use the first fil_space_t on fil_system.space_list.
+@return pointer to the next fil_space_t.
+@retval NULL if this was the last*/
+fil_space_t*
+fil_space_keyrotate_next(
+	fil_space_t*	prev_space)
+	MY_ATTRIBUTE((warn_unused_result));
+
 /********************************************************//**
 Creates the database directory for a table if it does not exist yet. */
 void
@@ -903,14 +876,20 @@ but only by InnoDB table locks, which may be broken by
 lock_remove_all_on_table().)
 @param[in]	table	persistent table
 checked @return whether the table is accessible */
-bool fil_table_accessible(const dict_table_t* table)
+bool
+fil_table_accessible(const dict_table_t* table)
 	MY_ATTRIBUTE((warn_unused_result, nonnull));
 
 /** Delete a tablespace and associated .ibd file.
 @param[in]	id		tablespace identifier
-@param[in]	if_exists	whether to ignore missing tablespace
 @return	DB_SUCCESS or error */
-dberr_t fil_delete_tablespace(ulint id, bool if_exists= false);
+dberr_t
+fil_delete_tablespace(
+	ulint id
+#ifdef BTR_CUR_HASH_ADAPT
+	, bool drop_ahi = false /*!< whether to drop the adaptive hash index */
+#endif /* BTR_CUR_HASH_ADAPT */
+	);
 
 /** Prepare to truncate an undo tablespace.
 @param[in]	space_id	undo tablespace id
@@ -1063,6 +1042,9 @@ memory cache. Note that if we have not done a crash recovery at the database
 startup, there may be many tablespaces which are not yet in the memory cache.
 @param[in]	id		Tablespace ID
 @param[in]	name		Tablespace name used in fil_space_create().
+@param[in]	print_error_if_does_not_exist
+				Print detailed error information to the
+error log if a matching tablespace is not found from memory.
 @param[in]	table_flags	table flags
 @return the tablespace
 @retval	NULL	if no matching tablespace exists in the memory cache */
@@ -1070,6 +1052,7 @@ fil_space_t*
 fil_space_for_table_exists_in_mem(
 	ulint		id,
 	const char*	name,
+	bool		print_error_if_does_not_exist,
 	ulint		table_flags);
 
 /** Try to extend a tablespace if it is smaller than the specified size.
@@ -1275,12 +1258,16 @@ fil_names_write_if_was_clean(
 	return(was_clean);
 }
 
+extern volatile bool	recv_recovery_on;
+
 /** During crash recovery, open a tablespace if it had not been opened
 yet, to get valid size and flags.
 @param[in,out]	space	tablespace */
-inline void fil_space_open_if_needed(fil_space_t* space)
+inline
+void
+fil_space_open_if_needed(
+	fil_space_t*	space)
 {
-	ut_d(extern volatile bool recv_recovery_on);
 	ut_ad(recv_recovery_on);
 
 	if (space->size == 0) {
@@ -1288,7 +1275,10 @@ inline void fil_space_open_if_needed(fil_space_t* space)
 		until the files are opened for the first time.
 		fil_space_get_size() will open the file
 		and adjust the size and flags. */
-		ut_d(ulint size	=) fil_space_get_size(space->id);
+#ifdef UNIV_DEBUG
+		ulint		size	=
+#endif /* UNIV_DEBUG */
+			fil_space_get_size(space->id);
 		ut_ad(size == space->size);
 	}
 }
@@ -1317,7 +1307,7 @@ UNIV_INTERN
 ulint
 fil_space_get_block_size(const fil_space_t* space, unsigned offset);
 
-#include "fil0fil.inl"
+#include "fil0fil.ic"
 #endif /* UNIV_INNOCHECKSUM */
 
 #endif /* fil0fil_h */
