@@ -2,7 +2,7 @@
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All rights reserved.
 Copyright (c) 2009, Google Inc.
-Copyright (c) 2017, 2021, MariaDB Corporation.
+Copyright (c) 2017, 2018, MariaDB Corporation.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -20,7 +20,7 @@ FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
-51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -40,10 +40,6 @@ Created 12/9/1995 Heikki Tuuri
 #include "os0event.h"
 #include "os0file.h"
 
-#ifndef UINT32_MAX
-#define UINT32_MAX             (4294967295U)
-#endif
-
 /** Maximum number of srv_n_log_files, or innodb_log_files_in_group */
 #define SRV_N_LOG_FILES_MAX 100
 
@@ -56,6 +52,12 @@ step which modifies the database, is started */
 #define LOG_CHECKPOINT_FREE_PER_THREAD	(4U << srv_page_size_shift)
 #define LOG_CHECKPOINT_EXTRA_FREE	(8U << srv_page_size_shift)
 
+typedef ulint (*log_checksum_func_t)(const byte* log_block);
+
+/** Pointer to the log checksum calculation function. Protected with
+log_sys.mutex. */
+extern log_checksum_func_t log_checksum_algorithm_ptr;
+
 /** Append a string to the log.
 @param[in]	str		string
 @param[in]	len		string length
@@ -174,15 +176,9 @@ log_write_up_to(
 /** write to the log file up to the last log entry.
 @param[in]	sync	whether we want the written log
 also to be flushed to disk. */
-void log_buffer_flush_to_disk(bool sync= true);
-
-
-/** Prepare to invoke log_write_and_flush(), before acquiring log_sys.mutex. */
-#define log_write_and_flush_prepare() log_write_mutex_enter()
-
-/** Durably write the log up to log_sys.lsn and release log_sys.mutex. */
-ATTRIBUTE_COLD void log_write_and_flush();
-
+void
+log_buffer_flush_to_disk(
+	bool sync = true);
 /****************************************************************//**
 This functions writes the log buffer to the log file and if 'flush'
 is set it forces a flush of the log file as well. This is meant to be
@@ -195,13 +191,25 @@ log_buffer_sync_in_background(
 /** Make a checkpoint. Note that this function does not flush dirty
 blocks from the buffer pool: it only checks what is lsn of the oldest
 modification in the pool, and writes information about the lsn in
-log files. Use log_make_checkpoint() to flush also the pool.
+log files. Use log_make_checkpoint_at() to flush also the pool.
 @param[in]	sync		whether to wait for the write to complete
+@param[in]	write_always	force a write even if no log
+has been generated since the latest checkpoint
 @return true if success, false if a checkpoint write was already running */
-bool log_checkpoint(bool sync);
+bool
+log_checkpoint(
+	bool	sync,
+	bool	write_always);
 
-/** Make a checkpoint */
-void log_make_checkpoint();
+/** Make a checkpoint at or after a specified LSN.
+@param[in]	lsn		the log sequence number, or LSN_MAX
+for the latest LSN
+@param[in]	write_always	force a write even if no log
+has been generated since the latest checkpoint */
+void
+log_make_checkpoint_at(
+	lsn_t			lsn,
+	bool			write_always);
 
 /****************************************************************//**
 Makes a checkpoint at the latest lsn and writes it to first page of each
@@ -266,6 +274,14 @@ log_block_set_data_len(
 /*===================*/
 	byte*	log_block,	/*!< in/out: log block */
 	ulint	len);		/*!< in: data length */
+/************************************************************//**
+Calculates the checksum for a log block.
+@return checksum */
+UNIV_INLINE
+ulint
+log_block_calc_checksum(
+/*====================*/
+	const byte*	block);	/*!< in: log block */
 
 /** Calculates the checksum for a log block using the CRC32 algorithm.
 @param[in]	block	log block
@@ -275,6 +291,12 @@ ulint
 log_block_calc_checksum_crc32(
 	const byte*	block);
 
+/** Calculates the checksum for a log block using the "no-op" algorithm.
+@return		the calculated checksum value */
+UNIV_INLINE
+ulint
+log_block_calc_checksum_none(const byte*);
+
 /************************************************************//**
 Gets a log block checksum field value.
 @return checksum */
@@ -351,7 +373,7 @@ void
 log_refresh_stats(void);
 /*===================*/
 
-/** Whether to require checksums on the redo log pages */
+/** Whether to generate and require checksums on the redo log pages */
 extern my_bool	innodb_log_checksums;
 
 /* Values used as flags */
@@ -481,12 +503,6 @@ Stored in LOG_HEADER_FORMAT. */
 					header */
 #define LOG_FILE_HDR_SIZE	(4 * OS_FILE_LOG_BLOCK_SIZE)
 
-/* As long as fil_io() is used to handle log io, log group max size is limited
-by (maximum page number) * (minimum page size). Page number type is uint32_t.
-Remove this limitation if page number is no longer used for log file io. */
-static const ulonglong log_group_max_size =
-	((ulonglong(UINT32_MAX) + 1) * UNIV_PAGE_SIZE_MIN - 1);
-
 typedef ib_mutex_t	LogSysMutex;
 typedef ib_mutex_t	FlushOrderMutex;
 
@@ -510,11 +526,19 @@ struct log_t{
 					mtr_commit and still ensure that
 					insertions in the flush_list happen
 					in the LSN order. */
-	/** log_buffer, append data here */
-	byte*		buf;
-	/** log_buffer, writing data to file from this buffer.
-	Before flushing write_buf is swapped with flush_buf */
-	byte*		flush_buf;
+	byte*		buf;		/*!< Memory of double the
+					srv_log_buffer_size is
+					allocated here. This pointer will change
+					however to either the first half or the
+					second half in turns, so that log
+					write/flush to disk don't block
+					concurrent mtrs which will write
+					log to this buffer. Care to switch back
+					to the first half before freeing/resizing
+					must be undertaken. */
+	bool		first_in_use;	/*!< true if buf points to the first
+					half of the aligned(buf_ptr), false
+					if the second half */
 	ulong		max_buf_free;	/*!< recommended maximum value of
 					buf_free for the buffer in use, after
 					which the buffer is flushed */
@@ -539,12 +563,16 @@ struct log_t{
     uint32_t				subformat;
     /** individual log file size in bytes, including the header */
     lsn_t				file_size;
-  private:
     /** lsn used to fix coordinates within the log group */
     lsn_t				lsn;
     /** the byte offset of the above lsn */
     lsn_t				lsn_offset;
-  public:
+
+    /** unaligned buffers */
+    byte*				file_header_bufs_ptr;
+    /** buffers for each file header in the group */
+    byte*				file_header_bufs[SRV_N_LOG_FILES_MAX];
+
     /** used only in recovery: recovery scan succeeded up to this
     lsn in this log group */
     lsn_t				scanned_lsn;
@@ -561,9 +589,8 @@ struct log_t{
     /** Set the field values to correspond to a given lsn. */
     void set_fields(lsn_t lsn)
     {
-      lsn_t c_lsn_offset = calc_lsn_offset(lsn);
-      set_lsn(lsn);
-      set_lsn_offset(c_lsn_offset);
+      lsn_offset = calc_lsn_offset(lsn);
+      this->lsn = lsn;
     }
 
     /** Read a log segment to log_sys.buf.
@@ -580,12 +607,11 @@ struct log_t{
     /** Close the redo log buffer. */
     void close()
     {
+      ut_free(file_header_bufs_ptr);
       n_files = 0;
+      file_header_bufs_ptr = NULL;
+      memset(file_header_bufs, 0, sizeof file_header_bufs);
     }
-    void set_lsn(lsn_t a_lsn);
-    lsn_t get_lsn() const { return lsn; }
-    void set_lsn_offset(lsn_t a_lsn);
-    lsn_t get_lsn_offset() const { return lsn_offset; }
   } log;
 
 	/** The fields involved in the log buffer flush @{ */
@@ -597,6 +623,8 @@ struct log_t{
 					later; this is advanced when a flush
 					operation is completed to all the log
 					groups */
+	volatile bool	is_extending;	/*!< this is set to true during extend
+					the log buffer size */
 	lsn_t		write_lsn;	/*!< last written lsn */
 	lsn_t		current_flush_lsn;/*!< end lsn for the current running
 					write + flush operation */
@@ -721,17 +749,6 @@ inline lsn_t log_t::files::calc_lsn_offset(lsn_t lsn) const
   return l + LOG_FILE_HDR_SIZE * (1 + l / (file_size - LOG_FILE_HDR_SIZE));
 }
 
-inline void log_t::files::set_lsn(lsn_t a_lsn) {
-      ut_ad(log_sys.mutex.is_owned() || log_sys.write_mutex.is_owned());
-      lsn = a_lsn;
-}
-
-inline void log_t::files::set_lsn_offset(lsn_t a_lsn) {
-      ut_ad(log_sys.mutex.is_owned() || log_sys.write_mutex.is_owned());
-      ut_ad((lsn % OS_FILE_LOG_BLOCK_SIZE) == (a_lsn % OS_FILE_LOG_BLOCK_SIZE));
-      lsn_offset = a_lsn;
-}
-
 /** Test if flush order mutex is owned. */
 #define log_flush_order_mutex_own()			\
 	mutex_own(&log_sys.log_flush_order_mutex)
@@ -783,6 +800,6 @@ extern os_event_t	log_scrub_event;
 /** Whether log_scrub_thread is active */
 extern bool		log_scrub_thread_active;
 
-#include "log0log.inl"
+#include "log0log.ic"
 
 #endif
