@@ -1,6 +1,6 @@
 /*****************************************************************************
-Copyright (c) 1994, 2019, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2013, 2020, MariaDB Corporation.
+Copyright (c) 1994, 2016, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2013, 2018, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -12,7 +12,7 @@ FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
-51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -32,7 +32,6 @@ Created 2/2/1994 Heikki Tuuri
 #include "buf0buf.h"
 #include "data0data.h"
 #include "dict0dict.h"
-#include "rem0types.h"
 #include "rem0rec.h"
 #endif /* !UNIV_INNOCHECKSUM*/
 #include "fsp0fsp.h"
@@ -733,7 +732,9 @@ page_rec_get_heap_no(
 /** Determine whether a page has any siblings.
 @param[in]	page	page frame
 @return true if the page has any siblings */
-inline bool page_has_siblings(const page_t* page)
+inline
+bool
+page_has_siblings(const page_t* page)
 {
 	compile_time_assert(!(FIL_PAGE_PREV % 8));
 	compile_time_assert(FIL_PAGE_NEXT == FIL_PAGE_PREV + 4);
@@ -742,10 +743,22 @@ inline bool page_has_siblings(const page_t* page)
 		!= ~uint64_t(0);
 }
 
+/** Determine whether a page is an index root page.
+@param[in]	page	page frame
+@return true if the page is a root page of an index */
+inline
+bool
+page_is_root(const page_t* page)
+{
+	return fil_page_index_page_check(page) && !page_has_siblings(page);
+}
+
 /** Determine whether a page has a predecessor.
 @param[in]	page	page frame
 @return true if the page has a predecessor */
-inline bool page_has_prev(const page_t* page)
+inline
+bool
+page_has_prev(const page_t* page)
 {
 	return *reinterpret_cast<const uint32_t*>(page + FIL_PAGE_PREV)
 		!= FIL_NULL;
@@ -754,7 +767,9 @@ inline bool page_has_prev(const page_t* page)
 /** Determine whether a page has a successor.
 @param[in]	page	page frame
 @return true if the page has a successor */
-inline bool page_has_next(const page_t* page)
+inline
+bool
+page_has_next(const page_t* page)
 {
 	return *reinterpret_cast<const uint32_t*>(page + FIL_PAGE_NEXT)
 		!= FIL_NULL;
@@ -857,22 +872,6 @@ page_rec_is_last(
 	const page_t*	page)	/*!< in: page */
 	MY_ATTRIBUTE((warn_unused_result));
 
-/************************************************************//**
-true if distance between the records (measured in number of times we have to
-move to the next record) is at most the specified value
-@param[in]	left_rec	lefter record
-@param[in]	right_rec	righter record
-@param[in]	val		specified value to compare
-@return true if the distance is smaller than the value */
-UNIV_INLINE
-bool
-page_rec_distance_is_at_most(
-/*=========================*/
-	const rec_t*	left_rec,
-	const rec_t*	right_rec,
-	ulint		val)
-	MY_ATTRIBUTE((warn_unused_result));
-
 /************************************************************//**
 true if the record is the second last user record on a page.
 @return true if the second last user record */
@@ -893,6 +892,17 @@ page_rec_find_owner_rec(
 /*====================*/
 	rec_t*	rec);	/*!< in: the physical record */
 
+/***********************************************************************//**
+Write a 32-bit field in a data dictionary record. */
+UNIV_INLINE
+void
+page_rec_write_field(
+/*=================*/
+	rec_t*	rec,	/*!< in/out: record to update */
+	ulint	i,	/*!< in: index of the field to update */
+	ulint	val,	/*!< in: value to write */
+	mtr_t*	mtr)	/*!< in/out: mini-transaction */
+	MY_ATTRIBUTE((nonnull));
 /************************************************************//**
 Returns the maximum combined size of records which can be inserted on top
 of record heap.
@@ -955,6 +965,20 @@ page_mem_alloc_free(
 				free record list */
 	ulint		need);	/*!< in: number of bytes allocated */
 /************************************************************//**
+Allocates a block of memory from the heap of an index page.
+@return pointer to start of allocated buffer, or NULL if allocation fails */
+byte*
+page_mem_alloc_heap(
+/*================*/
+	page_t*		page,	/*!< in/out: index page */
+	page_zip_des_t*	page_zip,/*!< in/out: compressed page with enough
+				space available for inserting the record,
+				or NULL */
+	ulint		need,	/*!< in: total number of bytes needed */
+	ulint*		heap_no);/*!< out: this contains the heap number
+				of the allocated record
+				if allocation succeeds */
+/************************************************************//**
 Puts a record to free list. */
 UNIV_INLINE
 void
@@ -966,7 +990,7 @@ page_mem_free(
 	rec_t*			rec,	/*!< in: pointer to the (origin of)
 					record */
 	const dict_index_t*	index,	/*!< in: index of rec */
-	const rec_offs*		offsets);/*!< in: array returned by
+	const ulint*		offsets);/*!< in: array returned by
 					 rec_get_offsets() */
 
 /** Read the PAGE_DIRECTION field from a byte.
@@ -1226,7 +1250,7 @@ void
 page_rec_print(
 /*===========*/
 	const rec_t*	rec,	/*!< in: physical record */
-	const rec_offs*	offsets);/*!< in: record descriptor */
+	const ulint*	offsets);/*!< in: record descriptor */
 # ifdef UNIV_BTR_PRINT
 /***************************************************************//**
 This is used to print the contents of the directory for
@@ -1273,7 +1297,7 @@ ibool
 page_rec_validate(
 /*==============*/
 	const rec_t*	rec,	/*!< in: physical record */
-	const rec_offs*	offsets);/*!< in: array returned by rec_get_offsets() */
+	const ulint*	offsets);/*!< in: array returned by rec_get_offsets() */
 #ifdef UNIV_DEBUG
 /***************************************************************//**
 Checks that the first directory slot points to the infimum record and
@@ -1302,12 +1326,15 @@ ibool
 page_simple_validate_new(
 /*=====================*/
 	const page_t*	page);	/*!< in: index page in ROW_FORMAT!=REDUNDANT */
-/** Check the consistency of an index page.
-@param[in]	page	index page
-@param[in]	index	B-tree or R-tree index
-@return	whether the page is valid */
-bool page_validate(const page_t* page, const dict_index_t* index)
-	MY_ATTRIBUTE((nonnull));
+/***************************************************************//**
+This function checks the consistency of an index page.
+@return TRUE if ok */
+ibool
+page_validate(
+/*==========*/
+	const page_t*	page,	/*!< in: index page */
+	dict_index_t*	index);	/*!< in: data dictionary index containing
+				the page record type definition */
 /***************************************************************//**
 Looks in the page record list for a record with the given heap number.
 @return record, NULL if not found */
@@ -1331,6 +1358,6 @@ page_find_rec_max_not_deleted(
 
 #endif /* !UNIV_INNOCHECKSUM */
 
-#include "page0page.inl"
+#include "page0page.ic"
 
 #endif
