@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1996, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2014, 2020, MariaDB Corporation.
+Copyright (c) 2014, 2018, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -13,7 +13,7 @@ FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
-51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -58,7 +58,7 @@ lock_wait_table_print(void)
 			(ulong) slot->in_use,
 			(ulong) slot->suspended,
 			slot->wait_timeout,
-			(ulong) difftime(time(NULL), slot->suspend_time));
+			(ulong) difftime(ut_time(), slot->suspend_time));
 	}
 }
 
@@ -155,7 +155,7 @@ lock_wait_table_reserve_slot(
 
 			os_event_reset(slot->event);
 			slot->suspended = TRUE;
-			slot->suspend_time = time(NULL);
+			slot->suspend_time = ut_time();
 			slot->wait_timeout = wait_timeout;
 
 			if (slot == lock_sys.last_slot) {
@@ -192,7 +192,8 @@ wsrep_is_BF_lock_timeout(
 	const trx_t*	trx,
 	bool		locked = true)
 {
-	if (trx->is_wsrep() && wsrep_thd_is_BF(trx->mysql_thd, FALSE)
+	if (wsrep_on_trx(trx)
+	    && wsrep_thd_is_BF(trx->mysql_thd, FALSE)
 	    && trx->error_state != DB_DEADLOCK) {
 		ib::info() << "WSREP: BF lock wait long for trx:" << ib::hex(trx->id)
 			   << " query: " << wsrep_thd_query(trx->mysql_thd);
@@ -230,8 +231,13 @@ lock_wait_suspend_thread(
 				user OS thread */
 {
 	srv_slot_t*	slot;
+	double		wait_time;
 	trx_t*		trx;
 	ibool		was_declared_inside_innodb;
+	int64_t		start_time = 0;
+	int64_t		finish_time;
+	ulint		sec;
+	ulint		ms;
 	ulong		lock_wait_timeout;
 
 	trx = thr_get_trx(thr);
@@ -277,12 +283,15 @@ lock_wait_suspend_thread(
 	lock_wait_mutex_exit();
 	trx_mutex_exit(trx);
 
-	ulonglong start_time = 0;
-
 	if (thr->lock_state == QUE_THR_LOCK_ROW) {
 		srv_stats.n_lock_wait_count.inc();
 		srv_stats.n_lock_wait_current_count.inc();
-		start_time = my_interval_timer();
+
+		if (ut_usectime(&sec, &ms) == -1) {
+			start_time = -1;
+		} else {
+			start_time = int64_t(sec) * 1000000 + int64_t(ms);
+		}
 	}
 
 	ulint	lock_type = ULINT_UNDEFINED;
@@ -362,23 +371,28 @@ lock_wait_suspend_thread(
 		row_mysql_freeze_data_dictionary(trx);
 	}
 
-	double wait_time = difftime(time(NULL), slot->suspend_time);
+	wait_time = ut_difftime(ut_time(), slot->suspend_time);
 
 	/* Release the slot for others to use */
 
 	lock_wait_table_release_slot(slot);
 
 	if (thr->lock_state == QUE_THR_LOCK_ROW) {
-		const ulonglong finish_time = my_interval_timer();
-
-		if (finish_time >= start_time) {
-			const ulint diff_time = static_cast<ulint>
-				((finish_time - start_time) / 1000);
+		int64_t	diff_time;
+		if (start_time == -1 || ut_usectime(&sec, &ms) == -1) {
+			finish_time = -1;
+			diff_time = 0;
+		} else {
+			finish_time = int64_t(sec) * 1000000 + int64_t(ms);
+			diff_time = std::max<int64_t>(
+				0, finish_time - start_time);
 			srv_stats.n_lock_wait_time.add(diff_time);
+
 			/* Only update the variable if we successfully
 			retrieved the start and finish times. See Bug#36819. */
-			if (diff_time > lock_sys.n_lock_max_wait_time) {
-				lock_sys.n_lock_max_wait_time = diff_time;
+			if (ulint(diff_time) > lock_sys.n_lock_max_wait_time) {
+				lock_sys.n_lock_max_wait_time
+					= ulint(diff_time);
 			}
 			/* Record the lock wait time for this thread */
 			thd_storage_lock_wait(trx->mysql_thd, diff_time);
@@ -398,9 +412,8 @@ lock_wait_suspend_thread(
 	if (lock_wait_timeout < 100000000
 	    && wait_time > (double) lock_wait_timeout
 #ifdef WITH_WSREP
-	    && (!trx->is_wsrep()
-		|| (!wsrep_is_BF_lock_timeout(trx, false)
-		    && trx->error_state != DB_DEADLOCK))
+	    && (!wsrep_on_trx(trx) ||
+	       (!wsrep_is_BF_lock_timeout(trx, false) && trx->error_state != DB_DEADLOCK))
 #endif /* WITH_WSREP */
 	    ) {
 
@@ -455,12 +468,19 @@ lock_wait_check_and_cancel(
 	const srv_slot_t*	slot)	/*!< in: slot reserved by a user
 					thread when the wait started */
 {
+	trx_t*		trx;
+	double		wait_time;
+	ib_time_t	suspend_time = slot->suspend_time;
+
 	ut_ad(lock_wait_mutex_own());
+
 	ut_ad(slot->in_use);
+
 	ut_ad(slot->suspended);
 
-	double wait_time = difftime(time(NULL), slot->suspend_time);
-	trx_t* trx = thr_get_trx(slot->thr);
+	wait_time = ut_difftime(ut_time(), suspend_time);
+
+	trx = thr_get_trx(slot->thr);
 
 	if (trx_is_interrupted(trx)
 	    || (slot->wait_timeout < 100000000
@@ -495,6 +515,7 @@ lock_wait_check_and_cancel(
 
 		trx_mutex_exit(trx);
 	}
+
 }
 
 /*********************************************************************//**
