@@ -2,7 +2,7 @@
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2009, Google Inc.
-Copyright (c) 2014, 2019, MariaDB Corporation.
+Copyright (c) 2014, 2018, MariaDB Corporation.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -20,7 +20,7 @@ FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
-51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -161,46 +161,90 @@ log_buf_pool_get_oldest_modification(void)
 @param[in]	len	requested minimum size in bytes */
 void log_buffer_extend(ulong len)
 {
-	const ulong new_buf_size = ut_calc_align(len, srv_page_size);
-	byte* new_buf = static_cast<byte*>(ut_malloc_dontdump(new_buf_size));
-	TRASH_ALLOC(new_buf, new_buf_size);
-	byte* new_flush_buf =
-		static_cast<byte*>(ut_malloc_dontdump(new_buf_size));
-	TRASH_ALLOC(new_flush_buf, new_buf_size);
+	byte	tmp_buf[OS_FILE_LOG_BLOCK_SIZE];
 
-	log_mutex_enter();
+	log_mutex_enter_all();
 
-	if (len <= srv_log_buffer_size) {
-		/* Already extended enough by the others */
-		log_mutex_exit();
-		ut_free_dodump(new_buf, new_buf_size);
-		ut_free_dodump(new_flush_buf, new_buf_size);
-		return;
+	while (log_sys.is_extending) {
+		/* Another thread is trying to extend already.
+		Needs to wait for. */
+		log_mutex_exit_all();
+
+		log_buffer_flush_to_disk();
+
+		log_mutex_enter_all();
+
+		if (srv_log_buffer_size > len) {
+			/* Already extended enough by the others */
+			log_mutex_exit_all();
+			return;
+		}
 	}
 
-	ib::warn() << "The redo log transaction size " << len <<
-		" exceeds innodb_log_buffer_size="
-		<< srv_log_buffer_size << " / 2). Trying to extend it.";
+	if (len >= srv_log_buffer_size / 2) {
+		DBUG_EXECUTE_IF("ib_log_buffer_is_short_crash",
+				DBUG_SUICIDE(););
 
-	byte* old_buf = log_sys.buf;
-	byte* old_flush_buf = log_sys.flush_buf;
-	const ulong old_buf_size = srv_log_buffer_size;
+		/* log_buffer is too small. try to extend instead of crash. */
+		ib::warn() << "The redo log transaction size " << len <<
+			" exceeds innodb_log_buffer_size="
+			<< srv_log_buffer_size << " / 2). Trying to extend it.";
+	}
 
-	srv_log_buffer_size = new_buf_size;
-	log_sys.buf = new_buf;
-	log_sys.flush_buf = new_flush_buf;
-	memcpy(new_buf, old_buf, log_sys.buf_free);
+	log_sys.is_extending = true;
 
-	log_sys.max_buf_free = new_buf_size / LOG_BUF_FLUSH_RATIO
+	while (ut_calc_align_down(log_sys.buf_free,
+				  OS_FILE_LOG_BLOCK_SIZE)
+	       != ut_calc_align_down(log_sys.buf_next_to_write,
+				     OS_FILE_LOG_BLOCK_SIZE)) {
+		/* Buffer might have >1 blocks to write still. */
+		log_mutex_exit_all();
+
+		log_buffer_flush_to_disk();
+
+		log_mutex_enter_all();
+	}
+
+	ulong move_start = ut_calc_align_down(
+		log_sys.buf_free,
+		OS_FILE_LOG_BLOCK_SIZE);
+	ulong move_end = log_sys.buf_free;
+
+	/* store the last log block in buffer */
+	ut_memcpy(tmp_buf, log_sys.buf + move_start,
+		  move_end - move_start);
+
+	log_sys.buf_free -= move_start;
+	log_sys.buf_next_to_write -= move_start;
+
+	/* free previous after getting the right address */
+	if (!log_sys.first_in_use) {
+		log_sys.buf -= srv_log_buffer_size;
+	}
+	ut_free_dodump(log_sys.buf, srv_log_buffer_size * 2);
+
+	/* reallocate log buffer */
+	srv_log_buffer_size = len;
+
+	log_sys.buf = static_cast<byte*>(
+		ut_malloc_dontdump(srv_log_buffer_size * 2));
+	TRASH_ALLOC(log_sys.buf, srv_log_buffer_size * 2);
+
+	log_sys.first_in_use = true;
+
+	log_sys.max_buf_free = srv_log_buffer_size / LOG_BUF_FLUSH_RATIO
 		- LOG_BUF_FLUSH_MARGIN;
 
-	log_mutex_exit();
+	/* restore the last log block */
+	ut_memcpy(log_sys.buf, tmp_buf, move_end - move_start);
 
-	ut_free_dodump(old_buf, old_buf_size);
-	ut_free_dodump(old_flush_buf, old_buf_size);
+	ut_ad(log_sys.is_extending);
+	log_sys.is_extending = false;
+
+	log_mutex_exit_all();
 
 	ib::info() << "innodb_log_buffer_size was extended to "
-		<< new_buf_size << ".";
+		<< srv_log_buffer_size << ".";
 }
 
 /** Calculate actual length in redo buffer and file including
@@ -286,7 +330,7 @@ log_margin_checkpoint_age(
 		if (!flushed_enough) {
 			os_thread_sleep(100000);
 		}
-		log_checkpoint(true);
+		log_checkpoint(true, false);
 
 		log_mutex_enter();
 	}
@@ -309,6 +353,20 @@ log_reserve_and_open(
 loop:
 	ut_ad(log_mutex_own());
 
+	if (log_sys.is_extending) {
+		log_mutex_exit();
+
+		/* Log buffer size is extending. Writing up to the next block
+		should wait for the extending finished. */
+
+		os_thread_sleep(100000);
+
+		ut_ad(++count < 50);
+
+		log_mutex_enter();
+		goto loop;
+	}
+
 	/* Calculate an upper limit for the space the string may take in the
 	log buffer */
 
@@ -551,10 +609,10 @@ void log_t::create()
   ut_ad(srv_log_buffer_size >= 16 * OS_FILE_LOG_BLOCK_SIZE);
   ut_ad(srv_log_buffer_size >= 4U << srv_page_size_shift);
 
-  buf= static_cast<byte*>(ut_malloc_dontdump(srv_log_buffer_size));
-  TRASH_ALLOC(buf, srv_log_buffer_size);
-  flush_buf= static_cast<byte*>(ut_malloc_dontdump(srv_log_buffer_size));
-  TRASH_ALLOC(flush_buf, srv_log_buffer_size);
+  buf= static_cast<byte*>(ut_malloc_dontdump(srv_log_buffer_size * 2));
+  TRASH_ALLOC(buf, srv_log_buffer_size * 2);
+
+  first_in_use= true;
 
   max_buf_free= srv_log_buffer_size / LOG_BUF_FLUSH_RATIO -
     LOG_BUF_FLUSH_MARGIN;
@@ -564,6 +622,7 @@ void log_t::create()
   last_printout_time= time(NULL);
 
   buf_next_to_write= 0;
+  is_extending= false;
   write_lsn= lsn;
   flushed_to_disk_lsn= 0;
   n_pending_flushes= 0;
@@ -615,6 +674,16 @@ void log_t::files::create(ulint n_files)
   file_size= srv_log_file_size;
   lsn= LOG_START_LSN;
   lsn_offset= LOG_FILE_HDR_SIZE;
+
+  byte* ptr= static_cast<byte*>(ut_zalloc_nokey(LOG_FILE_HDR_SIZE * n_files
+						+ OS_FILE_LOG_BLOCK_SIZE));
+  file_header_bufs_ptr= ptr;
+  ptr= static_cast<byte*>(ut_align(ptr, OS_FILE_LOG_BLOCK_SIZE));
+
+  memset(file_header_bufs, 0, sizeof file_header_bufs);
+
+  for (ulint i = 0; i < n_files; i++, ptr += LOG_FILE_HDR_SIZE)
+    file_header_bufs[i] = ptr;
 }
 
 /******************************************************//**
@@ -627,6 +696,7 @@ log_file_header_flush(
 	lsn_t		start_lsn)	/*!< in: log file data starts at this
 					lsn */
 {
+	byte*	buf;
 	lsn_t	dest_offset;
 
 	ut_ad(log_write_mutex_own());
@@ -635,10 +705,9 @@ log_file_header_flush(
 	ut_ad((log_sys.log.format & ~LOG_HEADER_FORMAT_ENCRYPTED)
 	      == LOG_HEADER_FORMAT_CURRENT);
 
-	// man 2 open suggests this buffer to be aligned by 512 for O_DIRECT
-	MY_ALIGNED(OS_FILE_LOG_BLOCK_SIZE)
-	byte buf[OS_FILE_LOG_BLOCK_SIZE] = {0};
+	buf = log_sys.log.file_header_bufs[nth_file];
 
+	memset(buf, 0, OS_FILE_LOG_BLOCK_SIZE);
 	mach_write_to_4(buf + LOG_HEADER_FORMAT, log_sys.log.format);
 	mach_write_to_4(buf + LOG_HEADER_SUBFORMAT, log_sys.log.subformat);
 	mach_write_to_8(buf + LOG_HEADER_START_LSN, start_lsn);
@@ -656,6 +725,8 @@ log_file_header_flush(
 
 	log_sys.n_log_ios++;
 
+	MONITOR_INC(MONITOR_LOG_IO);
+
 	srv_stats.os_log_pending_writes.inc();
 
 	const ulint	page_no = ulint(dest_offset >> srv_page_size_shift);
@@ -761,19 +832,18 @@ loop:
 	the trailer fields of the log blocks */
 
 	for (i = 0; i < write_len / OS_FILE_LOG_BLOCK_SIZE; i++) {
-#ifdef UNIV_DEBUG
-		ulint hdr_no_2 = log_block_get_hdr_no(buf) + i;
-		DBUG_EXECUTE_IF("innodb_small_log_block_no_limit",
-				hdr_no_2 = ((hdr_no_2 - 1) & 0xFUL) + 1;);
-#endif
 		ut_ad(pad_len >= len
-			|| i * OS_FILE_LOG_BLOCK_SIZE >= len - pad_len
-			|| log_block_get_hdr_no(buf + i * OS_FILE_LOG_BLOCK_SIZE) == hdr_no_2);
+		      || i * OS_FILE_LOG_BLOCK_SIZE >= len - pad_len
+		      || log_block_get_hdr_no(
+			      buf + i * OS_FILE_LOG_BLOCK_SIZE)
+			 == log_block_get_hdr_no(buf) + i);
 		log_block_store_checksum(buf + i * OS_FILE_LOG_BLOCK_SIZE);
 	}
 
 	log_sys.n_log_ios++;
 
+	MONITOR_INC(MONITOR_LOG_IO);
+
 	srv_stats.os_log_pending_writes.inc();
 
 	ut_a((next_offset >> srv_page_size_shift) <= ULINT_MAX);
@@ -817,6 +887,7 @@ log_write_flush_to_disk_low()
 		fil_flush(SRV_LOG_SPACE_FIRST_ID);
 	}
 
+	MONITOR_DEC(MONITOR_PENDING_LOG_FLUSH);
 
 	log_mutex_enter();
 	if (do_flush) {
@@ -828,8 +899,8 @@ log_write_flush_to_disk_low()
 	os_event_set(log_sys.flush_event);
 }
 
-/** Swap log buffers, and copy the content of last block
-from old buf to the head of the new buf. Thus, buf_free and
+/** Switch the log buffer in use, and copy the content of last block
+from old log buffer to the head of the to be used one. Thus, buf_free and
 buf_next_to_write would be changed accordingly */
 static inline
 void
@@ -838,16 +909,27 @@ log_buffer_switch()
 	ut_ad(log_mutex_own());
 	ut_ad(log_write_mutex_own());
 
-	ulong		area_end = ut_calc_align(
-		log_sys.buf_free, ulong(OS_FILE_LOG_BLOCK_SIZE));
+	const byte*	old_buf = log_sys.buf;
+	ulint		area_end = ut_calc_align(log_sys.buf_free,
+						 OS_FILE_LOG_BLOCK_SIZE);
+
+	if (log_sys.first_in_use) {
+		log_sys.first_in_use = false;
+		ut_ad(log_sys.buf == ut_align(log_sys.buf,
+					       OS_FILE_LOG_BLOCK_SIZE));
+		log_sys.buf += srv_log_buffer_size;
+	} else {
+		log_sys.first_in_use = true;
+		log_sys.buf -= srv_log_buffer_size;
+		ut_ad(log_sys.buf == ut_align(log_sys.buf,
+					       OS_FILE_LOG_BLOCK_SIZE));
+	}
 
 	/* Copy the last block to new buf */
-	ut_memcpy(log_sys.flush_buf,
-		  log_sys.buf + area_end - OS_FILE_LOG_BLOCK_SIZE,
+	ut_memcpy(log_sys.buf,
+		  old_buf + area_end - OS_FILE_LOG_BLOCK_SIZE,
 		  OS_FILE_LOG_BLOCK_SIZE);
 
-	std::swap(log_sys.buf, log_sys.flush_buf);
-
 	log_sys.buf_free %= OS_FILE_LOG_BLOCK_SIZE;
 	log_sys.buf_next_to_write = log_sys.buf_free;
 }
@@ -949,6 +1031,7 @@ loop:
 	if (flush_to_disk) {
 		log_sys.n_pending_flushes++;
 		log_sys.current_flush_lsn = log_sys.lsn;
+		MONITOR_INC(MONITOR_PENDING_LOG_FLUSH);
 		os_event_reset(log_sys.flush_event);
 
 		if (log_sys.buf_free == log_sys.buf_next_to_write) {
@@ -963,9 +1046,8 @@ loop:
 	start_offset = log_sys.buf_next_to_write;
 	end_offset = log_sys.buf_free;
 
-	area_start = ut_2pow_round(start_offset,
-				   ulint(OS_FILE_LOG_BLOCK_SIZE));
-	area_end = ut_calc_align(end_offset, ulint(OS_FILE_LOG_BLOCK_SIZE));
+	area_start = ut_calc_align_down(start_offset, OS_FILE_LOG_BLOCK_SIZE);
+	area_end = ut_calc_align(end_offset, OS_FILE_LOG_BLOCK_SIZE);
 
 	ut_ad(area_end - area_start > 0);
 
@@ -1006,7 +1088,7 @@ loop:
 		}
 	}
 
-	if (UNIV_UNLIKELY(srv_shutdown_state > SRV_SHUTDOWN_INITIATED)) {
+	if (UNIV_UNLIKELY(srv_shutdown_state != SRV_SHUTDOWN_NONE)) {
 		service_manager_extend_timeout(INNODB_EXTEND_TIMEOUT_INTERVAL,
 					       "InnoDB log write: "
 					       LSN_PF "," LSN_PF,
@@ -1121,11 +1203,14 @@ synchronization objects!
 this lsn
 @return false if there was a flush batch of the same type running,
 which means that we could not start this flush batch */
-static bool log_preflush_pool_modified_pages(lsn_t new_oldest)
+static
+bool
+log_preflush_pool_modified_pages(
+	lsn_t			new_oldest)
 {
 	bool	success;
 
-	if (recv_recovery_is_on()) {
+	if (recv_recovery_on) {
 		/* If the recovery is running, we must first apply all
 		log records to their respective file pages to get the
 		right modify lsn values to these pages: otherwise, there
@@ -1226,7 +1311,7 @@ log_group_checkpoint(lsn_t end_lsn)
 	ut_ad(end_lsn == 0 || end_lsn >= log_sys.next_checkpoint_lsn);
 	ut_ad(end_lsn <= log_sys.lsn);
 	ut_ad(end_lsn + SIZE_OF_MLOG_CHECKPOINT <= log_sys.lsn
-	      || srv_shutdown_state > SRV_SHUTDOWN_INITIATED);
+	      || srv_shutdown_state != SRV_SHUTDOWN_NONE);
 
 	DBUG_PRINT("ib_log", ("checkpoint " UINT64PF " at " LSN_PF
 			      " written",
@@ -1337,10 +1422,15 @@ log_append_on_checkpoint(
 /** Make a checkpoint. Note that this function does not flush dirty
 blocks from the buffer pool: it only checks what is lsn of the oldest
 modification in the pool, and writes information about the lsn in
-log files. Use log_make_checkpoint() to flush also the pool.
+log files. Use log_make_checkpoint_at() to flush also the pool.
 @param[in]	sync		whether to wait for the write to complete
+@param[in]	write_always	force a write even if no log
+has been generated since the latest checkpoint
 @return true if success, false if a checkpoint write was already running */
-bool log_checkpoint(bool sync)
+bool
+log_checkpoint(
+	bool	sync,
+	bool	write_always)
 {
 	lsn_t	oldest_lsn;
 
@@ -1383,15 +1473,9 @@ bool log_checkpoint(bool sync)
 	flushed up to oldest_lsn. */
 
 	ut_ad(oldest_lsn >= log_sys.last_checkpoint_lsn);
-	if (oldest_lsn
-	    > log_sys.last_checkpoint_lsn + SIZE_OF_MLOG_CHECKPOINT) {
-		/* Some log has been written since the previous checkpoint. */
-	} else if (srv_shutdown_state > SRV_SHUTDOWN_INITIATED) {
-		/* MariaDB 10.3 startup expects the redo log file to be
-		logically empty (not even containing a MLOG_CHECKPOINT record)
-		after a clean shutdown. Perform an extra checkpoint at
-		shutdown. */
-	} else {
+	if (!write_always
+	    && oldest_lsn
+	    <= log_sys.last_checkpoint_lsn + SIZE_OF_MLOG_CHECKPOINT) {
 		/* Do nothing, because nothing was logged (other than
 		a MLOG_CHECKPOINT marker) since the previous checkpoint. */
 		log_mutex_exit();
@@ -1411,7 +1495,7 @@ bool log_checkpoint(bool sync)
 	lsn_t		flush_lsn	= oldest_lsn;
 	const lsn_t	end_lsn		= log_sys.lsn;
 	const bool	do_write
-		= srv_shutdown_state <= SRV_SHUTDOWN_INITIATED
+		= srv_shutdown_state == SRV_SHUTDOWN_NONE
 		|| flush_lsn != end_lsn;
 
 	if (fil_names_clear(flush_lsn, do_write)) {
@@ -1423,6 +1507,20 @@ bool log_checkpoint(bool sync)
 
 	log_write_up_to(flush_lsn, true);
 
+	DBUG_EXECUTE_IF(
+		"using_wa_checkpoint_middle",
+		if (write_always) {
+			DEBUG_SYNC_C("wa_checkpoint_middle");
+
+			const my_bool b = TRUE;
+			buf_flush_page_cleaner_disabled_debug_update(
+				NULL, NULL, NULL, &b);
+			dict_stats_disabled_debug_update(
+				NULL, NULL, NULL, &b);
+			srv_master_thread_disabled_debug_update(
+				NULL, NULL, NULL, &b);
+		});
+
 	log_mutex_enter();
 
 	ut_ad(log_sys.flushed_to_disk_lsn >= flush_lsn);
@@ -1453,16 +1551,23 @@ bool log_checkpoint(bool sync)
 	return(true);
 }
 
-/** Make a checkpoint */
-void log_make_checkpoint()
+/** Make a checkpoint at or after a specified LSN.
+@param[in]	lsn		the log sequence number, or LSN_MAX
+for the latest LSN
+@param[in]	write_always	force a write even if no log
+has been generated since the latest checkpoint */
+void
+log_make_checkpoint_at(
+	lsn_t			lsn,
+	bool			write_always)
 {
 	/* Preflush pages synchronously */
 
-	while (!log_preflush_pool_modified_pages(LSN_MAX)) {
+	while (!log_preflush_pool_modified_pages(lsn)) {
 		/* Flush as much as we can */
 	}
 
-	while (!log_checkpoint(true)) {
+	while (!log_checkpoint(true, write_always)) {
 		/* Force a checkpoint */
 	}
 }
@@ -1542,7 +1647,7 @@ loop:
 	}
 
 	if (do_checkpoint) {
-		log_checkpoint(checkpoint_sync);
+		log_checkpoint(checkpoint_sync, FALSE);
 
 		if (checkpoint_sync) {
 
@@ -1676,7 +1781,7 @@ wait_suspend_loop:
 			"Waiting for %s to exit", thread_name);
 		if (srv_print_verbose_log && count > COUNT_INTERVAL) {
 			ib::info() << "Waiting for " << thread_name
-				   << " to exit";
+				   << "to exit";
 			count = 0;
 		}
 		goto loop;
@@ -1719,9 +1824,6 @@ wait_suspend_loop:
 				"Waiting for page cleaner");
 			ib::info() << "Waiting for page_cleaner to "
 				"finish flushing of buffer pool";
-			/* This is a workaround to avoid the InnoDB hang
-			when OS datetime changed backwards */
-			os_event_set(buf_flush_event);
 			count = 0;
 		}
 	}
@@ -1794,7 +1896,7 @@ wait_suspend_loop:
 	if (!srv_read_only_mode) {
 		service_manager_extend_timeout(INNODB_EXTEND_TIMEOUT_INTERVAL,
 			"ensuring dirty buffer pool are written to log");
-		log_make_checkpoint();
+		log_make_checkpoint_at(LSN_MAX, TRUE);
 
 		log_mutex_enter();
 
@@ -1938,13 +2040,19 @@ void log_t::close()
   m_initialised = false;
   log.close();
 
-  ut_free_dodump(buf, srv_log_buffer_size);
+  if (!first_in_use)
+    buf -= srv_log_buffer_size;
+  ut_free_dodump(buf, srv_log_buffer_size * 2);
   buf = NULL;
-  ut_free_dodump(flush_buf, srv_log_buffer_size);
-  flush_buf = NULL;
 
   os_event_destroy(flush_event);
+
   rw_lock_free(&checkpoint_lock);
+  /* rw_lock_free() already called checkpoint_lock.~rw_lock_t();
+  tame the debug assertions when the destructor will be called once more. */
+  ut_ad(checkpoint_lock.magic_n == 0);
+  ut_d(checkpoint_lock.magic_n = RW_LOCK_MAGIC_N);
+
   mutex_free(&mutex);
   mutex_free(&write_mutex);
   mutex_free(&log_flush_order_mutex);
