@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 2013, 2018, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, 2020, MariaDB Corporation.
+Copyright (c) 2017, 2018, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -13,7 +13,7 @@ FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
-51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -263,7 +263,7 @@ TruncateLogParser::scan(
 	while (fil_file_readdir_next_file(
 			&err, dir_path, dir, &fileinfo) == 0) {
 
-		const size_t nm_len = strlen(fileinfo.name);
+		ulint nm_len = strlen(fileinfo.name);
 
 		if (fileinfo.type == OS_FILE_TYPE_FILE
 		    && nm_len > sizeof "ib_trunc.log"
@@ -286,13 +286,18 @@ TruncateLogParser::scan(
 				err = DB_OUT_OF_MEMORY;
 				break;
 			}
+			memset(log_file_name, 0, sz);
 
-			memcpy(log_file_name, dir_path, dir_len);
-			char* e = log_file_name + dir_len;
-			if (e[-1] != OS_PATH_SEPARATOR) {
-				*e++ = OS_PATH_SEPARATOR;
+			strncpy(log_file_name, dir_path, dir_len);
+			ulint	log_file_name_len = strlen(log_file_name);
+			if (log_file_name[log_file_name_len - 1]
+				!= OS_PATH_SEPARATOR) {
+
+				log_file_name[log_file_name_len]
+					= OS_PATH_SEPARATOR;
+				log_file_name_len = strlen(log_file_name);
 			}
-			strcpy(e, fileinfo.name);
+			strcat(log_file_name, fileinfo.name);
 			log_files.push_back(log_file_name);
 		}
 	}
@@ -440,6 +445,70 @@ TruncateLogParser::scan_and_parse(
 	return(err);
 }
 
+/** Callback to drop indexes during TRUNCATE */
+class DropIndex : public Callback {
+
+public:
+	/**
+	Constructor
+
+	@param[in,out]	table	Table to truncate
+	@param[in]	noredo	whether to disable redo logging */
+	DropIndex(dict_table_t* table, bool noredo)
+		:
+		Callback(table->id, noredo),
+		m_table(table)
+	{
+		/* No op */
+	}
+
+	/**
+	@param mtr	mini-transaction covering the read
+	@param pcur	persistent cursor used for reading
+	@return DB_SUCCESS or error code */
+	dberr_t operator()(mtr_t* mtr, btr_pcur_t* pcur) const;
+
+private:
+	/** Table to be truncated */
+	dict_table_t*		m_table;
+};
+
+/** Callback to create the indexes during TRUNCATE */
+class CreateIndex : public Callback {
+
+public:
+	/**
+	Constructor
+
+	@param[in,out]	table	Table to truncate
+	@param[in]	noredo	whether to disable redo logging */
+	CreateIndex(dict_table_t* table, bool noredo)
+		:
+		Callback(table->id, noredo),
+		m_table(table)
+	{
+		/* No op */
+	}
+
+	/**
+	Create the new index and update the root page number in the
+	SysIndex table.
+
+	@param mtr	mini-transaction covering the read
+	@param pcur	persistent cursor used for reading
+	@return DB_SUCCESS or error code */
+	dberr_t operator()(mtr_t* mtr, btr_pcur_t* pcur) const;
+
+private:
+	// Disably copying
+	CreateIndex(const CreateIndex&);
+	CreateIndex& operator=(const CreateIndex&);
+
+private:
+	/** Table to be truncated */
+	dict_table_t*		m_table;
+};
+
 /** Check for presence of table-id in SYS_XXXX tables. */
 class TableLocator : public Callback {
 
@@ -478,6 +547,161 @@ private:
 	bool			m_table_found;
 };
 
+/**
+Drop an index in the table.
+
+@param mtr	mini-transaction covering the read
+@param pcur	persistent cursor used for reading
+@return DB_SUCCESS or error code */
+dberr_t
+DropIndex::operator()(mtr_t* mtr, btr_pcur_t* pcur) const
+{
+	rec_t*	rec = btr_pcur_get_rec(pcur);
+
+	bool	freed = dict_drop_index_tree(rec, pcur, mtr);
+
+#ifdef UNIV_DEBUG
+	{
+		ulint		len;
+		const byte*	field;
+		ulint		index_type;
+
+		field = rec_get_nth_field_old(
+			btr_pcur_get_rec(pcur), DICT_FLD__SYS_INDEXES__TYPE,
+			&len);
+		ut_ad(len == 4);
+
+		index_type = mach_read_from_4(field);
+
+		if (index_type & DICT_CLUSTERED) {
+			/* Clustered index */
+			DBUG_EXECUTE_IF("ib_trunc_crash_on_drop_of_clust_index",
+					log_buffer_flush_to_disk();
+					os_thread_sleep(2000000);
+					DBUG_SUICIDE(););
+		} else if (index_type & DICT_UNIQUE) {
+			/* Unique index */
+			DBUG_EXECUTE_IF("ib_trunc_crash_on_drop_of_uniq_index",
+					log_buffer_flush_to_disk();
+					os_thread_sleep(2000000);
+					DBUG_SUICIDE(););
+		} else if (index_type == 0) {
+			/* Secondary index */
+			DBUG_EXECUTE_IF("ib_trunc_crash_on_drop_of_sec_index",
+					log_buffer_flush_to_disk();
+					os_thread_sleep(2000000);
+					DBUG_SUICIDE(););
+		}
+	}
+#endif /* UNIV_DEBUG */
+
+	DBUG_EXECUTE_IF("ib_err_trunc_drop_index", return DB_ERROR;);
+
+	if (freed) {
+
+		/* We will need to commit and restart the
+		mini-transaction in order to avoid deadlocks.
+		The dict_drop_index_tree() call has freed
+		a page in this mini-transaction, and the rest
+		of this loop could latch another index page.*/
+		const mtr_log_t log_mode = mtr->get_log_mode();
+		mtr_commit(mtr);
+
+		mtr_start(mtr);
+		mtr->set_log_mode(log_mode);
+
+		btr_pcur_restore_position(BTR_MODIFY_LEAF, pcur, mtr);
+	} else {
+		if (!m_table->space) {
+			return DB_ERROR;
+		}
+	}
+
+	return(DB_SUCCESS);
+}
+
+/**
+Create the new index and update the root page number in the
+SysIndex table.
+
+@param mtr	mini-transaction covering the read
+@param pcur	persistent cursor used for reading
+@return DB_SUCCESS or error code */
+dberr_t
+CreateIndex::operator()(mtr_t* mtr, btr_pcur_t* pcur) const
+{
+	ulint	root_page_no;
+
+	root_page_no = dict_recreate_index_tree(m_table, pcur, mtr);
+
+#ifdef UNIV_DEBUG
+	{
+		ulint		len;
+		const byte*	field;
+		ulint		index_type;
+
+		field = rec_get_nth_field_old(
+			btr_pcur_get_rec(pcur), DICT_FLD__SYS_INDEXES__TYPE,
+			&len);
+		ut_ad(len == 4);
+
+		index_type = mach_read_from_4(field);
+
+		if (index_type & DICT_CLUSTERED) {
+			/* Clustered index */
+			DBUG_EXECUTE_IF(
+				"ib_trunc_crash_on_create_of_clust_index",
+				log_buffer_flush_to_disk();
+				os_thread_sleep(2000000);
+				DBUG_SUICIDE(););
+		} else if (index_type & DICT_UNIQUE) {
+			/* Unique index */
+			DBUG_EXECUTE_IF(
+				"ib_trunc_crash_on_create_of_uniq_index",
+				log_buffer_flush_to_disk();
+				os_thread_sleep(2000000);
+				DBUG_SUICIDE(););
+		} else if (index_type == 0) {
+			/* Secondary index */
+			DBUG_EXECUTE_IF(
+				"ib_trunc_crash_on_create_of_sec_index",
+				log_buffer_flush_to_disk();
+				os_thread_sleep(2000000);
+				DBUG_SUICIDE(););
+		}
+	}
+#endif /* UNIV_DEBUG */
+
+	DBUG_EXECUTE_IF("ib_err_trunc_create_index", return DB_ERROR;);
+
+	if (root_page_no != FIL_NULL) {
+
+		rec_t*	rec = btr_pcur_get_rec(pcur);
+
+		page_rec_write_field(
+			rec, DICT_FLD__SYS_INDEXES__PAGE_NO,
+			root_page_no, mtr);
+
+		/* We will need to commit and restart the
+		mini-transaction in order to avoid deadlocks.
+		The dict_create_index_tree() call has allocated
+		a page in this mini-transaction, and the rest of
+		this loop could latch another index page. */
+		mtr_commit(mtr);
+
+		mtr_start(mtr);
+
+		btr_pcur_restore_position(BTR_MODIFY_LEAF, pcur, mtr);
+
+	} else {
+		if (!m_table->space) {
+			return(DB_ERROR);
+		}
+	}
+
+	return(DB_SUCCESS);
+}
+
 /**
 Update system table to reflect new table id.
 @param old_table_id		old table id
@@ -613,7 +837,7 @@ row_truncate_update_sys_tables_during_fix_up(
 	}
 
 	trx_commit_for_mysql(trx);
-	trx->free();
+	trx_free(trx);
 
 	return(err);
 }
@@ -970,7 +1194,7 @@ truncate_t::fixup_tables_in_non_system_tablespace()
 
 	if (err == DB_SUCCESS && s_tables.size() > 0) {
 
-		log_make_checkpoint();
+		log_make_checkpoint_at(LSN_MAX, TRUE);
 	}
 
 	for (ulint i = 0; i < s_tables.size(); ++i) {
