@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1996, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2015, 2021, MariaDB Corporation.
+Copyright (c) 2015, 2018, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -13,7 +13,7 @@ FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
-51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -71,6 +71,17 @@ typedef std::set<
 	std::less<table_id_t>,
 	ut_allocator<table_id_t> >	table_id_set;
 
+/** Set flush observer for the transaction
+@param[in/out]	trx		transaction struct
+@param[in]	observer	flush observer */
+void
+trx_set_flush_observer(
+	trx_t*		trx,
+	FlushObserver*	observer)
+{
+	trx->flush_observer = observer;
+}
+
 /*************************************************************//**
 Set detailed error message for the transaction. */
 void
@@ -79,8 +90,7 @@ trx_set_detailed_error(
 	trx_t*		trx,	/*!< in: transaction struct */
 	const char*	msg)	/*!< in: detailed error message */
 {
-	strncpy(trx->detailed_error, msg, MAX_DETAILED_ERROR_LEN - 1);
-	trx->detailed_error[MAX_DETAILED_ERROR_LEN - 1] = '\0';
+	ut_strlcpy(trx->detailed_error, msg, MAX_DETAILED_ERROR_LEN);
 }
 
 /*************************************************************//**
@@ -112,7 +122,7 @@ trx_init(
 
 	trx->op_info = "";
 
-	trx->active_commit_ordered = false;
+	trx->active_commit_ordered = 0;
 
 	trx->isolation_level = TRX_ISO_REPEATABLE_READ;
 
@@ -140,7 +150,7 @@ trx_init(
 
 	trx->auto_commit = false;
 
-	trx->will_lock = false;
+	trx->will_lock = 0;
 
 	trx->ddl = false;
 
@@ -161,13 +171,8 @@ trx_init(
 	trx->lock.rec_cached = 0;
 
 	trx->lock.table_cached = 0;
-#ifdef WITH_WSREP
-	ut_ad(!trx->wsrep);
-	ut_ad(!trx->wsrep_event);
-	ut_ad(!trx->wsrep_UK_scan);
-#endif /* WITH_WSREP */
 
-	ut_ad(trx->get_flush_observer() == NULL);
+	trx->flush_observer = NULL;
 }
 
 /** For managing the life-cycle of the trx_t instance that we get
@@ -216,15 +221,6 @@ struct TrxFactory {
 	@param trx the transaction for which to release resources */
 	static void destroy(trx_t* trx)
 	{
-#ifdef __SANITIZE_ADDRESS__
-		/* Unpoison the memory for AddressSanitizer */
-		MEM_MAKE_ADDRESSABLE(trx, sizeof *trx);
-#else
-		/* Declare the contents as initialized for Valgrind;
-		we checked this in trx_t::free(). */
-		MEM_MAKE_DEFINED(trx, sizeof *trx);
-#endif
-
 		ut_a(trx->magic_n == TRX_MAGIC_N);
 		ut_ad(!trx->mysql_thd);
 
@@ -252,6 +248,36 @@ struct TrxFactory {
 
 		trx->read_view.~ReadView();
 	}
+
+	/** Enforce any invariants here, this is called before the transaction
+	is added to the pool.
+	@return true if all OK */
+	static bool debug(const trx_t* trx)
+	{
+		ut_a(trx->error_state == DB_SUCCESS);
+
+		ut_a(trx->magic_n == TRX_MAGIC_N);
+
+		ut_ad(!trx->read_only);
+
+		ut_ad(trx->state == TRX_STATE_NOT_STARTED);
+
+		ut_ad(trx->dict_operation == TRX_DICT_OP_NONE);
+
+		ut_ad(trx->mysql_thd == 0);
+
+		ut_a(trx->lock.wait_thr == NULL);
+		ut_a(trx->lock.wait_lock == NULL);
+		ut_a(trx->dict_operation_lock_mode == 0);
+
+		ut_a(UT_LIST_GET_LEN(trx->lock.trx_locks) == 0);
+
+		ut_ad(trx->autoinc_locks == NULL);
+
+		ut_ad(trx->lock.table_locks.empty());
+
+		return(true);
+	}
 };
 
 /** The lock strategy for TrxPool */
@@ -328,30 +354,18 @@ trx_pool_close()
 	trx_pools = 0;
 }
 
-/** @return an allocated transaction */
+/** @return a trx_t instance from trx_pools. */
 trx_t *trx_create()
 {
 	trx_t*	trx = trx_pools->get();
 
-#ifdef __SANITIZE_ADDRESS__
-	/* Unpoison the memory for AddressSanitizer.
-	It may have been poisoned in trx_t::free().*/
-	MEM_MAKE_ADDRESSABLE(trx, sizeof *trx);
-#else
-	/* Declare the memory initialized for Valgrind.
-	The trx_t that are released to the pool are
-	actually initialized; we checked that by
-	MEM_CHECK_DEFINED() in trx_t::free(). */
-	MEM_MAKE_DEFINED(trx, sizeof *trx);
-#endif
-
-	trx->assert_freed();
+	assert_trx_is_free(trx);
 
 	mem_heap_t*	heap;
 	ib_alloc_t*	alloc;
 
 	/* We just got trx from pool, it should be non locking */
-	ut_ad(!trx->will_lock);
+	ut_ad(trx->will_lock == 0);
 	ut_ad(trx->state == TRX_STATE_NOT_STARTED);
 	ut_ad(!trx->rw_trx_hash_pins);
 
@@ -361,16 +375,20 @@ trx_t *trx_create()
 
 	alloc = ib_heap_allocator_create(heap);
 
+	/* Remember to free the vector explicitly in trx_free(). */
 	trx->autoinc_locks = ib_vector_create(alloc, sizeof(void**), 4);
 
+	/* Should have been either just initialized or .clear()ed by
+	trx_free(). */
 	ut_ad(trx->mod_tables.empty());
+	ut_ad(trx->lock.table_locks.empty());
+	ut_ad(UT_LIST_GET_LEN(trx->lock.trx_locks) == 0);
 	ut_ad(trx->lock.n_rec_locks == 0);
 	ut_ad(trx->lock.table_cached == 0);
 	ut_ad(trx->lock.rec_cached == 0);
 
 #ifdef WITH_WSREP
 	trx->wsrep_event = NULL;
-	ut_ad(!trx->wsrep_UK_scan);
 #endif /* WITH_WSREP */
 
 	trx_sys.register_trx(trx);
@@ -378,152 +396,79 @@ trx_t *trx_create()
 	return(trx);
 }
 
-/** Free the memory to trx_pools */
-void trx_t::free()
+/**
+  Release a trx_t instance back to the pool.
+  @param trx the instance to release.
+*/
+void trx_free(trx_t*& trx)
 {
-  MEM_CHECK_DEFINED(this, sizeof *this);
+	ut_ad(!trx->declared_to_be_inside_innodb);
+	ut_ad(!trx->n_mysql_tables_in_use);
+	ut_ad(!trx->mysql_n_tables_locked);
+	ut_ad(!trx->internal);
 
-  ut_ad(!n_mysql_tables_in_use);
-  ut_ad(!mysql_n_tables_locked);
-  ut_ad(!internal);
-  ut_ad(!declared_to_be_inside_innodb);
-  ut_ad(!will_lock);
-  ut_ad(error_state == DB_SUCCESS);
-  ut_ad(magic_n == TRX_MAGIC_N);
-  ut_ad(!read_only);
-  ut_ad(!lock.wait_lock);
+	if (trx->declared_to_be_inside_innodb) {
 
-  dict_operation= TRX_DICT_OP_NONE;
-  trx_sys.deregister_trx(this);
-  assert_freed();
-  trx_sys.rw_trx_hash.put_pins(this);
+		ib::error() << "Freeing a trx (" << trx << ", "
+			<< trx_get_id_for_print(trx) << ") which is declared"
+			" to be processing inside InnoDB";
 
-  mysql_thd= NULL;
-  mysql_log_file_name= NULL;
+		trx_print(stderr, trx, 600);
+		putc('\n', stderr);
 
-  // FIXME: We need to avoid this heap free/alloc for each commit.
-  if (autoinc_locks)
-  {
-    ut_ad(ib_vector_is_empty(autoinc_locks));
-    /* We allocated a dedicated heap for the vector. */
-    ib_vector_free(autoinc_locks);
-    autoinc_locks= NULL;
-  }
+		/* This is an error but not a fatal error. We must keep
+		the counters like srv_conc.n_active accurate. */
+		srv_conc_force_exit_innodb(trx);
+	}
 
-  mod_tables.clear();
+	if (trx->n_mysql_tables_in_use != 0
+	    || trx->mysql_n_tables_locked != 0) {
 
-  MEM_NOACCESS(&n_ref, sizeof n_ref);
-  /* do not poison mutex */
-  MEM_NOACCESS(&id, sizeof id);
-  MEM_NOACCESS(&no, sizeof no);
-  MEM_NOACCESS(&state, sizeof state);
-  MEM_NOACCESS(&is_recovered, sizeof is_recovered);
-#ifdef WITH_WSREP
-  MEM_NOACCESS(&wsrep, sizeof wsrep);
-#endif
-  MEM_NOACCESS(&read_view, sizeof read_view);
-  MEM_NOACCESS(&trx_list, sizeof trx_list);
-  MEM_NOACCESS(&lock, sizeof lock);
-  MEM_NOACCESS(&op_info, sizeof op_info);
-  MEM_NOACCESS(&isolation_level, sizeof isolation_level);
-  MEM_NOACCESS(&check_foreigns, sizeof check_foreigns);
-  MEM_NOACCESS(&is_registered, sizeof is_registered);
-  MEM_NOACCESS(&active_commit_ordered, sizeof active_commit_ordered);
-  MEM_NOACCESS(&check_unique_secondary, sizeof check_unique_secondary);
-  MEM_NOACCESS(&flush_log_later, sizeof flush_log_later);
-  MEM_NOACCESS(&must_flush_log_later, sizeof must_flush_log_later);
-  MEM_NOACCESS(&duplicates, sizeof duplicates);
-  MEM_NOACCESS(&dict_operation, sizeof dict_operation);
-  MEM_NOACCESS(&declared_to_be_inside_innodb, sizeof declared_to_be_inside_innodb);
-  MEM_NOACCESS(&n_tickets_to_enter_innodb, sizeof n_tickets_to_enter_innodb);
-  MEM_NOACCESS(&dict_operation_lock_mode, sizeof dict_operation_lock_mode);
-  MEM_NOACCESS(&start_time, sizeof start_time);
-  MEM_NOACCESS(&start_time_micro, sizeof start_time_micro);
-  MEM_NOACCESS(&commit_lsn, sizeof commit_lsn);
-  MEM_NOACCESS(&table_id, sizeof table_id);
-  MEM_NOACCESS(&mysql_thd, sizeof mysql_thd);
-  MEM_NOACCESS(&mysql_log_file_name, sizeof mysql_log_file_name);
-  MEM_NOACCESS(&mysql_log_offset, sizeof mysql_log_offset);
-  MEM_NOACCESS(&n_mysql_tables_in_use, sizeof n_mysql_tables_in_use);
-  MEM_NOACCESS(&mysql_n_tables_locked, sizeof mysql_n_tables_locked);
-  MEM_NOACCESS(&error_state, sizeof error_state);
-  MEM_NOACCESS(&error_info, sizeof error_info);
-  MEM_NOACCESS(&error_key_num, sizeof error_key_num);
-  MEM_NOACCESS(&graph, sizeof graph);
-  MEM_NOACCESS(&trx_savepoints, sizeof trx_savepoints);
-  MEM_NOACCESS(&undo_no, sizeof undo_no);
-  MEM_NOACCESS(&last_sql_stat_start, sizeof last_sql_stat_start);
-  MEM_NOACCESS(&rsegs, sizeof rsegs);
-  MEM_NOACCESS(&roll_limit, sizeof roll_limit);
-  MEM_NOACCESS(&in_rollback, sizeof in_rollback);
-  MEM_NOACCESS(&pages_undone, sizeof pages_undone);
-  MEM_NOACCESS(&n_autoinc_rows, sizeof n_autoinc_rows);
-  MEM_NOACCESS(&autoinc_locks, sizeof autoinc_locks);
-  MEM_NOACCESS(&read_only, sizeof read_only);
-  MEM_NOACCESS(&auto_commit, sizeof auto_commit);
-  MEM_NOACCESS(&will_lock, sizeof will_lock);
-  MEM_NOACCESS(&fts_trx, sizeof fts_trx);
-  MEM_NOACCESS(&fts_next_doc_id, sizeof fts_next_doc_id);
-  MEM_NOACCESS(&flush_tables, sizeof flush_tables);
-  MEM_NOACCESS(&ddl, sizeof ddl);
-  MEM_NOACCESS(&internal, sizeof internal);
-#ifdef UNIV_DEBUG
-  MEM_NOACCESS(&start_line, sizeof start_line);
-  MEM_NOACCESS(&start_file, sizeof start_file);
-#endif /* UNIV_DEBUG */
-  MEM_NOACCESS(&xid, sizeof xid);
-  MEM_NOACCESS(&mod_tables, sizeof mod_tables);
-  MEM_NOACCESS(&detailed_error, sizeof detailed_error);
-  MEM_NOACCESS(&flush_observer, sizeof flush_observer);
-#ifdef WITH_WSREP
-  MEM_NOACCESS(&wsrep_event, sizeof wsrep_event);
-  ut_ad(!wsrep_UK_scan);
-  MEM_NOACCESS(&wsrep_UK_scan, sizeof wsrep_UK_scan);
-#endif /* WITH_WSREP */
-  MEM_NOACCESS(&magic_n, sizeof magic_n);
-  trx_pools->mem_free(this);
-}
+		ib::error() << "MySQL is freeing a thd though"
+			" trx->n_mysql_tables_in_use is "
+			<< trx->n_mysql_tables_in_use
+			<< " and trx->mysql_n_tables_locked is "
+			<< trx->mysql_n_tables_locked << ".";
 
-/** Transition to committed state, to release implicit locks. */
-inline void trx_t::commit_state()
-{
-  ut_ad(state == TRX_STATE_PREPARED
-	|| state == TRX_STATE_PREPARED_RECOVERED
-	|| state == TRX_STATE_ACTIVE);
-  /* This makes the transaction committed in memory and makes its
-  changes to data visible to other transactions. NOTE that there is a
-  small discrepancy from the strict formal visibility rules here: a
-  user of the database can see modifications made by another
-  transaction T even before the necessary redo log segment has been
-  flushed to the disk. If the database happens to crash before the
-  flush, the user has seen modifications from T which will never be a
-  committed transaction. However, any transaction T2 which sees the
-  modifications of the committing transaction T, and which also itself
-  makes modifications to the database, will get an lsn larger than the
-  committing transaction T. In the case where the log flush fails, and
-  T never gets committed, also T2 will never get committed. */
-  trx_mutex_enter(this);
-  state= TRX_STATE_COMMITTED_IN_MEMORY;
-  trx_mutex_exit(this);
-  ut_ad(id || !is_referenced());
-}
+		trx_print(stderr, trx, 600);
+		ut_print_buf(stderr, trx, sizeof(trx_t));
+		putc('\n', stderr);
+	}
 
-/** Release any explicit locks of a committing transaction. */
-inline void trx_t::release_locks()
-{
-  DBUG_ASSERT(state == TRX_STATE_COMMITTED_IN_MEMORY);
-  DBUG_ASSERT(!is_referenced());
+	trx->dict_operation = TRX_DICT_OP_NONE;
+	assert_trx_is_inactive(trx);
 
-  if (UT_LIST_GET_LEN(lock.trx_locks))
-  {
-    lock_release(this);
-    lock.n_rec_locks = 0;
-    ut_ad(UT_LIST_GET_LEN(lock.trx_locks) == 0);
-    ut_ad(ib_vector_is_empty(autoinc_locks));
-    mem_heap_empty(lock.lock_heap);
-  }
+	trx_sys.deregister_trx(trx);
 
-  lock.table_locks.clear();
+	assert_trx_is_free(trx);
+
+	trx_sys.rw_trx_hash.put_pins(trx);
+	trx->mysql_thd = 0;
+	trx->mysql_log_file_name = 0;
+
+	// FIXME: We need to avoid this heap free/alloc for each commit.
+	if (trx->autoinc_locks != NULL) {
+		ut_ad(ib_vector_is_empty(trx->autoinc_locks));
+		/* We allocated a dedicated heap for the vector. */
+		ib_vector_free(trx->autoinc_locks);
+		trx->autoinc_locks = NULL;
+	}
+
+	trx->mod_tables.clear();
+
+	/* trx locking state should have been reset before returning trx
+	to pool */
+	ut_ad(trx->will_lock == 0);
+
+	trx_pools->mem_free(trx);
+	/* Unpoison the memory for innodb_monitor_set_option;
+	it is operating also on the freed transaction objects. */
+	MEM_UNDEFINED(&trx->mutex, sizeof trx->mutex);
+	/* Declare the contents as initialized for Valgrind;
+	we checked that it was initialized in trx_pools->mem_free(trx). */
+	UNIV_MEM_VALID(&trx->mutex, sizeof trx->mutex);
+
+	trx = NULL;
 }
 
 /** At shutdown, frees a transaction object. */
@@ -532,26 +477,33 @@ trx_free_at_shutdown(trx_t *trx)
 {
 	ut_ad(trx->is_recovered);
 	ut_a(trx_state_eq(trx, TRX_STATE_PREPARED)
-	     || trx_state_eq(trx, TRX_STATE_PREPARED_RECOVERED)
 	     || (trx_state_eq(trx, TRX_STATE_ACTIVE)
-		 && (!srv_was_started || is_mariabackup_restore_or_export()
+		 && (!srv_was_started
+		     || srv_operation == SRV_OPERATION_RESTORE
+		     || srv_operation == SRV_OPERATION_RESTORE_EXPORT
 		     || srv_read_only_mode
 		     || srv_force_recovery >= SRV_FORCE_NO_TRX_UNDO
 		     || (!srv_is_being_started
 		         && !srv_undo_sources && srv_fast_shutdown))));
 	ut_a(trx->magic_n == TRX_MAGIC_N);
 
-	trx->commit_state();
-	trx->release_locks();
+	lock_trx_release_locks(trx);
 	trx_undo_free_at_shutdown(trx);
 
 	ut_a(!trx->read_only);
 
 	DBUG_LOG("trx", "Free prepared: " << trx);
 	trx->state = TRX_STATE_NOT_STARTED;
-	ut_ad(!UT_LIST_GET_LEN(trx->lock.trx_locks));
-	trx->id = 0;
-	trx->free();
+
+	/* Undo trx_resurrect_table_locks(). */
+	lock_trx_lock_list_init(&trx->lock.trx_locks);
+
+	/* Note: This vector is not guaranteed to be empty because the
+	transaction was never committed and therefore lock_trx_release()
+	was not called. */
+	trx->lock.table_locks.clear();
+
+	trx_free(trx);
 }
 
 
@@ -564,12 +516,10 @@ void trx_disconnect_prepared(trx_t *trx)
   ut_ad(trx_state_eq(trx, TRX_STATE_PREPARED));
   ut_ad(trx->mysql_thd);
   trx->read_view.close();
-  mutex_enter(&trx_sys.mutex);
   trx->is_recovered= true;
   trx->mysql_thd= NULL;
-  mutex_exit(&trx_sys.mutex);
   /* todo/fixme: suggest to do it at innodb prepare */
-  trx->will_lock= false;
+  trx->will_lock= 0;
 }
 
 /****************************************************************//**
@@ -665,8 +615,8 @@ trx_resurrect_table_locks(
 */
 
 static void trx_resurrect(trx_undo_t *undo, trx_rseg_t *rseg,
-                          time_t start_time, ulonglong start_time_micro,
-                          uint64_t *rows_to_undo)
+                          ib_time_t start_time, uint64_t *rows_to_undo,
+                          bool is_old_insert)
 {
   trx_state_t state;
   /*
@@ -689,6 +639,8 @@ static void trx_resurrect(trx_undo_t *undo, trx_rseg_t *rseg,
     state= TRX_STATE_PREPARED;
     break;
   default:
+    if (is_old_insert && srv_force_recovery < SRV_FORCE_NO_TRX_UNDO)
+      trx_undo_commit_cleanup(undo, false);
     return;
   }
 
@@ -698,7 +650,11 @@ static void trx_resurrect(trx_undo_t *undo, trx_rseg_t *rseg,
   ut_d(trx->start_line= __LINE__);
   ut_ad(trx->no == TRX_ID_MAX);
 
-  trx->rsegs.m_redo.undo= undo;
+  if (is_old_insert)
+    trx->rsegs.m_redo.old_insert= undo;
+  else
+    trx->rsegs.m_redo.undo= undo;
+
   trx->undo_no= undo->top_undo_no + 1;
   trx->rsegs.m_redo.rseg= rseg;
   /*
@@ -711,7 +667,6 @@ static void trx_resurrect(trx_undo_t *undo, trx_rseg_t *rseg,
   trx->id= undo->trx_id;
   trx->is_recovered= true;
   trx->start_time= start_time;
-  trx->start_time_micro= start_time_micro;
 
   if (undo->dict_operation)
   {
@@ -729,7 +684,8 @@ static void trx_resurrect(trx_undo_t *undo, trx_rseg_t *rseg,
 
 
 /** Initialize (resurrect) transactions at startup. */
-dberr_t trx_lists_init_at_db_start()
+void
+trx_lists_init_at_db_start()
 {
 	ut_a(srv_is_being_started);
 	ut_ad(!srv_was_started);
@@ -738,23 +694,20 @@ dberr_t trx_lists_init_at_db_start()
 		/* mariabackup --prepare only deals with
 		the redo log and the data files, not with
 		transactions or the data dictionary. */
-		return trx_rseg_array_init();
+		trx_rseg_array_init();
+		return;
 	}
 
 	if (srv_force_recovery >= SRV_FORCE_NO_UNDO_LOG_SCAN) {
-		return DB_SUCCESS;
+		return;
 	}
 
 	purge_sys.create();
-	if (dberr_t err = trx_rseg_array_init()) {
-		ib::info() << "Retry with innodb_force_recovery=5";
-		return err;
-	}
+	trx_rseg_array_init();
 
 	/* Look from the rollback segments if there exist undo logs for
 	transactions. */
-	const time_t	start_time	= time(NULL);
-	const ulonglong	start_time_micro= microsecond_interval_timer();
+	const ib_time_t	start_time	= ut_time();
 	uint64_t	rows_to_undo	= 0;
 
 	for (ulint i = 0; i < TRX_SYS_N_RSEGS; ++i) {
@@ -767,6 +720,17 @@ dberr_t trx_lists_init_at_db_start()
 		if (rseg == NULL) {
 			continue;
 		}
+
+		/* Resurrect transactions that were doing inserts
+		using the old separate insert_undo log. */
+		undo = UT_LIST_GET_FIRST(rseg->old_insert_list);
+		while (undo) {
+			trx_undo_t* next = UT_LIST_GET_NEXT(undo_list, undo);
+			trx_resurrect(undo, rseg, start_time, &rows_to_undo,
+				      true);
+			undo = next;
+		}
+
 		/* Ressurrect other transactions. */
 		for (undo = UT_LIST_GET_FIRST(rseg->undo_list);
 		     undo != NULL;
@@ -774,7 +738,7 @@ dberr_t trx_lists_init_at_db_start()
 			trx_t *trx = trx_sys.find(0, undo->trx_id, false);
 			if (!trx) {
 				trx_resurrect(undo, rseg, start_time,
-					      start_time_micro, &rows_to_undo);
+					      &rows_to_undo, false);
 			} else {
 				ut_ad(trx_state_eq(trx, TRX_STATE_ACTIVE) ||
 				      trx_state_eq(trx, TRX_STATE_PREPARED));
@@ -809,7 +773,6 @@ dberr_t trx_lists_init_at_db_start()
 		ib::info() << "Trx id counter is " << trx_sys.get_max_trx_id();
 	}
 	trx_sys.clone_oldest_view();
-	return DB_SUCCESS;
 }
 
 /** Assign a persistent rollback segment in a round-robin fashion,
@@ -902,27 +865,13 @@ static trx_rseg_t* trx_assign_rseg_low()
 	return(rseg);
 }
 
-/** Set the innodb_log_optimize_ddl page flush observer
-@param[in,out]	space	tablespace
-@param[in,out]	stage	performance_schema accounting */
-void trx_t::set_flush_observer(fil_space_t* space, ut_stage_alter_t* stage)
-{
-	flush_observer = UT_NEW_NOKEY(FlushObserver(space, this, stage));
-}
-
-/** Remove the flush observer */
-void trx_t::remove_flush_observer()
-{
-	UT_DELETE(flush_observer);
-	flush_observer = NULL;
-}
-
 /** Assign a rollback segment for modifying temporary tables.
 @return the assigned rollback segment */
-trx_rseg_t *trx_t::assign_temp_rseg()
+trx_rseg_t*
+trx_t::assign_temp_rseg()
 {
 	ut_ad(!rsegs.m_noredo.rseg);
-	ut_ad(!is_autocommit_non_locking());
+	ut_ad(!trx_is_autocommit_non_locking(this));
 	compile_time_assert(ut_is_2pow(TRX_SYS_N_RSEGS));
 
 	/* Choose a temporary rollback segment between 0 and 127
@@ -972,8 +921,8 @@ trx_start_low(
 		    && thd_trx_is_read_only(trx->mysql_thd));
 
 	if (!trx->auto_commit) {
-		trx->will_lock = true;
-	} else if (!trx->will_lock) {
+		++trx->will_lock;
+	} else if (trx->will_lock == 0) {
 		trx->read_only = true;
 	}
 
@@ -1016,7 +965,7 @@ trx_start_low(
 
 		trx_sys.register_rw(trx);
 	} else {
-		if (!trx->is_autocommit_non_locking()) {
+		if (!trx_is_autocommit_non_locking(trx)) {
 
 			/* If this is a read-only transaction that is writing
 			to a temporary table then it needs a transaction id
@@ -1031,10 +980,14 @@ trx_start_low(
 		}
 	}
 
-	trx->start_time = time(NULL);
-	trx->start_time_micro = trx->mysql_thd
-		? thd_query_start_micro(trx->mysql_thd)
-		: microsecond_interval_timer();
+	if (trx->mysql_thd != NULL) {
+		trx->start_time = thd_start_time_in_secs(trx->mysql_thd);
+		trx->start_time_micro = thd_query_start_micro(trx->mysql_thd);
+
+	} else {
+		trx->start_time = ut_time();
+		trx->start_time_micro = 0;
+	}
 
 	ut_a(trx->error_state == DB_SUCCESS);
 
@@ -1105,22 +1058,30 @@ trx_write_serialisation_history(
 	trx_rseg_t*	rseg = trx->rsegs.m_redo.rseg;
 	if (!rseg) {
 		ut_ad(!trx->rsegs.m_redo.undo);
+		ut_ad(!trx->rsegs.m_redo.old_insert);
 		return;
 	}
 
 	trx_undo_t*& undo = trx->rsegs.m_redo.undo;
+	trx_undo_t*& old_insert = trx->rsegs.m_redo.old_insert;
 
-	if (!undo) {
+	if (!undo && !old_insert) {
 		return;
 	}
 
 	ut_ad(!trx->read_only);
 	ut_ad(!undo || undo->rseg == rseg);
+	ut_ad(!old_insert || old_insert->rseg == rseg);
 	mutex_enter(&rseg->mutex);
 
 	/* Assign the transaction serialisation number and add any
 	undo log to the purge queue. */
 	trx_serialise(trx);
+
+	if (UNIV_LIKELY_NULL(old_insert)) {
+		UT_LIST_REMOVE(rseg->old_insert_list, old_insert);
+		trx_purge_add_undo_to_history(trx, old_insert, mtr);
+	}
 	if (undo) {
 		UT_LIST_REMOVE(rseg->undo_list, undo);
 		trx_purge_add_undo_to_history(trx, undo, mtr);
@@ -1144,14 +1105,26 @@ trx_finalize_for_fts_table(
 	fts_t*		  fts = ftt->table->fts;
 	fts_doc_ids_t*	  doc_ids = ftt->added_doc_ids;
 
-	ut_a(fts->add_wq);
+	mutex_enter(&fts->bg_threads_mutex);
 
-	mem_heap_t* heap = static_cast<mem_heap_t*>(doc_ids->self_heap->arg);
+	if (fts->fts_status & BG_THREAD_STOP) {
+		/* The table is about to be dropped, no use
+		adding anything to its work queue. */
 
-	ib_wqueue_add(fts->add_wq, doc_ids, heap);
+		mutex_exit(&fts->bg_threads_mutex);
+	} else {
+		mem_heap_t*     heap;
+		mutex_exit(&fts->bg_threads_mutex);
 
-	/* fts_trx_table_t no longer owns the list. */
-	ftt->added_doc_ids = NULL;
+		ut_a(fts->add_wq);
+
+		heap = static_cast<mem_heap_t*>(doc_ids->self_heap->arg);
+
+		ib_wqueue_add(fts->add_wq, doc_ids, heap);
+
+		/* fts_trx_table_t no longer owns the list. */
+		ftt->added_doc_ids = NULL;
+	}
 }
 
 /******************************************************************//**
@@ -1204,12 +1177,12 @@ trx_flush_log_if_needed_low(
 	bool	flush = srv_file_flush_method != SRV_NOSYNC;
 
 	switch (srv_flush_log_at_trx_commit) {
+	case 3:
 	case 2:
 		/* Write the log but do not flush it to disk */
 		flush = false;
 		/* fall through */
 	case 1:
-	case 3:
 		/* Write the log and optionally flush it to disk */
 		log_write_up_to(lsn, flush);
 		return;
@@ -1249,7 +1222,7 @@ trx_update_mod_tables_timestamp(
 {
 	/* consider using trx->start_time if calling time() is too
 	expensive here */
-	const time_t now = time(NULL);
+	time_t	now = ut_time();
 
 	trx_mod_tables_t::const_iterator	end = trx->mod_tables.end();
 
@@ -1265,8 +1238,7 @@ trx_update_mod_tables_timestamp(
 		"garbage" in table->update_time is justified because
 		protecting it with a latch here would be too performance
 		intrusive. */
-		dict_table_t* table = it->first;
-		table->update_time = now;
+		it->first->update_time = now;
 	}
 
 	trx->mod_tables.clear();
@@ -1286,19 +1258,16 @@ trx_commit_in_memory(
 	trx->must_flush_log_later = false;
 	trx->read_view.close();
 
-	if (trx->is_autocommit_non_locking()) {
+	if (trx_is_autocommit_non_locking(trx)) {
 		ut_ad(trx->id == 0);
 		ut_ad(trx->read_only);
-		ut_ad(!trx->will_lock);
 		ut_a(!trx->is_recovered);
 		ut_ad(trx->rsegs.m_redo.rseg == NULL);
-		ut_ad(trx->mysql_thd);
-		ut_ad(trx->state == TRX_STATE_ACTIVE);
 
 		/* Note: We are asserting without holding the lock mutex. But
 		that is OK because this transaction is not waiting and cannot
-		be rolled back and no new locks can (or should) be added
-		because it is flagged as a non-locking read-only transaction. */
+		be rolled back and no new locks can (or should not) be added
+		becuase it is flagged as a non-locking read-only transaction. */
 
 		ut_a(UT_LIST_GET_LEN(trx->lock.trx_locks) == 0);
 
@@ -1309,55 +1278,60 @@ trx_commit_in_memory(
 		and it cannot be removed from the trx_list and freed
 		without first acquiring the trx_sys_t::mutex. */
 
+		ut_ad(trx_state_eq(trx, TRX_STATE_ACTIVE));
+
 		MONITOR_INC(MONITOR_TRX_NL_RO_COMMIT);
 
 		DBUG_LOG("trx", "Autocommit in memory: " << trx);
 		trx->state = TRX_STATE_NOT_STARTED;
 	} else {
-#ifdef UNIV_DEBUG
-		if (!UT_LIST_GET_LEN(trx->lock.trx_locks)) {
-			for (lock_list::iterator it
-				= trx->lock.table_locks.begin();
-			     it != trx->lock.table_locks.end();
-			     it++) {
-				ut_ad(!*it);
-			}
-		}
-#endif /* UNIV_DEBUG */
-		trx->commit_state();
-
-		if (trx->id) {
+		if (trx->id > 0) {
+			/* For consistent snapshot, we need to remove current
+			transaction from rw_trx_hash before doing commit and
+			releasing locks. */
 			trx_sys.deregister_rw(trx);
-
-			/* Wait for any implicit-to-explicit lock
-			conversions to cease, so that there will be no
-			race condition in lock_release(). */
-			while (UNIV_UNLIKELY(trx->is_referenced())) {
-				ut_delay(srv_spin_wait_delay);
-			}
-		} else {
-			ut_ad(trx->read_only || !trx->rsegs.m_redo.rseg);
 		}
 
-		if (trx->read_only || !trx->rsegs.m_redo.rseg) {
+		/* trx->id will be cleared in lock_trx_release_locks(trx). */
+		ut_ad(trx->read_only || !trx->rsegs.m_redo.rseg || trx->id);
+		lock_trx_release_locks(trx);
+		ut_ad(trx->id == 0);
+
+		/* Remove the transaction from the list of active
+		transactions now that it no longer holds any user locks. */
+
+		ut_ad(trx_state_eq(trx, TRX_STATE_COMMITTED_IN_MEMORY));
+		DEBUG_SYNC_C("after_trx_committed_in_memory");
+
+		if (trx->read_only || trx->rsegs.m_redo.rseg == NULL) {
 			MONITOR_INC(MONITOR_TRX_RO_COMMIT);
 		} else {
 			trx_update_mod_tables_timestamp(trx);
 			MONITOR_INC(MONITOR_TRX_RW_COMMIT);
-			trx->is_recovered = false;
 		}
-
-		trx->release_locks();
-		trx->id = 0;
-		DEBUG_SYNC_C("after_trx_committed_in_memory");
 	}
 
 	ut_ad(!trx->rsegs.m_redo.undo);
 
+	if (trx_rseg_t*	rseg = trx->rsegs.m_redo.rseg) {
+		mutex_enter(&rseg->mutex);
+		ut_ad(rseg->trx_ref_count > 0);
+		--rseg->trx_ref_count;
+		mutex_exit(&rseg->mutex);
+
+		if (trx_undo_t*& insert = trx->rsegs.m_redo.old_insert) {
+			ut_ad(insert->rseg == rseg);
+			trx_undo_commit_cleanup(insert, false);
+			insert = NULL;
+		}
+	}
+
+	ut_ad(!trx->rsegs.m_redo.old_insert);
+
 	if (mtr != NULL) {
 		if (trx_undo_t*& undo = trx->rsegs.m_noredo.undo) {
 			ut_ad(undo->rseg == trx->rsegs.m_noredo.rseg);
-			trx_undo_commit_cleanup(undo);
+			trx_undo_commit_cleanup(undo, true);
 			undo = NULL;
 		}
 
@@ -1413,15 +1387,6 @@ trx_commit_in_memory(
 
 	ut_ad(!trx->rsegs.m_noredo.undo);
 
-	/* Only after trx_undo_commit_cleanup() it is safe to release
-	our rseg reference. */
-	if (trx_rseg_t* rseg = trx->rsegs.m_redo.rseg) {
-		mutex_enter(&rseg->mutex);
-		ut_ad(rseg->trx_ref_count > 0);
-		--rseg->trx_ref_count;
-		mutex_exit(&rseg->mutex);
-	}
-
 	/* Free all savepoints, starting from the first. */
 	trx_named_savept_t*	savep = UT_LIST_GET_FIRST(trx->trx_savepoints);
 
@@ -1433,15 +1398,17 @@ trx_commit_in_memory(
 
 	trx_mutex_enter(trx);
 	trx->dict_operation = TRX_DICT_OP_NONE;
-	trx->lock.was_chosen_as_deadlock_victim = false;
+
+#ifdef WITH_WSREP
+	if (trx->mysql_thd && wsrep_on(trx->mysql_thd)) {
+		trx->lock.was_chosen_as_deadlock_victim = FALSE;
+	}
+#endif
 
 	DBUG_LOG("trx", "Commit in memory: " << trx);
 	trx->state = TRX_STATE_NOT_STARTED;
-#ifdef WITH_WSREP
-	trx->wsrep = false;
-#endif
 
-	trx->assert_freed();
+	assert_trx_is_free(trx);
 
 	trx_init(trx);
 
@@ -1456,23 +1423,33 @@ trx_commit_in_memory(
 @param[in,out]	mtr	mini-transaction (NULL if no modifications) */
 void trx_commit_low(trx_t* trx, mtr_t* mtr)
 {
+	assert_trx_nonlocking_or_in_list(trx);
+	ut_ad(!trx_state_eq(trx, TRX_STATE_COMMITTED_IN_MEMORY));
 	ut_ad(!mtr || mtr->is_active());
 	ut_d(bool aborted = trx->in_rollback
 	     && trx->error_state == DB_DEADLOCK);
-	ut_ad(!mtr == (aborted || !trx->has_logged()));
+	ut_ad(!mtr == (aborted || !trx->has_logged_or_recovered()));
 	ut_ad(!mtr || !aborted);
 
 	/* undo_no is non-zero if we're doing the final commit. */
 	if (trx->fts_trx != NULL && trx->undo_no != 0) {
-		ut_a(!trx->is_autocommit_non_locking());
+		dberr_t	error;
+
+		ut_a(!trx_is_autocommit_non_locking(trx));
+
+		error = fts_commit(trx);
 
 		/* FTS-FIXME: Temporarily tolerate DB_DUPLICATE_KEY
 		instead of dying. This is a possible scenario if there
 		is a crash between insert to DELETED table committing
 		and transaction committing. The fix would be able to
 		return error from this function */
-		if (dberr_t error = fts_commit(trx)) {
-			ut_a(error == DB_DUPLICATE_KEY);
+		if (error != DB_SUCCESS && error != DB_DUPLICATE_KEY) {
+			/* FTS-FIXME: once we can return values from this
+			function, we should do so and signal an error
+			instead of just dying. */
+
+			ut_error;
 		}
 	}
 
@@ -1505,8 +1482,7 @@ void trx_commit_low(trx_t* trx, mtr_t* mtr)
 
 		DBUG_EXECUTE_IF("ib_crash_during_trx_commit_in_mem",
 				if (trx->has_logged()) {
-					log_write_up_to(mtr->commit_lsn(),
-							true);
+					log_make_checkpoint_at(LSN_MAX, TRUE);
 					DBUG_SUICIDE();
 				});
 		/*--------------*/
@@ -1538,7 +1514,10 @@ trx_commit(
 	mtr_t*	mtr;
 	mtr_t	local_mtr;
 
-	if (trx->has_logged()) {
+	DBUG_EXECUTE_IF("ib_trx_commit_crash_before_trx_commit_start",
+			DBUG_SUICIDE(););
+
+	if (trx->has_logged_or_recovered()) {
 		mtr = &local_mtr;
 		mtr->start();
 	} else {
@@ -1568,7 +1547,7 @@ trx_commit_or_rollback_prepare(
 
 	case TRX_STATE_ACTIVE:
 	case TRX_STATE_PREPARED:
-	case TRX_STATE_PREPARED_RECOVERED:
+
 		/* If the trx is in a lock wait state, moves the waiting
 		query thread to the suspended state */
 
@@ -1679,7 +1658,7 @@ trx_commit_for_mysql(
 		/* fall through */
 	case TRX_STATE_ACTIVE:
 	case TRX_STATE_PREPARED:
-	case TRX_STATE_PREPARED_RECOVERED:
+
 		trx->op_info = "committing";
 
 		trx_commit(trx);
@@ -1725,7 +1704,6 @@ trx_mark_sql_stat_end(
 
 	switch (trx->state) {
 	case TRX_STATE_PREPARED:
-	case TRX_STATE_PREPARED_RECOVERED:
 	case TRX_STATE_COMMITTED_IN_MEMORY:
 		break;
 	case TRX_STATE_NOT_STARTED:
@@ -1764,6 +1742,7 @@ trx_print_low(
 			/*!< in: mem_heap_get_size(trx->lock.lock_heap) */
 {
 	ibool		newline;
+	const char*	op_info;
 
 	fprintf(f, "TRANSACTION " TRX_ID_FMT, trx_get_id_for_print(trx));
 
@@ -1779,7 +1758,6 @@ trx_print_low(
 			(ulong) difftime(time(NULL), trx->start_time));
 		goto state_ok;
 	case TRX_STATE_PREPARED:
-	case TRX_STATE_PREPARED_RECOVERED:
 		fprintf(f, ", ACTIVE (PREPARED) %lu sec",
 			(ulong) difftime(time(NULL), trx->start_time));
 		goto state_ok;
@@ -1790,7 +1768,9 @@ trx_print_low(
 	fprintf(f, ", state %lu", (ulong) trx->state);
 	ut_ad(0);
 state_ok:
-	const char* op_info = trx->op_info;
+
+	/* prevent a race condition */
+	op_info = trx->op_info;
 
 	if (*op_info) {
 		putc(' ', f);
@@ -1941,8 +1921,11 @@ trx_weight_ge(
 /** Prepare a transaction.
 @return	log sequence number that makes the XA PREPARE durable
 @retval	0	if no changes needed to be made durable */
-static lsn_t trx_prepare_low(trx_t *trx)
+static
+lsn_t
+trx_prepare_low(trx_t* trx)
 {
+	ut_ad(!trx->rsegs.m_redo.old_insert);
 	ut_ad(!trx->is_recovered);
 
 	mtr_t	mtr;
@@ -2054,7 +2037,6 @@ struct trx_recover_for_mysql_callback_arg
 static my_bool trx_recover_for_mysql_callback(rw_trx_hash_element_t *element,
   trx_recover_for_mysql_callback_arg *arg)
 {
-  DBUG_ASSERT(arg->len > 0);
   mutex_enter(&element->mutex);
   if (trx_t *trx= element->trx)
   {
@@ -2066,38 +2048,17 @@ static my_bool trx_recover_for_mysql_callback(rw_trx_hash_element_t *element,
     if (trx_state_eq(trx, TRX_STATE_PREPARED))
     {
       ut_ad(trx->is_recovered);
-      ut_ad(trx->id);
       if (arg->count == 0)
         ib::info() << "Starting recovery for XA transactions...";
-      XID& xid= arg->xid_list[arg->count];
-      if (arg->count++ < arg->len)
-      {
-        trx->state= TRX_STATE_PREPARED_RECOVERED;
-        ib::info() << "Transaction " << trx->id
-                   << " in prepared state after recovery";
-        ib::info() << "Transaction contains changes to " << trx->undo_no
-                   << " rows";
-        xid= *trx->xid;
-      }
+      ib::info() << "Transaction " << trx_get_id_for_print(trx)
+                 << " in prepared state after recovery";
+      ib::info() << "Transaction contains changes to " << trx->undo_no
+                 << " rows";
+      arg->xid_list[arg->count++]= *trx->xid;
     }
   }
   mutex_exit(&element->mutex);
-  /* Do not terminate upon reaching arg->len; count all transactions */
-  return false;
-}
-
-
-static my_bool trx_recover_reset_callback(rw_trx_hash_element_t *element,
-  void*)
-{
-  mutex_enter(&element->mutex);
-  if (trx_t *trx= element->trx)
-  {
-    if (trx_state_eq(trx, TRX_STATE_PREPARED_RECOVERED))
-      trx->state= TRX_STATE_PREPARED;
-  }
-  mutex_exit(&element->mutex);
-  return false;
+  return arg->count == arg->len;
 }
 
 
@@ -2121,24 +2082,15 @@ int trx_recover_for_mysql(XID *xid_list, uint len)
   trx_sys.rw_trx_hash.iterate_no_dups(reinterpret_cast<my_hash_walk_action>
                                       (trx_recover_for_mysql_callback), &arg);
   if (arg.count)
-  {
     ib::info() << arg.count
-        << " transactions in prepared state after recovery";
-    /* After returning the full list, reset the state, because
-    init_server_components() wants to recover the collection of
-    transactions twice, by first calling tc_log->open() and then
-    ha_recover() directly. */
-    if (arg.count <= len)
-      trx_sys.rw_trx_hash.iterate(reinterpret_cast<my_hash_walk_action>
-                                  (trx_recover_reset_callback), NULL);
-  }
-  return int(std::min(arg.count, len));
+               << " transactions in prepared state after recovery";
+  return int(arg.count);
 }
 
 
 struct trx_get_trx_by_xid_callback_arg
 {
-  const XID *xid;
+  XID *xid;
   trx_t *trx;
 };
 
@@ -2150,36 +2102,31 @@ static my_bool trx_get_trx_by_xid_callback(rw_trx_hash_element_t *element,
   mutex_enter(&element->mutex);
   if (trx_t *trx= element->trx)
   {
-    trx_mutex_enter(trx);
-    if (trx->is_recovered &&
-	(trx_state_eq(trx, TRX_STATE_PREPARED) ||
-	 trx_state_eq(trx, TRX_STATE_PREPARED_RECOVERED)) &&
+    if (trx->is_recovered && trx_state_eq(trx, TRX_STATE_PREPARED) &&
         arg->xid->eq(reinterpret_cast<XID*>(trx->xid)))
     {
-#ifdef WITH_WSREP
-      /* The commit of a prepared recovered Galera
-      transaction needs a valid trx->xid for
-      invoking trx_sys_update_wsrep_checkpoint(). */
-      if (!wsrep_is_wsrep_xid(trx->xid))
-#endif
       /* Invalidate the XID, so that subsequent calls will not find it. */
       trx->xid->null();
       arg->trx= trx;
       found= 1;
     }
-    trx_mutex_exit(trx);
   }
   mutex_exit(&element->mutex);
   return found;
 }
 
-/** Look up an X/Open distributed transaction in XA PREPARE state.
-@param[in]	xid	X/Open XA transaction identifier
-@return	transaction on match (the trx_t::xid will be invalidated);
-note that the trx may have been committed before the caller acquires
-trx_t::mutex
-@retval	NULL if no match */
-trx_t* trx_get_trx_by_xid(const XID* xid)
+
+/**
+  Finds PREPARED XA transaction by xid.
+
+  trx may have been committed, unless the caller is holding lock_sys.mutex.
+
+  @param[in]  xid  X/Open XA transaction identifier
+
+  @return trx or NULL; on match, the trx->xid will be invalidated;
+*/
+
+trx_t *trx_get_trx_by_xid(XID *xid)
 {
   trx_get_trx_by_xid_callback_arg arg= { xid, 0 };
 
@@ -2215,7 +2162,6 @@ trx_start_if_not_started_xa_low(
 		}
 		return;
 	case TRX_STATE_PREPARED:
-	case TRX_STATE_PREPARED_RECOVERED:
 	case TRX_STATE_COMMITTED_IN_MEMORY:
 		break;
 	}
@@ -2243,7 +2189,6 @@ trx_start_if_not_started_low(
 		return;
 
 	case TRX_STATE_PREPARED:
-	case TRX_STATE_PREPARED_RECOVERED:
 	case TRX_STATE_COMMITTED_IN_MEMORY:
 		break;
 	}
@@ -2261,7 +2206,7 @@ trx_start_internal_low(
 	/* Ensure it is not flagged as an auto-commit-non-locking
 	transaction. */
 
-	trx->will_lock = true;
+	trx->will_lock = 1;
 
 	trx->internal = true;
 
@@ -2277,7 +2222,7 @@ trx_start_internal_read_only_low(
 	/* Ensure it is not flagged as an auto-commit-non-locking
 	transaction. */
 
-	trx->will_lock = true;
+	trx->will_lock = 1;
 
 	trx->internal = true;
 
@@ -2298,13 +2243,18 @@ trx_start_for_ddl_low(
 		the data dictionary will be locked in crash recovery. */
 
 		trx_set_dict_operation(trx, op);
+
+		/* Ensure it is not flagged as an auto-commit-non-locking
+		transation. */
+		trx->will_lock = 1;
+
 		trx->ddl= true;
+
 		trx_start_internal_low(trx);
 		return;
 
 	case TRX_STATE_ACTIVE:
 	case TRX_STATE_PREPARED:
-	case TRX_STATE_PREPARED_RECOVERED:
 	case TRX_STATE_COMMITTED_IN_MEMORY:
 		break;
 	}
@@ -2325,7 +2275,7 @@ trx_set_rw_mode(
 	trx_t*		trx)		/*!< in/out: transaction that is RW */
 {
 	ut_ad(trx->rsegs.m_redo.rseg == 0);
-	ut_ad(!trx->is_autocommit_non_locking());
+	ut_ad(!trx_is_autocommit_non_locking(trx));
 	ut_ad(!trx->read_only);
 	ut_ad(trx->id == 0);
 
@@ -2350,16 +2300,3 @@ trx_set_rw_mode(
 		trx->read_view.set_creator_trx_id(trx->id);
 	}
 }
-
-bool trx_t::has_stats_table_lock() const
-{
-  for (lock_list::const_iterator it= lock.table_locks.begin(),
-       end= lock.table_locks.end(); it != end; ++it)
-  {
-     const lock_t *lock= *it;
-     if (lock && lock->un_member.tab_lock.table->is_stats_table())
-       return true;
-  }
-
-  return false;
-}
