@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include <my_global.h>
 #include <my_sys.h>
@@ -34,7 +34,7 @@ void test_concurrently(const char *test, pthread_handler handler, int n, int m)
   bad= 0;
 
   diag("Testing %s with %d threads, %d iterations... ", test, n, m);
-  for (i= 0; i < n; i++)
+  for (i= n; i; i--)
   {
     if (pthread_create(&threads[i], 0, handler, &m) != 0)
     {
@@ -43,7 +43,7 @@ void test_concurrently(const char *test, pthread_handler handler, int n, int m)
     }
   }
 
-  for (i= 0; i < n; i++)
+  for (i= n; i; i--)
     pthread_join(threads[i], 0);
 
   now= my_interval_timer() - now;
